<?php
function woo_pd_cron_import( $filename = null, $gui = 0 ) {

	global $import, $product;

	$ip_address = woo_pd_get_visitor_ip_address();

	if( $filename ) {
		$upload_dir = wp_upload_dir();
		$alt_path = '/';
		if( isset( $_GET['file_path'] ) )
			$alt_path .= $_GET['file_path'] . '/';
		if( file_exists( $upload_dir['basedir'] . $alt_path . $filename ) ) {

			// Ready to begin

			include_once( ABSPATH . 'wp-admin/includes/admin.php' );

			$import = new stdClass;
			$import->cron = true;
			$import->cancel_import = false;
			$import->advanced_log = ( isset( $_GET['advanced_log'] ) ? absint( $_GET['advanced_log'] ) : 0 );
			$import->delimiter = woo_pd_get_option( 'delimiter', ',' );
			if( $import->delimiter == 'TAB' )
				$import->delimiter = "\t";
			else if( $import->delimiter == '' || $import->delimiter == false )
				$import->delimiter = ',';
			$import->category_separator = woo_pd_get_option( 'category_separator', '|' );
			$import->parent_child_delimiter = woo_pd_get_option( 'parent_child_separator', '>' );
			$import->special_category_assignment = ( isset( $_GET['special_category_assignment'] ) ? absint( $_GET['special_category_assignment'] ) : 0 );
			$import->force_refresh_images = ( isset( $_GET['force_refresh_images'] ) ? absint( $_GET['force_refresh_images'] ) : 0 );
			$import->import_method = ( isset( $_GET['import_method'] ) ? $_GET['import_method'] : 'new' );
			$import->encoding = woo_pd_get_option( 'encoding', 'UTF-8' );
			// Legacy support for $_GET merge
			if( isset( $_GET['merge'] ) ) {
				if( absint( $_GET['merge'] ) == 1 )
					$import->import_method = 'update';
			}
			$import->log = __( 'Preparing data...', 'woocommerce-product-importer' );

			$import->log .= "<br />" . sprintf( __( 'Attempting to upload: %s', 'woocommerce-product-importer' ), $upload_dir['basedir'] . $alt_path . $filename );

			woo_pd_update_option( 'csv', $upload_dir['basedir'] . '/' . $filename );
			$file = array(
				'name' => $filename,
				'type' => 'application/vnd.ms-excel',
				'tmp_name' => $upload_dir['basedir'] . $alt_path . $filename,
				'error' => 0,
				'size' => @filesize( $upload_dir['basedir'] . $alt_path . $filename )
			);
			if( $import->advanced_log )
				$import->log .= '<br />' . sprintf( __( 'Detected import method: %s...', 'woocommerce-product-importer' ), $import->import_method );
			woo_pd_prepare_data();
			$import->skip_first = 1;
			$import->default_weight_unit = get_option( 'woocommerce_weight_unit', 'kg' );
			$import->default_measurement_unit = get_option( 'woocommerce_dimension_unit', 'cm' );
			$import->options = woo_pd_product_fields();
			$import->options_size = count( $import->options );
			$first_row = array();
			if( isset( $import->lines ) ) {
				$first_row = $import->lines[0];
			}
			$import->columns = count( $first_row );
			// If we only detect a single column then the delimiter may be wrong
			if( $import->columns == 1 ) {
				$original_delimiter = $import->delimiter;
				$auto_delimiter = woo_pd_detect_file_delimiter( (string)$first_row[0] );
				if( $import->delimiter <> $auto_delimiter ) {
					$import->delimiter = $auto_delimiter;
					// Check for corrupted first line
					if( is_array( $import->lines[0] ) && count( $import->lines[0] ) == 1 ) {
						$import->lines[0] = $import->lines[0][0];
					}
					$first_row = str_getcsv( $import->lines[0], $import->delimiter );
					$import->columns = count( $first_row );
					// If the column count is unchanged then the CSV either has only a single column (which won't work) or we've failed our job
					if( $import->columns > 1 ) {
						$import->lines[1] = implode( $original_delimiter, $import->lines[1] );
						$import->lines[1] = explode( $auto_delimiter, $import->lines[1] );
						$import->log .= '<br />' . sprintf( __( 'It seems the field delimiter provided under Import Options - <code>%s</code> - didn\'t match this CSV so we automatically detected the CSV delimiter for you to <code>%s</code>.', 'woocommerce-product-importer' ), woo_pd_get_option( 'delimiter', ',' ), ( $auto_delimiter == "\t" ? 'TAB' : $auto_delimiter ) );
						woo_pd_update_option( 'delimiter', $import->delimiter );
						$import->log .= '<br />' . __( 'Running woo_pd_prepare_data() again with new field delimiter...', 'woocommerce-product-importer' );
						woo_pd_prepare_data();
					}
				}
				unset( $auto_delimiter, $original_delimiter );
			}
			$value_data = array();
			foreach( $first_row as $key => $cell ) {
				for( $k = 0; $k < $import->options_size; $k++ ) {
					if( woo_pd_search_column( $import->options[$k], woo_pd_format_column( $cell ) ) == woo_pd_format_column( $cell ) ) {
						$value_data[$key] = $import->options[$k]['name'];
						break;
					}
				}
			}
			woo_pd_detect_image_method();
			if( woo_pd_prepare_columns( $value_data ) ) {
				$import->log .= "<br />" . __( 'Product columns have been grouped', 'woocommerce-product-importer' );
				$import->products_added = 0;
				$import->variations_added = 0;
				$import->products_deleted = 0;
				$import->products_failed = 0;
				$import->products_updated = 0;
				$import->log .= "<br /><br />" . __( 'Generating Categories...', 'woocommerce-product-importer' );
				if( isset( $import->csv_category ) )
					woo_pd_generate_categories();
				else
					$import->log .= "<br />" . __( 'Categories skipped', 'woocommerce-product-importer' );
				$import->log .= "<br /><br />" . __( 'Generating Tags...', 'woocommerce-product-importer' );
				if( isset( $import->csv_tag ) )
					woo_pd_generate_tags();
				else
					$import->log .= "<br />" . __( 'Tags skipped', 'woocommerce-product-importer' );
				if( $import->import_method == 'update' )
					$import->log .= "<br /><br />" . __( 'Merging Product changes...', 'woocommerce-product-importer' );
				else if( $import->import_method == 'merge' )
					$import->log .= "<br /><br />" . __( 'Generating Products and merging Product changes...', 'woocommerce-product-importer' );
				else if( $import->import_method == 'new' )
					$import->log .= "<br /><br />" . __( 'Generating Products...', 'woocommerce-product-importer' );
				else if( $import->import_method == 'delete' )
					$import->log .= "<br /><br />" . __( 'Checking for matching Products to delete...', 'woocommerce-product-importer' );

				for( $i = 1; $i < $import->rows; $i++ ) {

					woo_pd_prepare_product( $i );

					if( in_array( $import->import_method, array( 'update', 'merge' ) ) && $product->duplicate_exists ) {
						if( !empty( $product->name ) )
							$import->log .= "<br />>>> " . sprintf( __( 'Merging %s...', 'woocommerce-product-importer' ), $product->name );
						else
							$import->log .= "<br />>>> " . sprintf( __( 'Merging (no title) - SKU: %s...', 'woocommerce-product-importer' ), $product->sku );
					} else if( $import->import_method == 'delete' ) {
						if( !empty( $product->name ) )
							$import->log .= "<br />>>> " . sprintf( __( 'Searching for %s...', 'woocommerce-product-importer' ), $product->name );
						else
							$import->log .= "<br />>>> " . sprintf( __( 'Searching for (no title) - SKU: %s...', 'woocommerce-product-importer' ), $product->sku );
					} else {
						if( !empty( $product->name ) )
							$import->log .= "<br />>>> " . sprintf( __( 'Importing %s...', 'woocommerce-product-importer' ), $product->name );
						else
							$import->log .= "<br />>>> " . sprintf( __( 'Importing (no title) - SKU: %s...', 'woocommerce-product-importer' ), $product->sku );
					}

					if( in_array( $import->import_method, array( 'new', 'merge', 'update' ) ) ) {
						// Build Categories
						if( isset( $product->category ) )
							woo_pd_process_categories();

						// Build Tags
						if( isset( $product->tag ) )
							woo_pd_process_tags();
					}

					// Check for duplicate SKU
					woo_pd_duplicate_product_exists();

					woo_pd_validate_product();

					if( $product->fail_requirements ) {

						$import->log .= "<br />>>>>>> " . sprintf( __( 'Skipping Product, reason: %s', 'woocommerce-product-importer' ), $product->failed_reason );
						$import->products_failed++;

					} else {

						if( in_array( $import->import_method, array( 'merge', 'update' ) ) && $product->duplicate_exists )
							woo_pd_merge_product();
						else if( $import->import_method == 'delete' && $product->duplicate_exists )
							woo_pd_delete_product();
						else if( in_array( $import->import_method, array( 'new', 'merge' ) ) )
							woo_pd_create_product();

						// Imported Product successfully
						if( in_array( $import->import_method, array( 'update', 'merge' ) ) && $product->duplicate_exists ) {
							if( $product->updated ) {
								if( !empty( $product->name ) )
									$import->log .= "<br />>>>>>> " . sprintf( __( '%s successfully updated', 'woocommerce-product-importer' ), $product->name );
								else
									$import->log .= "<br />>>>>>> " . sprintf( __( '(no title) - SKU: %s successfully updated', 'woocommerce-product-importer' ), $product->sku );
							} else {
								if( !empty( $product->name ) )
									$import->log .= "<br />>>>>>> " . sprintf( __( 'No changes were made to %s', 'woocommerce-product-importer' ), $product->name );
								else
									$import->log .= "<br />>>>>>> " . sprintf( __( 'No changes were made to (no title) - SKU: %s', 'woocommerce-product-importer' ), $product->sku );
							}
						} else if( $import->import_method == 'delete' ) {
							if( $product->deleted ) {
								if( !empty( $product->name ) )
									$import->log .= "<br />>>>>>> " . sprintf( __( '%s successfully deleted', 'woocommerce-product-importer' ), $product->name );
								else
									$import->log .= "<br />>>>>>> " . sprintf( __( '(no title) - SKU: %s successfully deleted', 'woocommerce-product-importer' ), $product->sku );
							} else {
								if( $import->advanced_log )
									$import->log .= "<br />>>>>>> " . sprintf( __( 'Skipping Product, see Import Report for full explanation. Reason: %s', 'woocommerce-product-importer' ), $product->failed_reason );
								else
									$import->log .= "<br />>>>>>> " . sprintf( __( 'Skipping Product, reason: %s', 'woocommerce-product-importer' ), $product->failed_reason );
							}
						} else {
							if( $product->imported ) {
								if( $import->import_method == 'new' ) {
									if( !empty( $product->name ) )
										$import->log .= "<br />>>>>>> " . sprintf( __( '%s successfully imported', 'woocommerce-product-importer' ), $product->name );
									else
										$import->log .= "<br />>>>>>> " . sprintf( __( '(no title) - SKU: %s successfully imported', 'woocommerce-product-importer' ), $product->sku );
								}
							} else {
								if( $import->advanced_log )
									$import->log .= "<br />>>>>>> " . sprintf( __( 'Skipping Product, see Import Report for full explanation. Reason: %s', 'woocommerce-product-importer' ), $product->failed_reason );
								else
									$import->log .= "<br />>>>>>> " . sprintf( __( 'Skipping Product, reason: %s', 'woocommerce-product-importer' ), $product->failed_reason );
							}
						}

					}
				}
				if( $import->import_method == 'update' )
					$import->log .= "<br />" . __( 'Product changes have been merged', 'woocommerce-product-importer' );
				else if( $import->import_method == 'merge' )
					$import->log .= "<br />" . __( 'Products have been generated and Product changes merged', 'woocommerce-product-importer' );
				else if( $import->import_method == 'new' )
					$import->log .= "<br />" . __( 'Products have been generated', 'woocommerce-product-importer' );
				else if( $import->import_method == 'delete' )
					$import->log .= "<br />" . __( 'Products have been deleted', 'woocommerce-product-importer' );
				$import->log .= "<br /><br />" . __( 'Cleaning up...', 'woocommerce-product-importer' );
				$import->log .= "<br />" . __( 'Clean up has completed', 'woocommerce-product-importer' );

				// Post-import Product details
				if( $import->advanced_log ) {
					$import->log .= "<br /><br />" . __( 'Import summary', 'woocommerce-product-importer' );
					if( in_array( $import->import_method, array( 'new', 'merge' ) ) )
						$import->log .= "<br />>>> " . sprintf( __( '%d Products added', 'woocommerce-product-importer' ), $import->products_added );
					else if( $import->import_method == 'delete' )
						$import->log .= "<br />>>> " . sprintf( __( '%d Products deleted', 'woocommerce-product-importer' ), $import->products_deleted );
					$import->log .= "<br />>>> " . sprintf( __( '%d Products updated', 'woocommerce-product-importer' ), $import->products_updated );
					$import->log .= "<br />>>> " . sprintf( __( '%d Products skipped', 'woocommerce-product-importer' ), $import->products_failed );
					$import->log .= "<br />>>> " . sprintf( __( '%d Variations added', 'woocommerce-product-importer' ), $import->variations_added );
				}

				$import->log .= "<br /><br />" . __( 'Import complete!', 'woocommerce-product-importer' );
			} else {
				woo_pd_error_log( sprintf( 'Error: %s', sprintf( __( 'Failed CRON import attempt from %s, %s', 'woocommerce-product-importer' ), $ip_address, $import->failed_import ) ) );
			}

			if( $gui )
				$message = '<textarea readonly="readonly" style="width:100%; height:100%;">' . str_replace( '<br />', "\n", $import->log ) . '</textarea>';

			if( $import->cancel_import && !$gui )
				return false;

		} else {
			if( $gui )
				$message = '<p>' . sprintf( __( 'Could not read file at <code>%s</code>.', 'woocommerce-product-importer' ), $upload_dir['basedir'] . $alt_path . $filename ) . '</p>';
			else
				return false;
		}
	} else {
		if( $gui ) {
			$message = '<p>' . __( 'No filename was provided.', 'woocommerce-product-importer' ) . '</p>';
		} else {
			woo_pd_error_log( sprintf( 'Error: %s', sprintf( __( 'Failed CRON import attempt from %s, no filename was provided', 'woocommerce-product-importer' ), $ip_address ) ) );
			return false;
		}
	}
	if( $gui && isset( $message ) ) {
		ob_start();
		$template = 'woo-admin_pd-cron.php';
		if( file_exists( WOO_PD_PATH . 'templates/admin/' . $template ) ) {
			include_once( WOO_PD_PATH . 'templates/admin/' . $template );
		} else {
			$message = sprintf( __( 'We couldn\'t load the import template file <code>%s</code> within <code>%s</code>, this file should be present.', 'woocommerce-product-importer' ), $template, WOO_PD_PATH . 'templates/admin/...' );
			woo_pd_admin_notice_html( $message, 'error' );
		}
		echo $message . '
	</body>
</html>';
		$output = ob_get_contents();
		ob_end_clean();
		echo $output;
		return false;
	} else if( isset( $message ) ) {
		woo_pd_error_log( sprintf( '%s: Error: %s', $filename, sprintf( __( 'CRON import of %s failed, reason: %s', 'woocommerce-exporter' ), $filename, $message ) ) );
		return false;
	}

	// If the CRON process gets this far, pass on the good news!
	woo_pd_error_log( sprintf( '%s: Success: %s', $filename, sprintf( __( 'CRON import of %s completed successfully', 'woocommerce-exporter' ), $filename ) ) );
	return true;

}
?>