<?php
function woo_pd_extend_custom_fields() {

	// WooCommerce Tab Manager - http://www.woothemes.com/products/woocommerce-tab-manager/
	if( woo_pd_detect_import_plugin( 'wc_tabmanager' ) == false )
		return;

	if( $custom_product_tabs = woo_pd_get_option( 'custom_product_tabs', '' ) )
		$custom_product_tabs = implode( "\n", $custom_product_tabs );

	ob_start();
?>
<tr>
	<th>
		<label><?php _e( 'Custom Product Tabs', 'woocommerce-product-importer' ); ?></label>
	</th>
	<td>
		<textarea name="custom_product_tabs" rows="5" cols="70"><?php echo esc_textarea( $custom_product_tabs ); ?></textarea>
		<p class="description"><?php _e( 'Include custom Product Tabs linked to individual Products within your import file by adding the Name of each Product Tab to a new line above.', 'woocommerce-product-importer' ); ?><br /><?php _e( 'For example', 'woocommerce-product-importer' ); ?>: <code>Ingredients</code> (new line) <code>Specification</code></p>
	</td>
</tr>
<?php
	ob_end_flush();

}
// add_filter( 'woo_pd_extend_custom_fields', 'woo_pd_extend_custom_fields' );

function woo_pd_extend_options_addons( $fields ) {

	// Custom Product meta
	$custom_products = woo_pd_get_option( 'custom_products', '' );
	if( !empty( $custom_products ) ) {
		$i = 0;
		$size = count( $custom_products );
		foreach( $custom_products as $custom_product ) {
			if( !empty( $custom_product ) ) {
				$field = array(
					'name' => $custom_product,
					'label' => $custom_product,
					'hover' => sprintf( apply_filters( 'woo_pd_extend_product_fields_custom_product_hover', '%s: %s' ), __( 'Custom Product', 'woocommerce-product-importer' ), $custom_product ),
					'alias' => array( woo_pd_format_column( $custom_product ) )
				);
				if( $i == 0 )
					$field['group_open'] = sprintf( '<optgroup label="%s">', __( 'Custom Product meta', 'woocommerce-product-importer' ) );
				else if( $i == ( $size - 1 ) )
					$field['group_close'] = '</optgroup>';
				$fields[] = $field;
				unset( $field );
			}
			$i++;
		}
	}
	unset( $custom_products, $custom_product );

	// Custom Attributes
	$custom_attributes = woo_pd_get_option( 'custom_attributes', '' );
	if( !empty( $custom_attributes ) ) {
		$i = 0;
		$size = count( $custom_attributes );
		foreach( $custom_attributes as $custom_attribute ) {
			if( !empty( $custom_attribute ) ) {
				$field = array(
					'name' => $custom_attribute,
					'label' => $custom_attribute,
					'hover' => sprintf( apply_filters( 'woo_pd_extend_product_fields_custom_attribute_hover', '%s: %s' ), __( 'Custom Attribute', 'woocommerce-product-importer' ), $custom_attribute ),
					'alias' => array( woo_pd_format_column( $custom_attribute ) )
				);
				if( $i == 0 )
					$field['group_open'] = sprintf( '<optgroup label="%s">', __( 'Custom Product Attributes', 'woocommerce-product-importer' ) );
				else if( $i == ( $size - 1 ) )
					$field['group_close'] = '</optgroup>';
				$fields[] = $field;
				unset( $field );
			}
			$i++;
		}
	}
	unset( $custom_attributes, $custom_attribute );

	// WooCommerce Brands Addon - http://woothemes.com/woocommerce/
	// WooCommerce Brands - http://proword.net/Woocommerce_Brands/
	if( woo_pd_detect_product_brands() ) {
		// Both Plugins share the same Term Taxonomy
		$fields[] = array(
			'name' => 'brand',
			'label' => __( 'Brand', 'woocommerce-product-importer' ),
			'alias' => array( 'product_brand', 'product_brands', 'brands' )
		);
		$fields[] = array(
			'name' => 'brand_1',
			'label' => __( 'Brand: Level 1', 'woocommerce-product-importer' )
		);
		$fields[] = array(
			'name' => 'brand_2',
			'label' => __( 'Brand: Level 2', 'woocommerce-product-importer' )
		);
		$fields[] = array(
			'name' => 'brand_3',
			'label' => __( 'Brand: Level 3', 'woocommerce-product-importer' )
		);
	}

	// Advanced Google Product Feed - http://plugins.leewillis.co.uk/downloads/woocommerce-google-product-feed/
	if( woo_pd_detect_import_plugin( 'gpf' ) ) {
		$fields[] = array(
			'name' => 'gpf_availability',
			'label' => __( 'Advanced Google Product Feed - Availability', 'woocommerce_gpf' ),
			'alias' => array( 'advanced_google_product_feed_availability' ),
			'group_open' => sprintf( '<optgroup label="%s">', __( 'Advanced Google Product Feed', 'woocommerce-product-importer' ) )
		);
		$fields[] = array(
			'name' => 'gpf_condition',
			'label' => __( 'Advanced Google Product Feed - Condition', 'woocommerce_gpf' ),
			'alias' => array( 'advanced_google_product_feed_condition' )
		);
		$fields[] = array(
			'name' => 'gpf_brand',
			'label' => __( 'Advanced Google Product Feed - Brand', 'woocommerce_gpf' ),
			'alias' => array( 'advanced_google_product_feed_brand', 'advanced_google_product_feed_brands' )
		);
		$fields[] = array(
			'name' => 'gpf_product_type',
			'label' => __( 'Advanced Google Product Feed - Product Type', 'woocommerce_gpf' ),
			'alias' => array( 'advanced_google_product_feed_product_type' )
		);
		$fields[] = array(
			'name' => 'gpf_google_product_category',
			'label' => __( 'Advanced Google Product Feed - Google Product Category', 'woocommerce_gpf' ),
			'alias' => array( 'advanced_google_product_feed_google_product_category' )
		);
		$fields[] = array(
			'name' => 'gpf_gtin',
			'label' => __( 'Advanced Google Product Feed - Global Trade Item Number (GTIN)', 'woocommerce_gpf' ),
			'alias' => array( 'advanced_google_product_feed_global_trade_item_number' )
		);
		$fields[] = array(
			'name' => 'gpf_mpn',
			'label' => __( 'Advanced Google Product Feed - Manufacturer Part Number (MPN)', 'woocommerce_gpf' ),
			'alias' => array( 'advanced_google_product_feed_manufacturer_part_number_mpn' )
		);
		$fields[] = array(
			'name' => 'gpf_gender',
			'label' => __( 'Advanced Google Product Feed - Gender', 'woocommerce_gpf' ),
			'alias' => array( 'advanced_google_product_feed_gender' )
		);
		$fields[] = array(
			'name' => 'gpf_age_group',
			'label' => __( 'Advanced Google Product Feed - Age Group', 'woocommerce_gpf' ),
			'alias' => array( 'advanced_google_product_feed_age_group' )
		);
		$fields[] = array(
			'name' => 'gpf_color',
			'label' => __( 'Advanced Google Product Feed - Colour', 'woocommerce_gpf' ),
			'alias' => array( 'advanced_google_product_feed_colour' )
		);
		$fields[] = array(
			'name' => 'gpf_size',
			'label' => __( 'Advanced Google Product Feed - Size', 'woocommerce_gpf' ),
			'alias' => array( 'advanced_google_product_feed_size' ),
			'group_close' => '</optgroup>'
		);
	}

	// All in One SEO Pack - http://wordpress.org/extend/plugins/all-in-one-seo-pack/
	if( woo_pd_detect_import_plugin( 'aioseop' ) ) {
		$fields[] = array(
			'name' => 'aioseop_keywords',
			'label' => __( 'All in One SEO - Keywords', 'woocommerce-product-importer' ),
			'group_open' => sprintf( '<optgroup label="%s">', __( 'All in One SEO', 'woocommerce-product-importer' ) )
		);
		$fields[] = array(
			'name' => 'aioseop_description',
			'label' => __( 'All in One SEO - Description', 'woocommerce-product-importer' )
		);
		$fields[] = array(
			'name' => 'aioseop_title',
			'label' => __( 'All in One SEO - Title', 'woocommerce-product-importer' )
		);
		$fields[] = array(
			'name' => 'aioseop_titleatr',
			'label' => __( 'All in One SEO - Title Attributes', 'woocommerce-product-importer' )
		);
		$fields[] = array(
			'name' => 'aioseop_menulabel',
			'label' => __( 'All in One SEO - Menu Label', 'woocommerce-product-importer' ),
			'group_close' => '</optgroup>'
		);
	}

	// WordPress SEO - http://wordpress.org/plugins/wordpress-seo/
	if( woo_pd_detect_import_plugin( 'wpseo' ) ) {
		$fields[] = array(
			'name' => 'wpseo_focuskw',
			'label' => __( 'WordPress SEO - Focus Keyword', 'woocommerce-product-importer' ),
			'alias' => array( 'wordpress_seo_focus_keyword' ),
			'group_open' => sprintf( '<optgroup label="%s">', __( 'WordPress SEO', 'woocommerce-product-importer' ) )
		);
		$fields[] = array(
			'name' => 'wpseo_metadesc',
			'label' => __( 'WordPress SEO - Meta Description', 'woocommerce-product-importer' ),
			'alias' => array( 'wordpress_seo_meta_description' )
		);
		$fields[] = array(
			'name' => 'wpseo_title',
			'label' => __( 'WordPress SEO - SEO Title', 'woocommerce-product-importer' ),
			'alias' => array( 'wordpress_seo_seo_title' )
		);
		$fields[] = array(
			'name' => 'wpseo_googleplus_description',
			'label' => __( 'WordPress SEO - Google+ Description', 'woocommerce-product-importer' ),
			'alias' => array( 'wordpress_seo_google+_description' )
		);
		$fields[] = array(
			'name' => 'wpseo_opengraph_description',
			'label' => __( 'WordPress SEO - Facebook Description', 'woocommerce-product-importer' ),
			'alias' => array( 'wordpress_seo_facebook_description' )
		);
		$fields[] = array(
			'name' => 'wpseo_meta_robots_noindex',
			'label' => __( 'WordPress SEO - Meta Robots Index', 'woocommerce-product-importer' ),
			'alias' => array( 'wordpress_seo_meta_robots_index', 'wordpress_seo_noindex' ),
		);
		$fields[] = array(
			'name' => 'wpseo_meta_robots_nofollow',
			'label' => __( 'WordPress SEO - Meta Robots Follow', 'woocommerce-product-importer' ),
			'alias' => array( 'wordpress_seo_meta_robots_follow', 'wordpress_seo_follow' )
		);
		$fields[] = array(
			'name' => 'wpseo_meta_robots_adv',
			'label' => __( 'WordPress SEO - Meta Robots Advanced', 'woocommerce-product-importer' ),
			'alias' => array( 'wordpress_seo_meta_robots_advanced' )
		);
		$fields[] = array(
			'name' => 'wpseo_sitemap_html_include',
			'label' => __( 'WordPress SEO - Include in HTML Sitemap', 'woocommerce-product-importer' ),
			'alias' => array( 'wordpress_seo_include_in_html_sitemap' )
		);
		$fields[] = array(
			'name' => 'wpseo_authorship',
			'label' => __( 'WordPress SEO - Authorship', 'woocommerce-product-importer' ),
			'alias' => array( 'wordpress_seo_authorship' )
		);
		$fields[] = array(
			'name' => 'wpseo_canonical',
			'label' =>  __( 'WordPress SEO - Canonical URL', 'woocommerce-product-importer' ),
			'alias' => array( 'wordpress_seo_canonical_url' )
		);
		$fields[] = array(
			'name' => 'wpseo_redirect',
			'label' => __( 'WordPress SEO - 301 Redirect', 'woocommerce-product-importer' ),
			'alias' => array( 'wordpress_seo_301_redirect' ),
			'group_close' => '</optgroup>'
		);
	}

	// Ultimate SEO - http://wordpress.org/plugins/seo-ultimate/
	if( woo_pd_detect_import_plugin( 'ultimate_seo' ) ) {
		$fields[] = array(
			'name' => 'useo_meta_title',
			'label' => __( 'Ultimate SEO - Title Tag', 'woocommerce-product-importer' ),
			'group_open' => sprintf( '<optgroup label="%s">', __( 'Ultimate SEO', 'woocommerce-product-importer' ) )
		);
		$fields[] = array(
			'name' => 'useo_meta_description',
			'label' => __( 'Ultimate SEO - Meta Description', 'woocommerce-product-importer' )
		);
		$fields[] = array(
			'name' => 'useo_meta_keywords',
			'label' => __( 'Ultimate SEO - Meta Keywords', 'woocommerce-product-importer' )
		);
		$fields[] = array(
			'name' => 'useo_social_title',
			'label' => __( 'Ultimate SEO - Social Title', 'woocommerce-product-importer' )
		);
		$fields[] = array(
			'name' => 'useo_social_description',
			'label' => __( 'Ultimate SEO - Social Description', 'woocommerce-product-importer' )
		);
		$fields[] = array(
			'name' => 'useo_meta_noindex',
			'label' => __( 'Ultimate SEO - NoIndex', 'woocommerce-product-importer' )
		);
		$fields[] = array(
			'name' => 'useo_meta_noautolinks',
			'label' => __( 'Ultimate SEO - Disable Autolinks', 'woocommerce-product-importer' ),
			'group_close' => '</optgroup>'
		);
	}

	// Cost of Goods - http://www.skyverge.com/product/woocommerce-cost-of-goods-tracking/
	if( woo_pd_detect_import_plugin( 'wc_cog' ) ) {
		$fields[] = array(
			'name' => 'cost_of_goods',
			'label' => __( 'Cost of Goods', 'woocommerce-product-importer' ),
			'alias' => array( 'cost_of_good', 'cog' )
		);
	}

	// WooCommerce Profit of Sales Report - http://codecanyon.net/item/woocommerce-profit-of-sales-report/9190590
	if( woo_pd_detect_import_plugin( 'wc_posr' ) ) {
		$fields[] = array(
			'name' => 'cost_of_goods',
			'label' => __( 'Cost of Goods', 'woocommerce-product-importer' ),
			'alias' => array( 'cost_of_good', 'cog' )
		);
	}

	// WooCommerce MSRP Pricing - http://www.woothemes.com/products/msrp-pricing/
	if( woo_pd_detect_import_plugin( 'wc_msrp' ) ) {
		$fields[] = array(
			'name' => 'msrp',
			'label' => __( 'MSRP Pricing', 'woocommerce-product-importer' ),
			'alias' => array( 'msrp_pricing', 'manufacturer_suggested_retail_price_msrp' )
		);
	}

	// Product Vendors - http://www.woothemes.com/products/product-vendors/
	if( woo_pd_detect_import_plugin( 'vendors' ) ) {
		$fields[] = array(
			'name' => 'vendor',
			'label' => __( 'Product Vendor', 'woocommerce-product-importer' ),
			'alias' => array( 'product_vendors', 'product_vendor', 'vendors' )
		);
	}

	// WooCommerce Custom Fields - http://www.rightpress.net/woocommerce-custom-fields
	if( woo_pd_detect_import_plugin( 'wc_customfields' ) ) {
		if( !get_option( 'wccf_migrated_to_20' ) ) {
			// Legacy WooCommerce Custom Fields was stored in a single Option
			$options = get_option( 'rp_wccf_options' );
			if( !empty( $options ) ) {
				$custom_fields = ( isset( $options[1]['product_admin_fb_config'] ) ? $options[1]['product_admin_fb_config'] : false );
				if( !empty( $custom_fields ) ) {
					$i = 0;
					$size = count( $custom_fields );
					foreach( $custom_fields as $custom_field ) {
						$field = array(
							'name' => sprintf( 'wccf_%s', sanitize_key( $custom_field['key'] ) ),
							'label' => ucfirst( $custom_field['label'] ),
							'hover' => __( 'WooCommerce Custom Fields', 'woocommerce-product-importer' )
						);
						if( $i == 0 )
							$field['group_open'] = sprintf( '<optgroup label="%s">', __( 'Product Properties', 'woocommerce-product-importer' ) );
						else if( $i == ( $size - 1 ) )
							$field['group_close'] = '</optgroup>';
						$fields[] = $field;
						unset( $field );
						$i++;
					}
				}
			}
			unset( $options );
		} else {
			// WooCommerce Custom Fields uses CPT for Product properties
			$custom_fields = woo_pd_get_wccf_product_properties();
			if( !empty( $custom_fields ) ) {
				$i = 0;
				$size = count( $custom_fields );
				foreach( $custom_fields as $custom_field ) {
					$label = get_post_meta( $custom_field->ID, 'label', true );
					$key = get_post_meta( $custom_field->ID, 'key', true );
					$field = array(
						'name' => sprintf( 'wccf_pp_%s', sanitize_key( $key ) ),
						'label' => ucfirst( $label ),
						'hover' => __( 'WooCommerce Custom Fields', 'woocommerce-product-importer' )
					);
					if( $i == 0 )
						$field['group_open'] = sprintf( '<optgroup label="%s">', __( 'Product Properties', 'woocommerce-product-importer' ) );
					else if( $i == ( $size - 1 ) )
						$field['group_close'] = '</optgroup>';
					$fields[] = $field;
					unset( $field );
					$i++;
				}
			}
			unset( $label, $key );
		}
		unset( $custom_fields, $custom_field );
	}

	// Barcodes for WooCommerce - http://www.wolkenkraft.com/produkte/barcodes-fuer-woocommerce/
	if( woo_pd_detect_import_plugin( 'wc_barcodes' ) ) {
		$fields[] = array(
			'name' => 'barcode_type',
			'label' => __( 'Barcode Type', 'woocommerce-product-importer' ),
			'hover' => __( 'Barcodes for WooCommerce', 'woocommerce-product-importer' )
		);
		$fields[] = array(
			'name' => 'barcode',
			'label' => __( 'Barcode', 'woocommerce-product-importer' ),
			'hover' => __( 'Barcodes for WooCommerce', 'woocommerce-product-importer' )
		);
	}

	// WooCommerce Subscriptions - http://www.woothemes.com/products/woocommerce-subscriptions/
	if( woo_pd_detect_import_plugin( 'subscriptions' ) ) {
		$fields[] = array(
			'name' => 'subscription_price',
			'label' => __( 'Subscription Price', 'woocommerce-product-importer' ),
			'hover' => __( 'WooCommerce Subscriptions', 'woocommerce-product-importer' ),
			'group_open' => sprintf( '<optgroup label="%s">', __( 'Subscriptions', 'woocommerce-product-importer' ) )
		);
		$fields[] = array(
			'name' => 'subscription_period_interval',
			'label' => __( 'Subscription Period Interval', 'woocommerce-product-importer' ),
			'hover' => __( 'WooCommerce Subscriptions', 'woocommerce-product-importer' )
		);
		$fields[] = array(
			'name' => 'subscription_period',
			'label' => __( 'Subscription Period', 'woocommerce-product-importer' ),
			'hover' => __( 'WooCommerce Subscriptions', 'woocommerce-product-importer' )
		);
		$fields[] = array(
			'name' => 'subscription_length',
			'label' => __( 'Subscription Length', 'woocommerce-product-importer' ),
			'hover' => __( 'WooCommerce Subscriptions', 'woocommerce-product-importer' )
		);
		$fields[] = array(
			'name' => 'subscription_sign_up_fee',
			'label' => __( 'Subscription Sign-up Fee', 'woocommerce-product-importer' ),
			'hover' => __( 'WooCommerce Subscriptions', 'woocommerce-product-importer' )
		);
		$fields[] = array(
			'name' => 'subscription_trial_length',
			'label' => __( 'Subscription Trial Length', 'woocommerce-product-importer' ),
			'hover' => __( 'WooCommerce Subscriptions', 'woocommerce-product-importer' )
		);
		$fields[] = array(
			'name' => 'subscription_trial_period',
			'label' => __( 'Subscription Trial Period', 'woocommerce-product-importer' ),
			'hover' => __( 'WooCommerce Subscriptions', 'woocommerce-product-importer' )
		);
		$fields[] = array(
			'name' => 'subscription_limit',
			'label' => __( 'Limit Subscription', 'woocommerce-product-importer' ),
			'hover' => __( 'WooCommerce Subscriptions', 'woocommerce-product-importer' ),
			'group_close' => '</optgroup>'
		);
	}

	// WooCommerce Ultimate Multi Currency Suite - https://codecanyon.net/item/woocommerce-ultimate-multi-currency-suite/11997014
	if( woo_pd_detect_import_plugin( 'wc_umcs' ) ) {
		$currencies = json_decode( get_option( 'wcumcs_available_currencies' ) );
		if( !empty( $currencies ) ) {
			$i = 0;
			$size = count( $currencies );

			// Reduce the count as we skip the base currency
			if( $size )
				$size--;

			$current_currency = ( function_exists( 'get_woocommerce_currency' ) ? get_woocommerce_currency() : false );
			foreach( $currencies as $currency_code => $currency_data ) {

				// Skip the base currency
				if( $currency_code == $current_currency )
					continue;

				// Regular Price
				$field = array(
					'name' => sprintf( 'wcumcs_regular_price_%s', sanitize_key( $currency_code ) ),
					'label' => sprintf( __( 'Regular Price (%s)', 'woocommerce-product-importer' ), $currency_code ),
					'hover' => __( 'WooCommerce Ultimate Multi Currency Suite', 'woocommerce-product-importer' ),
					'alias' => array( sprintf( 'regular_price_%s', sanitize_key( $currency_code ) ) )
				);
				if( $i == 0 )
					$field['group_open'] = sprintf( '<optgroup label="%s">', __( 'Ultimate Multi Currency Suite', 'woocommerce-product-importer' ) );
				$fields[] = $field;
				unset( $field );

				// Sale Price
				$field = array(
					'name' => sprintf( 'wcumcs_sale_price_%s', sanitize_key( $currency_code ) ),
					'label' => sprintf( __( 'Sale Price (%s)', 'woocommerce-product-importer' ), $currency_code ),
					'hover' => __( 'WooCommerce Ultimate Multi Currency Suite', 'woocommerce-product-importer' ),
					'alias' => array( sprintf( 'sale_price_%s', sanitize_key( $currency_code ) ) )
				);
				if( $i == ( $size - 1 ) )
					$field['group_close'] = '</optgroup>';
				$fields[] = $field;
				unset( $field );

				$i++;
			}
			unset( $currency_code, $currency_data, $current_currency );
		}
		unset( $currencies );
	}

	// WooCommerce Wholesale Prices - https://wordpress.org/plugins/woocommerce-wholesale-prices/
	if( woo_pd_detect_import_plugin( 'wc_wholesale_prices' ) ) {
		$wholesale_roles = woo_pd_get_wholesale_prices_roles();
		if( !empty( $wholesale_roles ) ) {
			$i = 0;
			$size = count( $wholesale_roles );
			foreach( $wholesale_roles as $key => $wholesale_role ) {
				$field = array(
					'name' => sprintf( '%s_wholesale_price', $key ),
					'label' => sprintf( __( 'Wholesale Price: %s', 'woocommerce-product-importer' ), $wholesale_role['roleName'] ),
					'hover' => __( 'WooCommerce Wholesale Prices', 'woocommerce-product-importer' )
				);
				if( $i == 0 )
					$field['group_open'] = sprintf( '<optgroup label="%s">', __( 'WooCommerce Wholesale Prices', 'woocommerce-product-importer' ) );
				if( $i == ( $size - 1 ) )
					$field['group_close'] = '</optgroup>';
				$fields[] = $field;
				unset( $field );

				$i++;
			}
		}
		unset( $wholesale_roles, $wholesale_role, $key );
	}

	// WooCommerce Multilingual - https://wordpress.org/plugins/woocommerce-multilingual/
	if( woo_pd_detect_wpml() || woo_pd_detect_import_plugin( 'wpml_wc' ) ) {
		$fields[] = array(
			'name' => 'language',
			'label' => __( 'Language', 'woocommerce-product-importer' ),
			'hover' => __( 'WooCommerce Multilingual', 'woocommerce-product-importer' )
		);
	}

	// WooCommerce Bookings - http://www.woothemes.com/products/woocommerce-bookings/
	if( woo_pd_detect_import_plugin( 'woocommerce_bookings' ) ) {
		$fields[] = array(
			'name' => 'booking_has_persons',
			'label' => __( 'Booking Has Persons', 'woocommerce-product-importer' ),
			'hover' => __( 'WooCommerce Bookings', 'woocommerce-product-importer' ),
			'group_open' => sprintf( '<optgroup label="%s">', __( 'Bookings', 'woocommerce-product-importer' ) )
		);
		$fields[] = array(
			'name' => 'booking_has_resources',
			'label' => __( 'Booking Has Resources', 'woocommerce-product-importer' ),
			'hover' => __( 'WooCommerce Bookings', 'woocommerce-product-importer' )
		);
		$fields[] = array(
			'name' => 'booking_base_cost',
			'label' => __( 'Booking Base Cost', 'woocommerce-product-importer' ),
			'hover' => __( 'WooCommerce Bookings', 'woocommerce-product-importer' )
		);
		$fields[] = array(
			'name' => 'booking_block_cost',
			'label' => __( 'Booking Block Cost', 'woocommerce-product-importer' ),
			'hover' => __( 'WooCommerce Bookings', 'woocommerce-product-importer' )
		);
		$fields[] = array(
			'name' => 'booking_display_cost',
			'label' => __( 'Booking Display Cost', 'woocommerce-product-importer' ),
			'hover' => __( 'WooCommerce Bookings', 'woocommerce-product-importer' )
		);
		$fields[] = array(
			'name' => 'booking_requires_confirmation',
			'label' => __( 'Booking Requires Confirmation', 'woocommerce-product-importer' ),
			'hover' => __( 'WooCommerce Bookings', 'woocommerce-product-importer' )
		);
		$fields[] = array(
			'name' => 'booking_user_can_cancel',
			'label' => __( 'Booking Can Be Cancelled', 'woocommerce-product-importer' ),
			'hover' => __( 'WooCommerce Bookings', 'woocommerce-product-importer' ),
			'alias' => array( 'booking_can_be_cancelled' ),
			'group_close' => '</optgroup>'
		);
	}

	// WooCommerce Show Single Variations - https://iconicwp.com/products/woocommerce-show-single-variations/
	if( woo_pd_detect_import_plugin( 'wc_show_single_variations' ) ) {
		$fields[] = array(
			'name' => 'show_search_results',
			'label' => __( 'Show in Search Results', 'woocommerce-product-importer' ),
			'hover' => __( 'WooCommerce Show Single Variations', 'woocommerce-product-importer' ),
			'alias' => array( 'show_search_results', 'show_in_search_results' ),
			'group_open' => sprintf( '<optgroup label="%s">', __( 'Show Single Variations', 'woocommerce-product-importer' ) )
		);
		$fields[] = array(
			'name' => 'show_filtered_results',
			'label' => __( 'Show in Filtered Results', 'woocommerce-product-importer' ),
			'hover' => __( 'WooCommerce Show Single Variations', 'woocommerce-product-importer' ),
			'alias' => array( 'show_in_filtered_results' )
		);
		$fields[] = array(
			'name' => 'show_catalog',
			'label' => __( 'Show in Catalog', 'woocommerce-product-importer' ),
			'hover' => __( 'WooCommerce Show Single Variations', 'woocommerce-product-importer' ),
			'alias' => array( 'show_in_catalog' )
		);
		$fields[] = array(
			'name' => 'disable_add_to_cart',
			'label' => __( 'Disable Add to Cart', 'woocommerce-product-importer' ),
			'hover' => __( 'WooCommerce Show Single Variations', 'woocommerce-product-importer' ),
			'alias' => array( 'disable_add_to_cart' ),
			'group_close' => '</optgroup>'
		);
	}

	// WooCommerce Tab Manager - http://www.woothemes.com/products/woocommerce-tab-manager/
	if( woo_pd_detect_import_plugin( 'wc_tabmanager' ) ) {
		$product_tabs = woo_pd_get_product_tabs();
/*
		// @mod - Add support for custom Product Tabs in 4.0
		// Custom Product Tabs
		$custom_product_tabs = woo_pd_get_option( 'custom_product_tabs', '' );
*/
		if( !empty( $product_tabs ) ) {
			$i = 0;
			$size = count( $product_tabs );
			foreach( $product_tabs as $product_tab ) {
				if( !empty( $product_tab ) ) {
					$field = array(
						'name' => sprintf( 'product_tab_%s', sanitize_key( $product_tab->post_name ) ),
						'label' => sprintf( __( 'Product Tab: %s', 'woocommerce-product-importer' ), $product_tab->post_title ),
						'hover' => sprintf( __( 'Product Tab: %s', 'woocommerce-product-importer' ), $product_tab->post_title )
					);
					if( $i == 0 )
						$field['group_open'] = sprintf( '<optgroup label="%s">', __( 'WooCommerce Tab Manager', 'woocommerce-product-importer' ) );
					if( $i == ( $size - 1 ) )
						$field['group_close'] = '</optgroup>';
					$fields[] = $field;
					unset( $field );

					$i++;
				}
			}
		}
		unset( $product_tabs, $product_tab );
	}

/*
	// Sample
	$fields[] = array(
		'name' => 'sample',
		'label' => __( 'Sample Product Meta', 'woocommerce-product-importer' )
	);
*/

	return $fields;

}
add_filter( 'woo_pd_options_addons', 'woo_pd_extend_options_addons' );

function woo_pd_extend_import_addons( $import, $csv_data ) {

	// Custom Product meta
	$custom_products = woo_pd_get_option( 'custom_products', '' );
	if( !empty( $custom_products ) ) {
		foreach( $custom_products as $custom_product ) {
			if( !empty( $custom_product ) ) {
				if( isset( $csv_data[$custom_product] ) ) {
					$import->headers[] = sanitize_key( $custom_product );
					$import->{sanitize_key( 'csv_' . $custom_product )} = array_filter( $csv_data[$custom_product], 'strlen' );
					$import->log .= "<br />>>> " . sprintf( __( 'Custom Product meta: %s has been detected and grouped', 'woocommerce-product-importer' ), $custom_product );
				}
			}
		}
	}

	// Custom Attributes
	$custom_attributes = woo_pd_get_option( 'custom_attributes', '' );
	if( !empty( $custom_attributes ) ) {
		foreach( $custom_attributes as $custom_attribute ) {
			if( !empty( $custom_attribute ) ) {
				if( isset( $csv_data[$custom_attribute] ) ) {
					$import->headers[] = sanitize_key( $custom_attribute );
					$import->{sanitize_key( 'csv_' . $custom_attribute )} = array_filter( $csv_data[$custom_attribute], 'strlen' );
					$import->log .= "<br />>>> " . sprintf( __( 'Custom Product Attribute: %s has been detected and grouped', 'woocommerce-product-importer' ), $custom_attribute );
				}
			}
		}
	}

	// WooCommerce Brands Addon - http://woothemes.com/woocommerce/
	// WooCommerce Brands - http://proword.net/Woocommerce_Brands/
	if( woo_pd_detect_product_brands() ) {
		if( isset( $csv_data['brand'] ) ) {
			$import->headers[] = 'brand';
			$import->csv_brand = array_filter( $csv_data['brand'] );
			$import->log .= "<br />>>> " . __( 'Brand has been detected and grouped', 'woocommerce-product-importer' );
		}
		if( isset( $csv_data['brand_1'] ) ) {
			$import->headers[] = 'brand_1';
			$import->csv_brand_1 = array_filter( $csv_data['brand_1'] );
			$import->log .= "<br />>>> " . __( 'Brand > Child has been detected and grouped', 'woocommerce-product-importer' );
		}
		if( isset( $csv_data['brand_2'] ) ) {
			$import->headers[] = 'brand_2';
			$import->csv_brand_2 = array_filter( $csv_data['brand_2'] );
			$import->log .= "<br />>>> " . __( 'Brand > Child > Child has been detected and grouped', 'woocommerce-product-importer' );
		}
		if( isset( $csv_data['brand_3'] ) ) {
			$import->headers[] = 'brand_3';
			$import->csv_brand_3 = array_filter( $csv_data['brand_3'] );
			$import->log .= "<br />>>> " . __( 'Brand > Child > Child > Child has been detected and grouped', 'woocommerce-product-importer' );
		}
	}

	// All in One SEO Pack integration
	if( isset( $csv_data['aioseop_keywords'] ) ) {
		$import->headers[] = 'aioseop_keywords';
		$import->csv_aioseop_keywords = array_filter( $csv_data['aioseop_keywords'] );
		$import->log .= "<br />>>> " . __( 'All in One SEO Pack - Keywords has been detected and grouped', 'woocommerce-product-importer' );
	}
	if( isset( $csv_data['aioseop_description'] ) ) {
		$import->headers[] = 'aioseop_description';
		$import->csv_aioseop_description = array_filter( $csv_data['aioseop_description'] );
		$import->log .= "<br />>>> " . __( 'All in One SEO Pack - Description has been detected and grouped', 'woocommerce-product-importer' );
	}
	if( isset( $csv_data['aioseop_title'] ) ) {
		$import->headers[] = 'aioseop_title';
		$import->csv_aioseop_title = array_filter( $csv_data['aioseop_title'] );
		$import->log .= "<br />>>> " . __( 'All in One SEO Pack - Title has been detected and grouped', 'woocommerce-product-importer' );
	}
	if( isset( $csv_data['aioseop_titleatr'] ) ) {
		$import->headers[] = 'aioseop_titleatr';
		$import->csv_aioseop_titleatr = array_filter( $csv_data['aioseop_titleatr'] );
		$import->log .= "<br />>>> " . __( 'All in One SEO Pack - Title Attributes has been detected and grouped', 'woocommerce-product-importer' );
	}
	if( isset( $csv_data['aioseop_menulabel'] ) ) {
		$import->headers[] = 'aioseop_menulabel';
		$import->csv_aioseop_menulabel = array_filter( $csv_data['aioseop_menulabel'] );
		$import->log .= "<br />>>> " . __( 'All in One SEO Pack - Menu Label has been detected and grouped', 'woocommerce-product-importer' );
	}

	// Advanced Google Product Feed - http://www.leewillis.co.uk/wordpress-plugins/
	if( woo_pd_detect_import_plugin( 'gpf' ) ) {
		if( isset( $csv_data['gpf_availability'] ) ) {
			$import->headers[] = 'gpf_availability';
			$import->csv_gpf_availability = array_filter( $csv_data['gpf_availability'] );
			$import->log .= "<br />>>> " . __( 'Advanced Google Product Feed - Availability has been detected and grouped', 'woocommerce-product-importer' );
		}
		if( isset( $csv_data['gpf_condition'] ) ) {
			$import->headers[] = 'gpf_condition';
			$import->csv_gpf_condition = array_filter( $csv_data['gpf_condition'] );
			$import->log .= "<br />>>> " . __( 'Advanced Google Product Feed - Condition has been detected and grouped', 'woocommerce-product-importer' );
		}
		if( isset( $csv_data['gpf_brand'] ) ) {
			$import->headers[] = 'gpf_brand';
			$import->csv_gpf_brand = array_filter( $csv_data['gpf_brand'] );
			$import->log .= "<br />>>> " . __( 'Advanced Google Product Feed - Brand has been detected and grouped', 'woocommerce-product-importer' );
		}
		if( isset( $csv_data['gpf_product_type'] ) ) {
			$import->headers[] = 'gpf_product_type';
			$import->csv_gpf_product_type = array_filter( $csv_data['gpf_product_type'] );
			$import->log .= "<br />>>> " . __( 'Advanced Google Product Feed - Product Type has been detected and grouped', 'woocommerce-product-importer' );
		}
		if( isset( $csv_data['gpf_google_product_category'] ) ) {
			$import->headers[] = 'gpf_google_product_category';
			$import->csv_gpf_google_product_category = array_filter( $csv_data['gpf_google_product_category'] );
			$import->log .= "<br />>>> " . __( 'Advanced Google Product Feed - Google Product Category has been detected and grouped', 'woocommerce-product-importer' );
		}
		if( isset( $csv_data['gpf_gtin'] ) ) {
			$import->headers[] = 'gpf_gtin';
			$import->csv_gpf_gtin = array_filter( $csv_data['gpf_gtin'] );
			$import->log .= "<br />>>> " . __( 'Advanced Google Product Feed - Global Trade Item Number (GTIN) has been detected and grouped', 'woocommerce-product-importer' );
		}
		if( isset( $csv_data['gpf_mpn'] ) ) {
			$import->headers[] = 'gpf_mpn';
			$import->csv_gpf_mpn = array_filter( $csv_data['gpf_mpn'] );
			$import->log .= "<br />>>> " . __( 'Advanced Google Product Feed - Manufacturer Part Number (MPN) has been detected and grouped', 'woocommerce-product-importer' );
		}
		if( isset( $csv_data['gpf_gender'] ) ) {
			$import->headers[] = 'gpf_gender';
			$import->csv_gpf_gender = array_filter( $csv_data['gpf_gender'] );
			$import->log .= "<br />>>> " . __( 'Advanced Google Product Feed - Gender has been detected and grouped', 'woocommerce-product-importer' );
		}
		if( isset( $csv_data['gpf_age_group'] ) ) {
			$import->headers[] = 'gpf_age_group';
			$import->csv_gpf_age_group = array_filter( $csv_data['gpf_age_group'] );
			$import->log .= "<br />>>> " . __( 'Advanced Google Product Feed - Age Group has been detected and grouped', 'woocommerce-product-importer' );
		}
		if( isset( $csv_data['gpf_color'] ) ) {
			$import->headers[] = 'gpf_color';
			$import->csv_gpf_color = array_filter( $csv_data['gpf_color'] );
			$import->log .= "<br />>>> " . __( 'Advanced Google Product Feed - Colour has been detected and grouped', 'woocommerce-product-importer' );
		}
		if( isset( $csv_data['gpf_size'] ) ) {
			$import->headers[] = 'gpf_size';
			$import->csv_gpf_size = array_filter( $csv_data['gpf_size'], 'strlen' );
			$import->log .= "<br />>>> " . __( 'Advanced Google Product Feed - Size has been detected and grouped', 'woocommerce-product-importer' );
		}
	}

	// Ultimate SEO - http://wordpress.org/plugins/seo-ultimate/
	if( woo_pd_detect_import_plugin( 'ultimate_seo' ) ) {
		if( isset( $csv_data['useo_meta_title'] ) ) {
			$import->headers[] = 'useo_meta_title';
			$import->csv_useo_meta_title = array_filter( $csv_data['useo_meta_title'] );
			$import->log .= "<br />>>> " . __( 'Ultimate SEO - Meta Title has been detected and grouped', 'woocommerce-product-importer' );
		}
		if( isset( $csv_data['useo_meta_description'] ) ) {
			$import->headers[] = 'useo_meta_description';
			$import->csv_useo_meta_description = array_filter( $csv_data['useo_meta_description'] );
			$import->log .= "<br />>>> " . __( 'Ultimate SEO - Meta Description has been detected and grouped', 'woocommerce-product-importer' );
		}
		if( isset( $csv_data['useo_meta_keywords'] ) ) {
			$import->headers[] = 'useo_meta_keywords';
			$import->csv_useo_meta_keywords = array_filter( $csv_data['useo_meta_keywords'] );
			$import->log .= "<br />>>> " . __( 'Ultimate SEO - Meta Keywords has been detected and grouped', 'woocommerce-product-importer' );
		}
		if( isset( $csv_data['useo_social_title'] ) ) {
			$import->headers[] = 'useo_social_title';
			$import->csv_social_title = array_filter( $csv_data['useo_social_title'] );
			$import->log .= "<br />>>> " . __( 'Ultimate SEO - Social Title has been detected and grouped', 'woocommerce-product-importer' );
		}
		if( isset( $csv_data['useo_social_description'] ) ) {
			$import->headers[] = 'useo_social_description';
			$import->csv_useo_social_description = array_filter( $csv_data['useo_social_description'] );
			$import->log .= "<br />>>> " . __( 'Ultimate SEO - Social Description has been detected and grouped', 'woocommerce-product-importer' );
		}
		if( isset( $csv_data['useo_meta_noindex'] ) ) {
			$import->headers[] = 'useo_meta_noindex';
			$import->csv_useo_meta_noindex = array_filter( $csv_data['useo_meta_noindex'], 'strlen' );
			$import->log .= "<br />>>> " . __( 'Ultimate SEO - noindex has been detected and grouped', 'woocommerce-product-importer' );
		}
		if( isset( $csv_data['useo_meta_noautolinks'] ) ) {
			$import->headers[] = 'useo_meta_noautolinks';
			$import->csv_useo_meta_noautolinks = array_filter( $csv_data['useo_meta_noautolinks'], 'strlen' );
			$import->log .= "<br />>>> " . __( 'Ultimate SEO - Disable Autolinks has been detected and grouped', 'woocommerce-product-importer' );
		}
	}

	// WordPress SEO - http://yoast.com/wordpress/seo/#utm_source=wpadmin&utm_medium=plugin&utm_campaign=wpseoplugin
	if( woo_pd_detect_import_plugin( 'wpseo' ) ) {
		if( isset( $csv_data['wpseo_focuskw'] ) ) {
			$import->headers[] = 'wpseo_focuskw';
			$import->csv_wpseo_focuskw = array_filter( $csv_data['wpseo_focuskw'] );
			$import->log .= "<br />>>> " . __( 'WordPress SEO - Focus Keyword has been detected and grouped', 'woocommerce-product-importer' );
		}
		if( isset( $csv_data['wpseo_metadesc'] ) ) {
			$import->headers[] = 'wpseo_metadesc';
			$import->csv_wpseo_metadesc = array_filter( $csv_data['wpseo_metadesc'] );
			$import->log .= "<br />>>> " . __( 'WordPress SEO - Meta Description has been detected and grouped', 'woocommerce-product-importer' );
		}
		if( isset( $csv_data['wpseo_title'] ) ) {
			$import->headers[] = 'wpseo_title';
			$import->csv_wpseo_title = array_filter( $csv_data['wpseo_title'] );
			$import->log .= "<br />>>> " . __( 'WordPress SEO - SEO Title has been detected and grouped', 'woocommerce-product-importer' );
		}
		if( isset( $csv_data['wpseo_googleplus_description'] ) ) {
			$import->headers[] = 'wpseo_googleplus_description';
			$import->csv_wpseo_googleplus_description = array_filter( $csv_data['wpseo_googleplus_description'] );
			$import->log .= "<br />>>> " . __( 'WordPress SEO - Google+ Description has been detected and grouped', 'woocommerce-product-importer' );
		}
		if( isset( $csv_data['wpseo_opengraph_description'] ) ) {
			$import->headers[] = 'wpseo_opengraph_description';
			$import->csv_wpseo_opengraph_description = array_filter( $csv_data['wpseo_opengraph_description'] );
			$import->log .= "<br />>>> " . __( 'WordPress SEO - Facebook Description has been detected and grouped', 'woocommerce-product-importer' );
		}
		if( isset( $csv_data['wpseo_meta_robots_noindex'] ) ) {
			$import->headers[] = 'wpseo_meta_robots_noindex';
			$import->csv_wpseo_meta_robots_noindex = array_filter( $csv_data['wpseo_meta_robots_noindex'] );
			$import->log .= "<br />>>> " . __( 'WordPress SEO - Meta Robots Index has been detected and grouped', 'woocommerce-product-importer' );
		}
		if( isset( $csv_data['wpseo_meta_robots_nofollow'] ) ) {
			$import->headers[] = 'wpseo_meta_robots_nofollow';
			$import->csv_wpseo_meta_robots_nofollow = array_filter( $csv_data['wpseo_meta_robots_nofollow'] );
			$import->log .= "<br />>>> " . __( 'WordPress SEO - Meta Robots Follow has been detected and grouped', 'woocommerce-product-importer' );
		}
		if( isset( $csv_data['wpseo_meta_robots_adv'] ) ) {
			$import->headers[] = 'wpseo_meta_robots_adv';
			$import->csv_wpseo_meta_robots_adv = array_filter( $csv_data['wpseo_meta_robots_adv'] );
			$import->log .= "<br />>>> " . __( 'WordPress SEO - Meta Robots Advanced has been detected and grouped', 'woocommerce-product-importer' );
		}
		if( isset( $csv_data['wpseo_sitemap_html_include'] ) ) {
			$import->headers[] = 'wpseo_sitemap_html_include';
			$import->csv_wpseo_sitemap_html_include = array_filter( $csv_data['wpseo_sitemap_html_include'] );
			$import->log .= "<br />>>> " . __( 'WordPress SEO - Include in HTML Sitemap Advanced has been detected and grouped', 'woocommerce-product-importer' );
		}
		if( isset( $csv_data['wpseo_authorship'] ) ) {
			$import->headers[] = 'wpseo_authorship';
			$import->csv_wpseo_authorship = array_filter( $csv_data['wpseo_authorship'] );
			$import->log .= "<br />>>> " . __( 'WordPress SEO - Authorship has been detected and grouped', 'woocommerce-product-importer' );
		}
		if( isset( $csv_data['wpseo_canonical'] ) ) {
			$import->headers[] = 'wpseo_canonical';
			$import->csv_wpseo_canonical = array_filter( $csv_data['wpseo_canonical'] );
			$import->log .= "<br />>>> " . __( 'WordPress SEO - Canonical URL has been detected and grouped', 'woocommerce-product-importer' );
		}
		if( isset( $csv_data['wpseo_redirect'] ) ) {
			$import->headers[] = 'wpseo_redirect';
			$import->csv_wpseo_redirect = array_filter( $csv_data['wpseo_redirect'] );
			$import->log .= "<br />>>> " . __( 'WordPress SEO - 301 Redirect has been detected and grouped', 'woocommerce-product-importer' );
		}
	}

	// Cost of Goods - http://www.skyverge.com/product/woocommerce-cost-of-goods-tracking/
	if( woo_pd_detect_import_plugin( 'wc_cog' ) ) {
		if( isset( $csv_data['cost_of_goods'] ) ) {
			$import->headers[] = 'cost_of_goods';
			$import->csv_cost_of_goods = array_filter( $csv_data['cost_of_goods'] );
			$import->log .= "<br />>>> " . __( 'Cost of Goods has been detected and grouped', 'woocommerce-product-importer' );
		}
	}

	// WooCommerce Profit of Sales Report - http://codecanyon.net/item/woocommerce-profit-of-sales-report/9190590
	if( woo_pd_detect_import_plugin( 'wc_posr' ) ) {
		if( isset( $csv_data['cost_of_goods'] ) ) {
			$import->headers[] = 'cost_of_goods';
			$import->csv_cost_of_goods = array_filter( $csv_data['cost_of_goods'] );
			$import->log .= "<br />>>> " . __( 'Cost of Goods has been detected and grouped', 'woocommerce-product-importer' );
		}
	}

	// WooCommerce MSRP Pricing - http://www.woothemes.com/products/msrp-pricing/
	if( woo_pd_detect_import_plugin( 'wc_msrp' ) ) {
		if( isset( $csv_data['msrp'] ) ) {
			$import->headers[] = 'msrp';
			$import->csv_msrp = array_filter( $csv_data['msrp'] );
			$import->log .= "<br />>>> " . __( 'MSRP Pricing has been detected and grouped', 'woocommerce-product-importer' );
		}
	}

	// WooCommerce Custom Fields - http://www.rightpress.net/woocommerce-custom-fields
	if( woo_pd_detect_import_plugin( 'wc_customfields' ) ) {
		if( !get_option( 'wccf_migrated_to_20' ) ) {
			// Legacy WooCommerce Custom Fields was stored in a single Option
			$options = get_option( 'rp_wccf_options' );
			if( !empty( $options ) ) {
				$custom_fields = ( isset( $options[1]['product_admin_fb_config'] ) ? $options[1]['product_admin_fb_config'] : false );
				if( !empty( $custom_fields ) ) {
					foreach( $custom_fields as $custom_field ) {
						if( isset( $csv_data[sprintf( 'wccf_%s', sanitize_key( $custom_field['key'] ) )] ) ) {
							$import->headers[] = sprintf( 'wccf_%s', sanitize_key( $custom_field['key'] ) );
							$import->{'csv_' . sprintf( 'wccf_%s', sanitize_key( $custom_field['key'] ) )} = array_filter( $csv_data[sprintf( 'wccf_%s', sanitize_key( $custom_field['key'] ) )] );
							$import->log .= "<br />>>> " . sprintf( __( 'Product Properties: %s has been detected and grouped', 'woocommerce-product-importer' ), ucfirst( $custom_field['label'] ) );
						}
					}
				}
			}
			unset( $options );
		} else {
			// WooCommerce Custom Fields uses CPT for Product properties
			$custom_fields = woo_pd_get_wccf_product_properties();
			if( !empty( $custom_fields ) ) {
				foreach( $custom_fields as $custom_field ) {
					$label = get_post_meta( $custom_field->ID, 'label', true );
					$key = get_post_meta( $custom_field->ID, 'key', true );
					if( isset( $csv_data[sprintf( 'wccf_pp_%s', sanitize_key( $key ) )] ) ) {
						$import->headers[] = sprintf( 'wccf_pp_%s', sanitize_key( $key ) );
						$import->{'csv_' . sprintf( 'wccf_pp_%s', sanitize_key( $key ) )} = array_filter( $csv_data[sprintf( 'wccf_pp_%s', sanitize_key( $key ) )] );
						$import->log .= "<br />>>> " . sprintf( __( 'Product Properties: %s has been detected and grouped', 'woocommerce-product-importer' ), ucfirst( $label ) );
					}
				}
			}
			unset( $label, $key );
		}
		unset( $custom_fields, $custom_field );
	}

	// Barcodes for WooCommerce - http://www.wolkenkraft.com/produkte/barcodes-fuer-woocommerce/
	if( woo_pd_detect_import_plugin( 'wc_barcodes' ) ) {
		if( isset( $csv_data['barcode_type'] ) ) {
			$import->headers[] = 'barcode_type';
			$import->csv_barcode_type = array_filter( $csv_data['barcode_type'] );
			$import->log .= "<br />>>> " . __( 'Barcode Type has been detected and grouped', 'woocommerce-product-importer' );
		}
		if( isset( $csv_data['barcode'] ) ) {
			$import->headers[] = 'barcode';
			$import->csv_barcode = array_filter( $csv_data['barcode'] );
			$import->log .= "<br />>>> " . __( 'Barcode has been detected and grouped', 'woocommerce-product-importer' );
		}
	}

	// WooCommerce Subscriptions - http://www.woothemes.com/products/woocommerce-subscriptions/
	if( woo_pd_detect_import_plugin( 'subscriptions' ) ) {
		if( isset( $csv_data['subscription_price'] ) ) {
			$import->headers[] = 'subscription_price';
			$import->csv_subscription_price = array_filter( $csv_data['subscription_price'] );
			$import->log .= "<br />>>> " . __( 'Subscription Price has been detected and grouped', 'woocommerce-product-importer' );
		}
		if( isset( $csv_data['subscription_period'] ) ) {
			$import->headers[] = 'subscription_period';
			$import->csv_subscription_period = array_filter( $csv_data['subscription_period'] );
			$import->log .= "<br />>>> " . __( 'Subscription Period has been detected and grouped', 'woocommerce-product-importer' );
		}
		if( isset( $csv_data['subscription_period_interval'] ) ) {
			$import->headers[] = 'subscription_period_interval';
			$import->csv_subscription_period_interval = array_filter( $csv_data['subscription_period_interval'] );
			$import->log .= "<br />>>> " . __( 'Subscription Period Interval has been detected and grouped', 'woocommerce-product-importer' );
		}
		if( isset( $csv_data['subscription_length'] ) ) {
			$import->headers[] = 'subscription_length';
			$import->csv_subscription_length = array_filter( $csv_data['subscription_length'], 'strlen' );
			$import->log .= "<br />>>> " . __( 'Subscription Length has been detected and grouped', 'woocommerce-product-importer' );
		}
		if( isset( $csv_data['subscription_sign_up_fee'] ) ) {
			$import->headers[] = 'subscription_sign_up_fee';
			$import->csv_subscription_sign_up_fee = array_filter( $csv_data['subscription_sign_up_fee'] );
			$import->log .= "<br />>>> " . __( 'Subscription Sign-up Fee has been detected and grouped', 'woocommerce-product-importer' );
		}
		if( isset( $csv_data['subscription_trial_length'] ) ) {
			$import->headers[] = 'subscription_trial_length';
			$import->csv_subscription_trial_length = array_filter( $csv_data['subscription_trial_length'] );
			$import->log .= "<br />>>> " . __( 'Subscription Trial Length has been detected and grouped', 'woocommerce-product-importer' );
		}
		if( isset( $csv_data['subscription_trial_period'] ) ) {
			$import->headers[] = 'subscription_trial_period';
			$import->csv_subscription_trial_period = array_filter( $csv_data['subscription_trial_period'] );
			$import->log .= "<br />>>> " . __( 'Subscription Trial Period has been detected and grouped', 'woocommerce-product-importer' );
		}
		if( isset( $csv_data['subscription_limit'] ) ) {
			$import->headers[] = 'subscription_limit';
			$import->csv_subscription_limit = array_filter( $csv_data['subscription_limit'] );
			$import->log .= "<br />>>> " . __( 'Subscription Limit has been detected and grouped', 'woocommerce-product-importer' );
		}
	}

	// WooCommerce Ultimate Multi Currency Suite - https://codecanyon.net/item/woocommerce-ultimate-multi-currency-suite/11997014
	if( woo_pd_detect_import_plugin( 'wc_umcs' ) ) {
		$currencies = json_decode( get_option( 'wcumcs_available_currencies' ) );
		if( !empty( $currencies ) ) {
			foreach( $currencies as $currency_code => $currency_data ) {
				if( isset( $csv_data[sprintf( 'wcumcs_regular_price_%s', sanitize_key( $currency_code ) )] ) ) {
					$import->headers[] = sprintf( 'wcumcs_regular_price_%s', sanitize_key( $currency_code ) );
					$import->{'csv_' . sprintf( 'wcumcs_regular_price_%s', sanitize_key( $currency_code ) )} = array_filter( $csv_data[sprintf( 'wcumcs_regular_price_%s', sanitize_key( $currency_code ) )] );
					$import->log .= "<br />>>> " . sprintf( __( 'Regular Price (%s) has been detected and grouped', 'woocommerce-product-importer' ), $currency_code );
				}
				if( isset( $csv_data[sprintf( 'wcumcs_sale_price_%s', sanitize_key( $currency_code ) )] ) ) {
					$import->headers[] = sprintf( 'wcumcs_sale_price_%s', sanitize_key( $currency_code ) );
					$import->{'csv_' . sprintf( 'wcumcs_sale_price_%s', sanitize_key( $currency_code ) )} = array_filter( $csv_data[sprintf( 'wcumcs_sale_price_%s', sanitize_key( $currency_code ) )] );
					$import->log .= "<br />>>> " . sprintf( __( 'Sale Price (%s) has been detected and grouped', 'woocommerce-product-importer' ), $currency_code );
				}
			}
			unset( $currency_code, $currency_data );
		}
		unset( $currencies );
	}

	// WooCommerce Wholesale Prices - https://wordpress.org/plugins/woocommerce-wholesale-prices/
	if( woo_pd_detect_import_plugin( 'wc_wholesale_prices' ) ) {
		$wholesale_roles = woo_pd_get_wholesale_prices_roles();
		if( !empty( $wholesale_roles ) ) {
			foreach( $wholesale_roles as $key => $wholesale_role ) {
				if( isset( $csv_data[sprintf( '%s_wholesale_price', $key )] ) ) {
					$import->headers[] = sprintf( '%s_wholesale_price', $key );
					$import->{'csv_' . sprintf( '%s_wholesale_price', $key )} = array_filter( $csv_data[sprintf( '%s_wholesale_price', $key )] );
					$import->log .= "<br />>>> " . sprintf( __( '%s has been detected and grouped', 'woocommerce-product-importer' ), sprintf( __( 'Wholesale Price: %s', 'woocommerce-product-importer' ), $wholesale_role['roleName'] ) );
				}
			}
		}
		unset( $wholesale_roles, $wholesale_role, $key );
	}

	// WooCommerce Multilingual - https://wordpress.org/plugins/woocommerce-multilingual/
	if( woo_pd_detect_wpml() || woo_pd_detect_import_plugin( 'wpml_wc' ) ) {
		if( isset( $csv_data['language'] ) ) {
			$import->headers[] = 'language';
			$import->csv_language = array_filter( $csv_data['language'] );
			$import->log .= "<br />>>> " . __( 'Language has been detected and grouped', 'woocommerce-product-importer' );
		}
	}

	// WooCommerce Bookings - http://www.woothemes.com/products/woocommerce-bookings/
	if( woo_pd_detect_import_plugin( 'woocommerce_bookings' ) ) {
		if( isset( $csv_data['booking_has_persons'] ) ) {
			$import->headers[] = 'booking_has_persons';
			$import->csv_booking_has_persons = array_filter( $csv_data['booking_has_persons'] );
			$import->log .= "<br />>>> " . __( 'Booking Has Persons has been detected and grouped', 'woocommerce-product-importer' );
		}
		if( isset( $csv_data['booking_has_resources'] ) ) {
			$import->headers[] = 'booking_has_resources';
			$import->csv_booking_has_resources = array_filter( $csv_data['booking_has_resources'] );
			$import->log .= "<br />>>> " . __( 'Booking Has Resources has been detected and grouped', 'woocommerce-product-importer' );
		}
		if( isset( $csv_data['booking_base_cost'] ) ) {
			$import->headers[] = 'booking_base_cost';
			$import->csv_booking_base_cost = array_filter( $csv_data['booking_base_cost'] );
			$import->log .= "<br />>>> " . __( 'Booking Base Cost has been detected and grouped', 'woocommerce-product-importer' );
		}
		if( isset( $csv_data['booking_block_cost'] ) ) {
			$import->headers[] = 'booking_block_cost';
			$import->csv_booking_block_cost = array_filter( $csv_data['booking_block_cost'] );
			$import->log .= "<br />>>> " . __( 'Booking Block Cost has been detected and grouped', 'woocommerce-product-importer' );
		}
		if( isset( $csv_data['booking_display_cost'] ) ) {
			$import->headers[] = 'booking_display_cost';
			$import->csv_booking_display_cost = array_filter( $csv_data['booking_display_cost'] );
			$import->log .= "<br />>>> " . __( 'Booking Display Cost has been detected and grouped', 'woocommerce-product-importer' );
		}
		if( isset( $csv_data['booking_requires_confirmation'] ) ) {
			$import->headers[] = 'booking_requires_confirmation';
			$import->csv_booking_requires_confirmation = array_filter( $csv_data['booking_requires_confirmation'] );
			$import->log .= "<br />>>> " . __( 'Booking Requires Confirmation has been detected and grouped', 'woocommerce-product-importer' );
		}
		if( isset( $csv_data['booking_user_can_cancel'] ) ) {
			$import->headers[] = 'booking_user_can_cancel';
			$import->csv_booking_user_can_cancel = array_filter( $csv_data['booking_user_can_cancel'] );
			$import->log .= "<br />>>> " . __( 'Booking Can Be Cancelled has been detected and grouped', 'woocommerce-product-importer' );
		}
	}

	// WooCommerce Show Single Variations - https://iconicwp.com/products/woocommerce-show-single-variations/
	if( woo_pd_detect_import_plugin( 'wc_show_single_variations' ) ) {
		if( isset( $csv_data['show_search_results'] ) ) {
			$import->headers[] = 'show_search_results';
			$import->csv_show_search_results = array_filter( $csv_data['show_search_results'] );
			$import->log .= "<br />>>> " . __( 'Show in Search Results has been detected and grouped', 'woocommerce-product-importer' );
		}
		if( isset( $csv_data['show_filtered_results'] ) ) {
			$import->headers[] = 'show_filtered_results';
			$import->csv_show_filtered_results = array_filter( $csv_data['show_filtered_results'] );
			$import->log .= "<br />>>> " . __( 'Show in Filtered Results has been detected and grouped', 'woocommerce-product-importer' );
		}
		if( isset( $csv_data['show_catalog'] ) ) {
			$import->headers[] = 'show_catalog';
			$import->csv_show_catalog = array_filter( $csv_data['show_catalog'] );
			$import->log .= "<br />>>> " . __( 'Show in Catalog has been detected and grouped', 'woocommerce-product-importer' );
		}
		if( isset( $csv_data['disable_add_to_cart'] ) ) {
			$import->headers[] = 'disable_add_to_cart';
			$import->csv_disable_add_to_cart = array_filter( $csv_data['disable_add_to_cart'] );
			$import->log .= "<br />>>> " . __( 'Disable Add to Cart has been detected and grouped', 'woocommerce-product-importer' );
		}
	}

	// WooCommerce Tab Manager - http://www.woothemes.com/products/woocommerce-tab-manager/
	if( woo_pd_detect_import_plugin( 'wc_tabmanager' ) ) {
		$product_tabs = woo_pd_get_product_tabs();
		if( !empty( $product_tabs ) ) {
			foreach( $product_tabs as $product_tab ) {
				if( !empty( $product_tab ) ) {
					if( isset( $csv_data[sprintf( 'product_tab_%s', sanitize_key( $product_tab->post_name ) )] ) ) {
						$import->headers[] = sprintf( 'product_tab_%s', sanitize_key( $product_tab->post_name ) );
						$import->{'csv_' . sprintf( 'product_tab_%s', sanitize_key( $product_tab->post_name ) )} = array_filter( $csv_data[sprintf( 'product_tab_%s', sanitize_key( $product_tab->post_name ) )] );
						$import->log .= "<br />>>> " . sprintf( __( '%s has been detected and grouped', 'woocommerce-product-importer' ), sprintf( __( 'Product Tab: %s', 'woocommerce-product-importer' ), $product_tab->post_title ) );
					}
				}
			}
		}
		unset( $product_tabs, $product_tab );
	}

/*
	if( isset( $csv_data['sample'] ) ) {
		$import->headers[] = 'sample';
		$import->csv_sample = array_filter( $csv_data['sample'] );
		$import->log .= "<br />>>> " . __( 'Sample Product Meta has been detected and grouped', 'woocommerce-product-importer' );
	}
*/

	return $import;

}
add_filter( 'woo_pd_import_addons', 'woo_pd_extend_import_addons', 10, 2 );

function woo_pd_extend_product_addons( $product, $import, $count ) {

	// WooCommerce Brands Addon - http://woothemes.com/woocommerce/
	// WooCommerce Brands - http://proword.net/Woocommerce_Brands/
	if( woo_pd_detect_product_brands() ) {
		$product->brand = null;
		if( isset( $import->csv_brand[$count] ) && isset( $import->csv_brand_1[$count] ) && isset( $import->csv_brand_2[$count] ) && isset( $import->csv_brand_3[$count] ) )
			$product->brand = $import->csv_brand[$count] . $import->parent_child_delimiter . $import->csv_brand_1[$count] . $import->parent_child_delimiter . $import->csv_brand_2[$count] . $import->parent_child_delimiter . $import->csv_brand_3[$count];
		else if( isset( $import->csv_brand[$count] ) && isset( $import->csv_brand_1[$count] ) && isset( $import->csv_brand_2[$count] ) )
			$product->brand = $import->csv_brand[$count] . $import->parent_child_delimiter . $import->csv_brand_1[$count] . $import->parent_child_delimiter . $import->csv_brand_2[$count];
		else if( isset( $import->csv_brand[$count] ) && isset( $import->csv_brand_1[$count] ) )
			$product->brand = $import->csv_brand[$count] . $import->parent_child_delimiter . $import->csv_brand_1[$count];
		else if( isset( $import->csv_brand[$count] ) )
			$product->brand = $import->csv_brand[$count];
	}

	// Advanced Google Product Feed - http://plugins.leewillis.co.uk/downloads/wp-e-commerce-product-feeds/
	if( woo_pd_detect_import_plugin( 'gpf' ) ) {
		$product->gpf_availability = ( isset( $import->csv_gpf_availability[$count] ) ? woo_pd_is_valid_gpf_availability( $import->csv_gpf_availability[$count] ) : null );
		$product->gpf_condition = ( isset( $import->csv_gpf_condition[$count] ) ? woo_pd_is_valid_gpf_condition( $import->csv_gpf_condition[$count] ) : null );
		$product->gpf_brand = ( isset( $import->csv_gpf_brand[$count] ) ? $import->csv_gpf_brand[$count] : null );
		$product->gpf_product_type = ( isset( $import->csv_gpf_product_type[$count] ) ? $import->csv_gpf_product_type[$count] : null );
		$product->gpf_google_product_category = ( isset( $import->csv_gpf_google_product_category[$count] ) ? $import->csv_gpf_google_product_category[$count] : null );
		$product->gpf_gtin = ( isset( $import->csv_gpf_gtin3[$count] ) ? $import->csv_gpf_gtin[$count] : null );
		$product->gpf_mpn = ( isset( $import->csv_gpf_mpn[$count] ) ? $import->csv_gpf_mpn[$count] : null );
		$product->gpf_gender = ( isset( $import->csv_gpf_gender[$count] ) ? woo_pd_is_valid_gpf_gender( $import->csv_gpf_gender[$count] ) : null );
		$product->gpf_age_group = ( isset( $import->csv_gpf_age_group[$count] ) ? woo_pd_is_valid_gpf_age_group( $import->csv_gpf_age_group[$count] ) : null );
		$product->gpf_color = ( isset( $import->csv_gpf_color[$count] ) ? $import->csv_gpf_color[$count] : null );
		$product->gpf_size = ( isset( $import->csv_gpf_size[$count] ) ? $import->csv_gpf_size[$count] : null );
	}

	// All in One SEO Pack - http://wordpress.org/extend/plugins/all-in-one-seo-pack/
	if( woo_pd_detect_import_plugin( 'aioseop' ) ) {
		$product->aioseop_keywords = ( isset( $import->csv_aioseop_keywords[$count] ) ? $import->csv_aioseop_keywords[$count] : null );
		$product->aioseop_description = ( isset( $import->csv_aioseop_description[$count] ) ? $import->csv_aioseop_description[$count] : null );
		$product->aioseop_title = ( isset( $import->csv_aioseop_title[$count] ) ? $import->csv_aioseop_title[$count] : null );
		$product->aioseop_titleatr = ( isset( $import->csv_aioseop_titleatr[$count] ) ? $import->csv_aioseop_titleatr[$count] : null );
		$product->aioseop_menulabel = ( isset( $import->csv_aioseop_menulabel[$count] ) ? $import->csv_aioseop_menulabel[$count] : null );
	}

	// WordPress SEO - http://yoast.com/wordpress/seo/#utm_source=wpadmin&utm_medium=plugin&utm_campaign=wpseoplugin
	if( woo_pd_detect_import_plugin( 'wpseo' ) ) {
		$product->wpseo_focuskw = ( isset( $import->csv_wpseo_focuskw[$count] ) ? $import->csv_wpseo_focuskw[$count] : null );
		$product->wpseo_metadesc = ( isset( $import->csv_wpseo_metadesc[$count] ) ? $import->csv_wpseo_metadesc[$count] : null );
		$product->wpseo_title = ( isset( $import->csv_wpseo_title[$count] ) ? $import->csv_wpseo_title[$count] : null );
		$product->wpseo_googleplus_description = ( isset( $import->csv_wpseo_googleplus_description[$count] ) ? $import->csv_wpseo_googleplus_description[$count] : null );
		$product->wpseo_opengraph_description = ( isset( $import->csv_wpseo_opengraph_description[$count] ) ? $import->csv_wpseo_opengraph_description[$count] : null );
		$product->wpseo_meta_robots_noindex = ( isset( $import->csv_wpseo_meta_robots_noindex[$count] ) ? woo_pd_is_valid_meta_robots_noindex( $import->csv_wpseo_meta_robots_noindex[$count] ) : null );
		$product->wpseo_meta_robots_nofollow = ( isset( $import->csv_wpseo_meta_robots_nofollow[$count] ) ? woo_pd_is_valid_meta_robots_nofollow( $import->csv_wpseo_meta_robots_nofollow[$count] ) : null );
		$product->wpseo_meta_robots_advs = null;
		$product->wpseo_meta_robots_adv = ( isset( $import->csv_wpseo_meta_robots_adv[$count] ) ? woo_pd_is_valid_meta_robots_adv( $import->csv_wpseo_meta_robots_adv[$count] ) : null );
		$product->wpseo_sitemap_html_include = ( isset( $import->csv_wpseo_sitemap_html_include[$count] ) ? woo_pd_is_valid_sitemap_html_include( $import->csv_wpseo_sitemap_html_include[$count] ) : null );
		$product->wpseo_authorship = ( isset( $import->csv_wpseo_authorship[$count] ) ? woo_pd_is_valid_sitemap_html_include( $import->csv_wpseo_authorship[$count] ) : null );
		$product->wpseo_canonical = ( isset( $import->csv_wpseo_canonical[$count] ) ? $import->csv_wpseo_canonical[$count] : null );
		$product->wpseo_redirect = ( isset( $import->csv_wpseo_redirect[$count] ) ? $import->csv_wpseo_redirect[$count] : null );
	}

	// Ultimate SEO - http://wordpress.org/plugins/seo-ultimate/
	if( woo_pd_detect_import_plugin( 'ultimate_seo' ) ) {
		$product->useo_meta_title = ( isset( $import->csv_useo_meta_title[$count] ) ? $import->csv_useo_meta_title[$count] : null );
		$product->useo_meta_description = ( isset( $import->csv_useo_meta_description[$count] ) ? $import->csv_useo_meta_description[$count] : null );
		$product->useo_meta_keywords = ( isset( $import->csv_useo_meta_keywords[$count] ) ? $import->csv_useo_meta_keywords[$count] : null );
		$product->useo_social_title = ( isset( $import->csv_social_title ) ? $import->csv_social_title[$count] : null );
		$product->useo_social_description = ( isset( $import->csv_useo_social_description ) ? $import->csv_useo_social_description[$count] : null );
		$product->useo_meta_noindex = ( isset( $import->csv_useo_meta_noindex ) ? woo_pd_is_valid_switch( $import->csv_useo_meta_noindex[$count] ) : null );
		$product->useo_meta_noautolinks = ( isset( $import->csv_useo_meta_noautolinks ) ? woo_pd_is_valid_switch( $import->csv_useo_meta_noautolinks[$count] ) : null );
	}

	// Cost of Goods - http://www.skyverge.com/product/woocommerce-cost-of-goods-tracking/
	if( woo_pd_detect_import_plugin( 'wc_cog' ) ) {
		$product->cost_of_goods = ( isset( $import->csv_cost_of_goods[$count] ) ? $import->csv_cost_of_goods[$count] : null );
	}

	// WooCommerce Profit of Sales Report - http://codecanyon.net/item/woocommerce-profit-of-sales-report/9190590
	if( woo_pd_detect_import_plugin( 'wc_posr' ) ) {
		$product->cost_of_goods = ( isset( $import->csv_cost_of_goods[$count] ) ? $import->csv_cost_of_goods[$count] : null );
	}

	// WooCommerce MSRP Pricing - http://www.woothemes.com/products/msrp-pricing/
	if( woo_pd_detect_import_plugin( 'wc_msrp' ) ) {
		$product->msrp = ( isset( $import->csv_msrp[$count] ) ? $import->csv_msrp[$count] : null );
	}

	// Product Vendors - http://www.woothemes.com/products/product-vendors/
	if( woo_pd_detect_import_plugin( 'vendors' ) ) {
		$product->vendor = ( isset( $import->csv_vendor[$count] ) ? $import->csv_vendor[$count] : null );
	}

	// WooCommerce Custom Fields - http://www.rightpress.net/woocommerce-custom-fields
	if( woo_pd_detect_import_plugin( 'wc_customfields' ) ) {
		if( !get_option( 'wccf_migrated_to_20' ) ) {
			// Legacy WooCommerce Custom Fields was stored in a single Option
			$options = get_option( 'rp_wccf_options' );
			if( !empty( $options ) ) {
				$custom_fields = ( isset( $options[1]['product_admin_fb_config'] ) ? $options[1]['product_admin_fb_config'] : false );
				if( !empty( $custom_fields ) ) {
					foreach( $custom_fields as $custom_field ) {
						$product->{sprintf( 'wccf_%s', sanitize_key( $custom_field['key'] ) )} = ( isset( $import->{'csv_' . sprintf( 'wccf_%s', sanitize_key( $custom_field['key'] ) )}[$count] ) ? $import->{'csv_' . sprintf( 'wccf_%s', sanitize_key( $custom_field['key'] ) )}[$count] : null );
					}
				}
			}
			unset( $options );
		} else {
			// WooCommerce Custom Fields uses CPT for Product properties
			$custom_fields = woo_pd_get_wccf_product_properties();
			if( !empty( $custom_fields ) ) {
				foreach( $custom_fields as $custom_field ) {
					$label = get_post_meta( $custom_field->ID, 'label', true );
					$key = get_post_meta( $custom_field->ID, 'key', true );
					$product->{sprintf( 'wccf_pp_%s', sanitize_key( $key ) )} = ( isset( $import->{'csv_' . sprintf( 'wccf_pp_%s', sanitize_key( $key ) )}[$count] ) ? $import->{'csv_' . sprintf( 'wccf_pp_%s', sanitize_key( $key ) )}[$count] : null );
				}
			}
			unset( $label, $key );
		}
		unset( $custom_fields, $custom_field );
	}

	// Barcodes for WooCommerce - http://www.wolkenkraft.com/produkte/barcodes-fuer-woocommerce/
	if( woo_pd_detect_import_plugin( 'wc_barcodes' ) ) {
		$product->barcode_type = ( isset( $import->csv_barcode_type[$count] ) ? $import->csv_barcode_type[$count] : null );
		$product->barcode = ( isset( $import->csv_barcode[$count] ) ? $import->csv_barcode[$count] : null );
	}

	// WooCommerce Subscriptions - http://www.woothemes.com/products/woocommerce-subscriptions/
	if( woo_pd_detect_import_plugin( 'subscriptions' ) ) {
		$product->subscription_price = ( isset( $import->csv_subscription_price[$count] ) ? $import->csv_subscription_price[$count] : null );
		$product->subscription_period_interval = ( isset( $import->csv_subscription_period_interval[$count] ) ? woo_pd_format_product_subscription_period_interval( $import->csv_subscription_period_interval[$count] ) : null );
		$product->subscription_period = ( isset( $import->csv_subscription_period[$count] ) ? $import->csv_subscription_period[$count] : null );
		$product->subscription_length = ( isset( $import->csv_subscription_length[$count] ) ? woo_pd_format_product_subscription_length( $import->csv_subscription_length[$count] ) : null );
		$product->subscription_sign_up_fee = ( isset( $import->csv_subscription_sign_up_fee[$count] ) ? $import->csv_subscription_sign_up_fee[$count] : null );
		$product->subscription_trial_length = ( isset( $import->csv_subscription_trial_length[$count] ) ? $import->csv_subscription_trial_length[$count] : null );
		$product->subscription_trial_period = ( isset( $import->csv_subscription_trial_period[$count] ) ? $import->csv_subscription_trial_period[$count] : null );
		$product->subscription_limit = ( isset( $import->csv_subscription_limit[$count] ) ? woo_pd_format_product_subscription_limit( $import->csv_subscription_limit[$count] ) : null );
	}

	// WooCommerce Ultimate Multi Currency Suite - https://codecanyon.net/item/woocommerce-ultimate-multi-currency-suite/11997014
	if( woo_pd_detect_import_plugin( 'wc_umcs' ) ) {
		$currencies = json_decode( get_option( 'wcumcs_available_currencies' ) );
		if( !empty( $currencies ) ) {
			$current_currency = ( function_exists( 'get_woocommerce_currency' ) ? get_woocommerce_currency() : false );
			foreach( $currencies as $currency_code => $currency_data ) {
				$product->{sprintf( 'wcumcs_regular_price_%s', sanitize_key( $currency_code ) )} = ( isset( $import->{'csv_' . sprintf( 'wcumcs_regular_price_%s', sanitize_key( $currency_code ) )}[$count] ) ? $import->{'csv_' . sprintf( 'wcumcs_regular_price_%s', sanitize_key( $currency_code ) )}[$count] : null );
				$product->{sprintf( 'wcumcs_sale_price_%s', sanitize_key( $currency_code ) )} = ( isset( $import->{'csv_' . sprintf( 'wcumcs_sale_price_%s', sanitize_key( $currency_code ) )}[$count] ) ? $import->{'csv_' . sprintf( 'wcumcs_sale_price_%s', sanitize_key( $currency_code ) )}[$count] : null );
			}
			unset( $currency_code, $currency_data, $current_currency );
		}
		unset( $currencies );
	}

	// WooCommerce Wholesale Prices - https://wordpress.org/plugins/woocommerce-wholesale-prices/
	if( woo_pd_detect_import_plugin( 'wc_wholesale_prices' ) ) {
		$wholesale_roles = woo_pd_get_wholesale_prices_roles();
		if( !empty( $wholesale_roles ) ) {
			foreach( $wholesale_roles as $key => $wholesale_role ) {
				$product->{sprintf( '%s_wholesale_price', $key )} = ( isset( $import->{'csv_' . sprintf( '%s_wholesale_price', $key )}[$count] ) ? $import->{'csv_' . sprintf( '%s_wholesale_price', $key )}[$count] : null );
			}
		}
		unset( $wholesale_roles, $wholesale_role, $key );
	}

	// WooCommerce Multilingual - https://wordpress.org/plugins/woocommerce-multilingual/
	if( woo_pd_detect_wpml() || woo_pd_detect_import_plugin( 'wpml_wc' ) ) {
		$product->language = ( isset( $import->csv_language[$count] ) ? woo_pd_format_wpml_language_code( $import->csv_language[$count] ) : null );
	}

	// WooCommerce Bookings - http://www.woothemes.com/products/woocommerce-bookings/
	if( woo_pd_detect_import_plugin( 'woocommerce_bookings' ) ) {
		$product->booking_has_persons = ( isset( $import->csv_booking_has_persons[$count] ) ? woo_pd_is_valid_switch( $import->csv_booking_has_persons[$count], 'boolean' ) : null );
		$product->booking_has_resources = ( isset( $import->csv_booking_has_resources[$count] ) ? woo_pd_is_valid_switch( $import->csv_booking_has_resources[$count], 'boolean' ) : null );
		$product->booking_base_cost = ( isset( $import->csv_booking_base_cost[$count] ) ? $import->csv_booking_base_cost[$count] : null );
		$product->booking_block_cost = ( isset( $import->csv_booking_block_cost[$count] ) ? $import->csv_booking_block_cost[$count] : null );
		$product->booking_display_cost = ( isset( $import->csv_booking_display_cost[$count] ) ? $import->csv_booking_display_cost[$count] : null );
		$product->booking_requires_confirmation = ( isset( $import->csv_booking_requires_confirmation[$count] ) ? woo_pd_is_valid_switch( $import->csv_booking_requires_confirmation[$count], 'boolean' ) : null );
		$product->booking_user_can_cancel = ( isset( $import->csv_booking_user_can_cancel[$count] ) ? woo_pd_is_valid_switch( $import->csv_booking_user_can_cancel[$count], 'boolean' ) : null );
	}

	// WooCommerce Show Single Variations - https://iconicwp.com/products/woocommerce-show-single-variations/
	if( woo_pd_detect_import_plugin( 'wc_show_single_variations' ) ) {
		$product->show_search_results = ( isset( $import->csv_show_search_results[$count] ) ? woo_pd_is_valid_switch( $import->csv_show_search_results[$count] ) : null );
		$product->show_filtered_results = ( isset( $import->csv_show_filtered_results[$count] ) ? woo_pd_is_valid_switch( $import->csv_show_filtered_results[$count] ) : null );
		$product->show_catalog = ( isset( $import->csv_show_catalog[$count] ) ? woo_pd_is_valid_switch( $import->csv_show_catalog[$count], 'boolean' ) : null );
		$product->disable_add_to_cart = ( isset( $import->csv_disable_add_to_cart[$count] ) ? woo_pd_is_valid_switch( $import->csv_disable_add_to_cart[$count] ) : null );
	}

	// WooCommerce Tab Manager - http://www.woothemes.com/products/woocommerce-tab-manager/
	if( woo_pd_detect_import_plugin( 'wc_tabmanager' ) ) {
		$product_tabs = woo_pd_get_product_tabs();
		if( !empty( $product_tabs ) ) {
			foreach( $product_tabs as $product_tab ) {
				if( !empty( $product_tab ) )
					$product->{sprintf( 'product_tab_%s', sanitize_key( $product_tab->post_name ) )} = ( isset( $import->{'csv_' . sprintf( 'product_tab_%s', sanitize_key( $product_tab->post_name ) )}[$count] ) ? $import->{'csv_' . sprintf( 'product_tab_%s', sanitize_key( $product_tab->post_name ) )}[$count] : null );
			}
		}
		unset( $product_tabs, $product_tab );
	}

	// $product->sample = ( isset( $import->csv_sample[$count] ) ? $import->csv_sample[$count] : null );

	return $product;

}
add_filter( 'woo_pd_product_addons', 'woo_pd_extend_product_addons', 10, 3 );

function woo_pd_extend_merge_product_defaults( $defaults ) {

	// All in One SEO Pack - http://wordpress.org/extend/plugins/all-in-one-seo-pack/
	if( woo_pd_detect_import_plugin( 'aioseop' ) ) {
		$aioseop_defaults = array(
			'aioseop_keywords' => null,
			'aioseop_description' => null,
			'aioseop_title' => null,
			'aioseop_titleatr' => null,
			'aioseop_menulabel' => null
		);
		$defaults = $defaults + $aioseop_defaults;
		unset( $aioseop_defaults );
	}

	// Advanced Google Product Feed - http://www.leewillis.co.uk/wordpress-plugins/
	if( woo_pd_detect_import_plugin( 'gpf' ) ) {
		$gpf_defaults = array(
			'gpf_availability' => null,
			'gpf_condition' => null,
			'gpf_brand' => null,
			'gpf_product_type' => null,
			'gpf_google_product_category' => null,
			'gpf_gtin' => null,
			'gpf_mpn' => null,
			'gpf_gender' => null,
			'gpf_age_group' => null,
			'gpf_color' => null,
			'gpf_size' => null
		);
		$defaults = $defaults + $gpf_defaults;
		unset( $gpf_defaults );
	}

	// WordPress SEO - http://yoast.com/wordpress/seo/#utm_source=wpadmin&utm_medium=plugin&utm_campaign=wpseoplugin
	if( woo_pd_detect_import_plugin( 'wpseo' ) ) {
		$wpseo_defaults = array(
			'wpseo_focuskw' => null,
			'wpseo_metadesc' => null,
			'wpseo_title' => null,
			'wpseo_googleplus_description' => null,
			'wpseo_opengraph_description' => null,
			'wpseo_meta_robots_noindex' => null,
			'wpseo_meta_robots_nofollow' => null,
			'wpseo_meta_robots_adv' => null,
			'wpseo_sitemap_html_include' => null,
			'wpseo_authorship' => null,
			'wpseo_canonical' => null,
			'wpseo_redirect' => null
		);
		$defaults = $defaults + $wpseo_defaults;
		unset( $wpseo_defaults );
	}

	// Ultimate SEO - http://wordpress.org/plugins/seo-ultimate/
	if( woo_pd_detect_import_plugin( 'ultimate_seo' ) ) {
		$useo_defaults = array(
			'useo_meta_title' => null,
			'useo_meta_description' => null,
			'useo_meta_keywords' => null,
			'useo_social_title' => null,
			'useo_social_description' => null,
			'useo_meta_noindex' => null,
			'useo_meta_noautolinks' => null
		);
		$defaults = $defaults + $useo_defaults;
		unset( $useo_defaults );
	}

	// MSRP Pricing - http://www.woothemes.com/products/msrp-pricing/
	if( woo_pd_detect_import_plugin( 'wc_msrp' ) ) {
		$defaults['msrp'] = null;
	}

	// WooCommerce Brands Addon - http://woothemes.com/woocommerce/
	// WooCommerce Brands - http://proword.net/Woocommerce_Brands/
	if( woo_pd_detect_product_brands() ) {
		$defaults['brand_term_id'] = array();
	}

	// Product Vendors - http://www.woothemes.com/products/product-vendors/
	if( woo_pd_detect_import_plugin( 'vendors' ) ) {
		$defaults['vendor_term_id'] = array();
	}

	// Cost of Goods - http://www.skyverge.com/product/woocommerce-cost-of-goods-tracking/
	if( woo_pd_detect_import_plugin( 'wc_cog' ) ) {
		$defaults['cost_of_goods'] = null;
	}

	// WooCommerce Profit of Sales Report - http://codecanyon.net/item/woocommerce-profit-of-sales-report/9190590
	if( woo_pd_detect_import_plugin( 'wc_posr' ) ) {
		$defaults['cost_of_goods'] = null;
	}

	// Barcodes for WooCommerce - http://www.wolkenkraft.com/produkte/barcodes-fuer-woocommerce/
	if( woo_pd_detect_import_plugin( 'wc_barcodes' ) ) {
		$defaults['barcode_type'] = null;
		$defaults['barcode'] = null;
	}

	// WooCommerce Subscriptions - http://www.woothemes.com/products/woocommerce-subscriptions/
	if( woo_pd_detect_import_plugin( 'subscriptions' ) ) {
		$defaults['subscription_price'] = null;
		$defaults['subscription_period_interval'] = null;
		$defaults['subscription_period'] = null;
		$defaults['subscription_length'] = null;
		$defaults['subscription_sign_up_fee'] = null;
		$defaults['subscription_trial_length'] = null;
		$defaults['subscription_trial_period'] = null;
		$defaults['subscription_limit'] = null;
	}

	// WooCommerce Ultimate Multi Currency Suite - https://codecanyon.net/item/woocommerce-ultimate-multi-currency-suite/11997014
	if( woo_pd_detect_import_plugin( 'wc_umcs' ) ) {
		$currencies = json_decode( get_option( 'wcumcs_available_currencies' ) );
		if( !empty( $currencies ) ) {
			$current_currency = ( function_exists( 'get_woocommerce_currency' ) ? get_woocommerce_currency() : false );
			foreach( $currencies as $currency_code => $currency_data ) {
				$defaults[sprintf( 'wcumcs_regular_price_%s', sanitize_key( $currency_code ) )] = null;
				$defaults[sprintf( 'wcumcs_sale_price_%s', sanitize_key( $currency_code ) )] = null;
			}
			unset( $currency_code, $currency_data, $current_currency );
		}
		unset( $currencies );
	}

	// WooCommerce Multilingual - https://wordpress.org/plugins/woocommerce-multilingual/
	if( woo_pd_detect_wpml() || woo_pd_detect_import_plugin( 'wpml_wc' ) ) {
		$defaults['language'] = null;
	}

	// WooCommerce Wholesale Prices - https://wordpress.org/plugins/woocommerce-wholesale-prices/
	if( woo_pd_detect_import_plugin( 'wc_wholesale_prices' ) ) {
		$wholesale_roles = woo_pd_get_wholesale_prices_roles();
		if( !empty( $wholesale_roles ) ) {
			foreach( $wholesale_roles as $key => $wholesale_role ) {
				$defaults[sprintf( '%s_wholesale_price', $key )] = null;
			}
		}
		unset( $wholesale_roles, $wholesale_role, $key );
	}

	// WooCommerce Bookings - http://www.woothemes.com/products/woocommerce-bookings/
	if( woo_pd_detect_import_plugin( 'woocommerce_bookings' ) ) {
		$defaults['booking_has_persons'] = null;
		$defaults['booking_has_resources'] = null;
		$defaults['booking_base_cost'] = null;
		$defaults['booking_block_cost'] = null;
		$defaults['booking_display_cost'] = null;
		$defaults['booking_requires_confirmation'] = null;
		$defaults['booking_user_can_cancel'] = null;
	}

	// WooCommerce Show Single Variations - https://iconicwp.com/products/woocommerce-show-single-variations/
	if( woo_pd_detect_import_plugin( 'wc_show_single_variations' ) ) {
		$defaults['show_search_results'] = null;
		$defaults['show_filtered_results'] = null;
		$defaults['show_catalog'] = null;
		$defaults['disable_add_to_cart'] = null;
	}

	// WooCommerce Tab Manager - http://www.woothemes.com/products/woocommerce-tab-manager/
	if( woo_pd_detect_import_plugin( 'wc_tabmanager' ) ) {
		$product_tabs = woo_pd_get_product_tabs();
		if( !empty( $product_tabs ) ) {
			foreach( $product_tabs as $product_tab ) {
				if( !empty( $product_tab ) )
					$defaults[sprintf( 'product_tab_%s', sanitize_key( $product_tab->post_name ) )] = null;
			}
		}
		unset( $product_tabs, $product_tab );
	}

	return $defaults;

}
add_filter( 'woo_pd_merge_product_defaults', 'woo_pd_extend_merge_product_defaults' );

// WooCommerce Custom Fields - http://www.rightpress.net/woocommerce-custom-fields
function woo_pd_get_wccf_product_properties() {

	$post_type = 'wccf_product_prop';
	$args = array(
		'post_type' => $post_type,
		'post_status' => 'publish',
		'posts_per_page' => -1
	);
	$product_fields = new WP_Query( $args );
	if( !empty( $product_fields->posts ) ) {
		return $product_fields->posts;
	}

}

// WooCommerce Subscriptions - http://www.woothemes.com/products/woocommerce-subscriptions/
function woo_pd_format_product_subscription_period_interval( $interval ) {

	$output = $interval;
	if( !empty( $interval ) ) {
		$interval = strtolower( $interval );
		switch( $interval ) {

			case 'per':
				$output = 1;
				break;

			case 'every 2nd':
				$output = 2;
				break;

			case 'every 3rd':
				$output = 3;
				break;

			case 'every 4th':
				$output = 4;
				break;

			case 'every 5th':
				$output = 5;
				break;

			case 'every 6th':
				$output = 6;
				break;

		}
	}
	return $output;

}

// WooCommerce Subscriptions - http://www.woothemes.com/products/woocommerce-subscriptions/
function woo_pd_format_product_subscription_length( $length ) {

	$output = $length;
	if( !empty( $length ) ) {
		$length = strtolower( $length );
		if( $length == 'all time' ) {
			$output = 0;
		}
	}
	return $output;

}

// WooCommerce Subscriptions - http://www.woothemes.com/products/woocommerce-subscriptions/
function woo_pd_format_product_subscription_limit( $limit ) {

	$output = $limit;
	if( !empty( $limit ) ) {
		$limit = strtolower( $limit );
		switch( $limit ) {

			case 'active subscription':
				$output = 'active';
				break;

			case 'any subscription':
				$output = 'any';
				break;

			case 'do not limit':
				$output = 'no';
				break;

		}
	}
	return $output;

}

// WooCommerce Wholesale Prices - https://wordpress.org/plugins/woocommerce-wholesale-prices/
function woo_pd_get_wholesale_prices_roles() {

	$output = false;
	$option_name = ( defined( 'WWP_OPTIONS_REGISTERED_CUSTOM_ROLES' ) ? WWP_OPTIONS_REGISTERED_CUSTOM_ROLES : 'wwp_options_registered_custom_roles' );
	$wholesale_roles = unserialize( get_option( $option_name ) );
	if( is_array( $wholesale_roles ) )
		$output = $wholesale_roles;
	unset( $wholesale_roles );
	return $output;

}

// WooCommerce Multilingual - https://wordpress.org/plugins/woocommerce-multilingual/
function woo_pd_format_wpml_language_code( $language_code ) {

	$output = $language_code;
	if( !empty( $language_code ) ) {
		$languages = ( function_exists( 'icl_get_languages' ) ? icl_get_languages( 'skip_missing=N' ) : array() );
		if( !empty( $languages ) ) {
			foreach( $languages as $key => $language ) {
				// Check if the language is a 2 character language code
				if( strlen( $language_code ) == 2 ) {
					if( strtolower( $language_code ) == $key ) {
						$output = $key;
						break;
					}
				} else {
					// Check if the language is valid
					if( sanitize_key( $language['translated_name'] ) == sanitize_key( $language_code ) ) {
						$output = $key;
						break;
					}
				}
			}
		}
	}
	return $output;

}

// WooCommerce Tab Manager - http://www.woothemes.com/products/woocommerce-tab-manager/
function woo_pd_get_product_tabs() {

	$post_type = 'wc_product_tab';
	$args = array(
		'post_type' => $post_type,
		'post_status' => 'publish',
		'posts_per_page' => -1
	);
	$product_tabs = new WP_Query( $args );
	if( !empty( $product_tabs->posts ) ) {
		return $product_tabs->posts;
	}

}
?>