<?php
function woo_pd_create_product_defaults() {

	global $product;

	$defaults = array(
		'_price' => '',
		'_sale_price' => '',
		'_sale_price_dates_from' => '',
		'_sale_price_dates_to' => '',
		'_sku' => '',
		'_weight' => '',
		'_length' => '',
		'_width' => '',
		'_height' => '',
		'_manage_stock' => 'no',
		'_stock_status' => 'instock',
		// '_stock' => '', // Causes problems with syncing Stock Status
		'_featured' => 'no',
		'_virtual' => 'no',
		'_downloadable' => 'no',
		'_product_attributes' => array(),
		'_purchase_note' => ''
	);

	if( $product->is_variation ) {
		$defaults = array_merge( $defaults, array(
			'_variation_description' => ''
		) );
	} else {
		$defaults = array_merge( $defaults, array(
			'_visibility' => 'visible',
			'_tax_status' => 'taxable',
			'_tax_class' => '',
			'_sold_individually' => '',
			'_upsell_ids' => array(),
			'_crosssell_ids' => array(),
			'_backorders' => 'no',
			'total_sales' => 0
		) );
	}
	if( $defaults = apply_filters( 'woo_pd_create_product_defaults', $defaults, $product->ID ) ) {
		if( WOO_PD_DEBUG !== true ) {
			foreach( $defaults as $key => $default )
				update_post_meta( $product->ID, $key, $default );
		}
	}

}

function woo_pd_create_product() {

	global $wpdb, $product, $import, $user_ID;

	$product->post_type = $post_type = 'product';
	// Check the Product Type
	if( $product->type !== null ) {
		// If the Product Type is Variation then change the Post Type
		if( in_array( $product->type, array( 'variation', 'product_variation' ) ) )
			$product->post_type = $post_type = 'product_variation';
	}
	$post_data = array(
		'post_author' => $user_ID,
		'post_date' => current_time( 'mysql' ),
		'post_date_gmt' => current_time( 'mysql', 1 ),
		'post_title' => ( !is_null( $product->name ) ? $product->name : '' ),
		'post_status' => 'publish',
		'comment_status' => 'closed',
		'ping_status' => 'closed',
		'post_type' => $post_type,
		'post_content' => ( !is_null( $product->description ) ? $product->description : '' ),
		'post_excerpt' => ( !is_null( $product->excerpt ) ? $product->excerpt : '' ),
	);
	// Default non-Variations to Simple Product Type
	if( $post_type == 'product' && $product->type === null ) {
		$product->type = 'simple';
	}

	remove_filter( 'post_type_link', array( 'WC_Post_Data', 'variation_post_link' ) );
	$product->ID = wp_insert_post( $post_data, true );
	add_filter( 'post_type_link', array( 'WC_Post_Data', 'variation_post_link' ) );

	if( is_wp_error( $product->ID ) !== true ) {

		// Manually refresh the Post GUID
		$wpdb->update( $wpdb->posts, array(
			'guid' => sprintf( '%s/?post_type=%s&p=%d', get_bloginfo( 'url' ), $post_type, $product->ID )
		), array( 'ID' => $product->ID ) );

		// Product Variations
		if( $product->is_variation ) {
			// Variation #1254 of Sample Variable Product
			$pieces = explode( ' - ', $product->name );
			$variable_name = $pieces[0];
			$product->name = sprintf( 'Variation #... of %s', $pieces[0] );
			$product->slug = 'product-#...-variation';
			// Get the Variable Parent ID
			$post_type = 'product';
			// get_page_by_title is limited to a single Post Type
			$products = get_page_by_title( $pieces[0], false, $post_type );
			// People do like putting dashes in Product Names...
			if( is_null( $products ) && isset( $pieces[1] ) ) {
				$variable_name = $pieces[0] . ' - ' . $pieces[1];
				$products = get_page_by_title( $variable_name, false, $post_type );
			}
			if( $products !== null ) {
				$product->post_parent = ( isset( $products->ID ) ? $products->ID : false );
				$product->slug = str_replace( '#...', $product->post_parent, $product->slug );
				if( $import->advanced_log )
					$import->log .= "<br />>>>>>> " . sprintf( __( 'Linked Variation to Variable: %s', 'woocommerce-product-importer' ), $variable_name );
				// Check if the Product Type of the Variable is infact a Variable Product; this can happen where different languages are used
				$product_type = wp_get_object_terms( $product->post_parent, 'product_type', array( 'fields' => 'slugs' ) );
				if( $product_type == 'simple' ) {
					// Detected a Simple Product that should be a Variable, lets change that
					$product_type = 'variable';
					$term_taxonomy = 'product_type';
					$term_id = term_exists( $product_type, $term_taxonomy );
					// Check if the Term exists
					if( $term_id !== null && !is_wp_error( $term_id ) ) {
						$term_id = $term_id['term_id'];
						wp_set_object_terms( $product->post_parent, absint( $term_id ), $term_taxonomy );
						if( $import->advanced_log )
							$import->log .= "<br />>>>>>> " . sprintf( __( 'Forcing Type of Variable: %s', 'woocommerce-product-importer' ), $product_type );
						else
							$import->log .= "<br />>>>>>> " . __( 'Forcing Type of Variable', 'woocommerce-product-importer' );
					} else {
						if( $import->advanced_log )
							$import->log .= "<br />>>>>>> " . __( 'Skipping forcing Type of Variable - You may need to manually adjust the Product Type for this Simple Product for linked Variations to appear', 'woocommerce-product-importer' );
					}
				}
			} else if( $import->advanced_log ) {
				$import->log .= "<br />>>>>>> " . sprintf( __( 'Could not link Variation to Variable: %s - No Post could be found by that Post Title', 'woocommerce-product-importer' ), $variable_name );
				$product->is_variation = false;
				$product->type = 'simple';
				$import->log .= "<br />>>>>>> " . __( 'Defaulting unlinked Variation back to Simple Product Type', 'woocommerce-product-importer' );
			}
			unset( $pieces );
			$product->name = str_replace( '#...', sprintf( '#%d', $product->ID ), $product->name );
			$post_data = array(
				'ID' => $product->ID,
				'post_title' => $product->name,
				'post_name' => sanitize_title_with_dashes( $product->slug )
			);
			if( WOO_PD_DEBUG !== true )
				$response = wp_update_post( $post_data, true );
			else
				$response = true;
		}

		woo_pd_create_product_defaults();
		woo_pd_create_product_details();
		if( function_exists( 'wc_delete_product_transients' ) )
			wc_delete_product_transients( $product->ID );
		if( $product->is_variation )
			$import->variations_added++;
		else
			$import->products_added++;
		$product->imported = true;

	} else {

		ob_start();
		var_dump( $post_data );
		$output = ob_get_contents();
		ob_end_clean();
		woo_pd_error_log( sprintf( '%s: Warning: %s', basename( woo_pd_get_option( 'csv' ) ), sprintf( __( 'Could not save the Product, wp_insert_post() returned false. $post_data: %s | $product: %s', 'woocommerce-product-importer' ), $output, print_r( $product, true ) ) ) );

		$product->failed_reason = __( 'Could not save the Product, wp_insert_post() returned false.', 'woocommerce-product-importer' );
		if( $errors = $product->ID->get_error_messages() ) {
			foreach( $errors as $error ) {
				$product->failed_reason = $error;
				break;
			}
		}

		$import->products_failed++;

	}

}

function woo_pd_create_product_details() {

	global $wpdb, $product, $import, $user_ID;

	woo_pd_upload_directories();

	// Product Name
	if( $product->name !== null ) {
		if( $import->advanced_log )
			$import->log .= "<br />>>>>>> " . sprintf( __( 'Setting Product Name: %s', 'woocommerce-product-importer' ), $product->name );
		else
			$import->log .= "<br />>>>>>> " . __( 'Setting Product Name', 'woocommerce-product-importer' );
	} else if( $import->advanced_log ) {
		$import->log .= "<br />>>>>>> " . __( 'Skipping Product Name', 'woocommerce-product-importer' );
	}

	// SKU
	if( $product->sku !== null ) {
		if( WOO_PD_DEBUG !== true )
			update_post_meta( $product->ID, '_sku', $product->sku );
		if( $import->advanced_log )
			$import->log .= "<br />>>>>>> " . sprintf( __( 'Setting SKU: %s', 'woocommerce-product-importer' ), $product->sku );
		else
			$import->log .= "<br />>>>>>> " . __( 'Setting SKU', 'woocommerce-product-importer' );
	} else if( $import->advanced_log ) {
		$import->log .= "<br />>>>>>> " . __( 'Skipping SKU', 'woocommerce-product-importer' );
	}

	// Description
	if( $product->description !== null ) {
		if( $product->is_variation ) {
			update_post_meta( $product->ID, '_variation_description', $product->description );
			$post_data = array(
				'ID' => $product->ID,
				'post_content' => ''
			);
			if( WOO_PD_DEBUG !== true )
				$response = wp_update_post( $post_data, true );
			else
				$response = true;
		}
		if( $import->advanced_log )
			$import->log .= "<br />>>>>>> " . sprintf( __( 'Setting Description: %s', 'woocommerce-product-importer' ), $product->description );
		else
			$import->log .= "<br />>>>>>> " . __( 'Setting Description', 'woocommerce-product-importer' );
	} else if( $import->advanced_log ) {
		$import->log .= "<br />>>>>>> " . __( 'Skipping Description', 'woocommerce-product-importer' );
	}

	// Excerpt
	if( $product->excerpt !== null ) {
		if( $import->advanced_log )
			$import->log .= "<br />>>>>>> " . sprintf( __( 'Setting Excerpt: %s', 'woocommerce-product-importer' ), $product->excerpt );
		else
			$import->log .= "<br />>>>>>> " . __( 'Setting Excerpt', 'woocommerce-product-importer' );
	} else if( $import->advanced_log ) {
		$import->log .= "<br />>>>>>> " . __( 'Skipping Excerpt', 'woocommerce-product-importer' );
	}

	// Type
	if( $product->type !== null ) {
		// Check for Virtual Product Type
		if( $product->type == 'virtual' ) {
			// Override the Product Type back to Simple and set the virtual Post meta key
			$product->type = 'simple';
			if( WOO_PD_DEBUG !== true )
				update_post_meta( $product->ID, '_virtual', 'yes' );
		}
		// Check for External Product Type
		if( $product->product_url !== null && $product->post_type == 'product' ) {
			$product->type = 'external';
		}
		// Get the Term ID for the Product Type
		$term_taxonomy = 'product_type';
		$term_id = term_exists( $product->type, $term_taxonomy );
		// Check if the Term exists
		if( $term_id !== null && !is_wp_error( $term_id ) ) {
			$term_id = $term_id['term_id'];
			wp_set_object_terms( $product->ID, absint( $term_id ), $term_taxonomy );
			if( $import->advanced_log )
				$import->log .= "<br />>>>>>> " . sprintf( __( 'Setting Type: %s', 'woocommerce-product-importer' ), $product->type );
			else
				$import->log .= "<br />>>>>>> " . __( 'Setting Type', 'woocommerce-product-importer' );
		} else {
			if( $import->advanced_log )
				$import->log .= "<br />>>>>>> " . __( 'Skipping Type', 'woocommerce-product-importer' );
		}
	} else if( $import->advanced_log ) {
		$import->log .= "<br />>>>>>> " . __( 'Skipping Type', 'woocommerce-product-importer' );
	}

	// Price
	if( $product->price !== null ) {
		if( $product->type <> 'variable' ) {
			if( WOO_PD_DEBUG !== true ) {
				update_post_meta( $product->ID, '_regular_price', wc_format_decimal( $product->price ) );
				update_post_meta( $product->ID, '_price', wc_format_decimal( $product->price ) );
			}
			if( $import->advanced_log )
				$import->log .= "<br />>>>>>> " . sprintf( __( 'Setting Price: %s', 'woocommerce-product-importer' ), $product->price );
			else
				$import->log .= "<br />>>>>>> " . __( 'Setting Price', 'woocommerce-product-importer' );
		} else {
			if( $import->advanced_log )
				$import->log .= "<br />>>>>>> " . __( 'Skipping Price, cannot set Price for Variables', 'woocommerce-product-importer' );
		}
	} else if( $import->advanced_log ) {
		$import->log .= "<br />>>>>>> " . __( 'Skipping Price', 'woocommerce-product-importer' );
	}

	// Sale Price
	if( $product->sale_price !== null ) {
		if( $product->post_type <> 'product_variation' ) {
			if( $product->type <> 'variable' ) {
				if( WOO_PD_DEBUG !== true ) {
					update_post_meta( $product->ID, '_sale_price', $product->sale_price );
					update_post_meta( $product->ID, '_price', $product->sale_price );
				}
				if( $import->advanced_log )
					$import->log .= "<br />>>>>>> " . sprintf( __( 'Setting Sale Price: %s', 'woocommerce-product-importer' ), $product->sale_price );
				else
					$import->log .= "<br />>>>>>> " . __( 'Setting Sale Price', 'woocommerce-product-importer' );
			} else {
				if( $import->advanced_log )
					$import->log .= "<br />>>>>>> " . __( 'Skipping Sale Price, cannot set Sale Price for Variables', 'woocommerce-product-importer' );
			}
		} else {
			// Check if a Price has been provided
			if( $product->price !== null ) {
				if( WOO_PD_DEBUG !== true ) {
					update_post_meta( $product->ID, '_regular_price', wc_format_decimal( $product->price ) );
					update_post_meta( $product->ID, '_sale_price', wc_format_decimal( $product->sale_price ) );
					update_post_meta( $product->ID, '_price', wc_format_decimal( $product->sale_price ) );
				}
				if( $import->advanced_log )
					$import->log .= "<br />>>>>>> " . sprintf( __( 'Setting Sale Price: %s', 'woocommerce-product-importer' ), $product->sale_price );
				else
					$import->log .= "<br />>>>>>> " . __( 'Setting Sale Price', 'woocommerce-product-importer' );
			} else {
				if( $import->advanced_log )
					$import->log .= "<br />>>>>>> " . __( 'Skipping Sale Price, cannot set Sale Price without Price for Variations', 'woocommerce-product-importer' );
			}
		}
	} else if( $import->advanced_log ) {
		$import->log .= "<br />>>>>>> " . __( 'Skipping Sale Price', 'woocommerce-product-importer' );
	}

	// Sale Price Dates From
	if( $product->sale_price_dates_from !== null ) {
		if( $product->type <> 'variable' ) {
			if( WOO_PD_DEBUG !== true )
				update_post_meta( $product->ID, '_sale_price_dates_from', $product->sale_price_dates_from );
			if( $import->advanced_log )
				$import->log .= "<br />>>>>>> " . sprintf( __( 'Setting Sale Price Dates From: %s', 'woocommerce-product-importer' ), $product->sale_price_dates_from );
			else
				$import->log .= "<br />>>>>>> " . __( 'Setting Sale Price Dates From', 'woocommerce-product-importer' );
		} else {
			if( $import->advanced_log )
				$import->log .= "<br />>>>>>> " . __( 'Skipping Sale Price Dates From, cannot set Sale Price Dates From for Variables', 'woocommerce-product-importer' );
		}
	} else if( $import->advanced_log ) {
		$import->log .= "<br />>>>>>> " . __( 'Skipping Sale Price Dates From', 'woocommerce-product-importer' );
	}

	// Sale Price Dates To
	if( $product->sale_price_dates_to !== null ) {
		if( $product->type <> 'variable' ) {
			if( WOO_PD_DEBUG !== true )
				update_post_meta( $product->ID, '_sale_price_dates_to', $product->sale_price_dates_to );
			if( $import->advanced_log )
				$import->log .= "<br />>>>>>> " . sprintf( __( 'Setting Sale Price Dates To: %s', 'woocommerce-product-importer' ), $product->sale_price_dates_to );
			else
				$import->log .= "<br />>>>>>> " . __( 'Setting Sale Price Dates To', 'woocommerce-product-importer' );
		} else {
			if( $import->advanced_log )
				$import->log .= "<br />>>>>>> " . __( 'Skipping Sale Price Dates To, cannot set Sale Price To From for Variables', 'woocommerce-product-importer' );
		}
	} else if( $import->advanced_log ) {
		$import->log .= "<br />>>>>>> " . __( 'Skipping Sale Price Dates To', 'woocommerce-product-importer' );
	}

	// Featured
	if( $product->featured !== null ) {
		if( woo_pd_detect_import_plugin( 'wc_show_single_variations' ) || ( $product->post_type <> 'product_variation' && !woo_pd_detect_import_plugin( 'wc_show_single_variations' ) ) ) {
			if( WOO_PD_DEBUG !== true )
				update_post_meta( $product->ID, '_featured', $product->featured );
			if( $import->advanced_log )
				$import->log .= "<br />>>>>>> " . sprintf( __( 'Setting Featured: %s', 'woocommerce-product-importer' ), $product->featured );
			else
				$import->log .= "<br />>>>>>> " . __( 'Setting Featured', 'woocommerce-product-importer' );
		} else {
			if( $import->advanced_log )
				$import->log .= "<br />>>>>>> " . __( 'Skipping Featured, cannot set Featured for Variations', 'woocommerce-product-importer' );
		}
	} else if( $import->advanced_log ) {
		$import->log .= "<br />>>>>>> " . __( 'Skipping Featured', 'woocommerce-product-importer' );
	}

	// Virtual
	if( $product->virtual !== null ) {
		if( WOO_PD_DEBUG !== true )
			update_post_meta( $product->ID, '_virtual', $product->virtual );
		if( $import->advanced_log )
			$import->log .= "<br />>>>>>> " . sprintf( __( 'Setting Virtual: %s', 'woocommerce-product-importer' ), $product->virtual );
		else
			$import->log .= "<br />>>>>>> " . __( 'Setting Virtual', 'woocommerce-product-importer' );
	} else if( $import->advanced_log ) {
		$import->log .= "<br />>>>>>> " . __( 'Skipping Virtual', 'woocommerce-product-importer' );
	}

	// Downloadable
	if( $product->downloadable !== null ) {
		if( WOO_PD_DEBUG !== true )
			update_post_meta( $product->ID, '_downloadable', $product->downloadable );
		if( $import->advanced_log )
			$import->log .= "<br />>>>>>> " . sprintf( __( 'Setting Downloadable: %s', 'woocommerce-product-importer' ), $product->downloadable );
		else
			$import->log .= "<br />>>>>>> " . __( 'Setting Downloadable', 'woocommerce-product-importer' );
	} else if( $import->advanced_log ) {
		$import->log .= "<br />>>>>>> " . __( 'Skipping Downloadable', 'woocommerce-product-importer' );
	}

	// Slug
	if( $product->slug !== null ) {
		$post_data = array(
			'ID' => $product->ID,
			'post_name' => sanitize_title_with_dashes( $product->slug )
		);
		if( WOO_PD_DEBUG !== true )
			$response = wp_update_post( $post_data, true );
		else
			$response = true;
		if( $import->advanced_log )
			$import->log .= "<br />>>>>>> " . sprintf( __( 'Setting Slug: %s', 'woocommerce-product-importer' ), $product->slug );
		else
			$import->log .= "<br />>>>>>> " . __( 'Setting Slug', 'woocommerce-product-importer' );
		if( is_wp_error( $response ) ) {
			if( $import->advanced_log )
				$import->log .= "<br />>>>>>>>>> " . sprintf( __( 'Could not set Slug using wp_update_post(): %s - Error: %s', 'woocommerce-product-importer' ), $product->slug, $response->get_error_message() );
			else
				$import->log .= "<br />>>>>>>>>> " . __( 'Could not set Slug using wp_update_post()', 'woocommerce-product-importer' );
			woo_pd_force_update_post( $product->ID, 'post_name', $product->slug );
			if( $import->advanced_log )
				$import->log .= "<br />>>>>>>>>> " . sprintf( __( 'Force set Slug: %s', 'woocommerce-product-importer' ), $product->slug );
			else
				$import->log .= "<br />>>>>>>>>> " . __( 'Force set Slug', 'woocommerce-product-importer' );
		}
	} else if( $import->advanced_log ) {
		$import->log .= "<br />>>>>>> " . __( 'Skipping Slug', 'woocommerce-product-importer' );
	}

	// Post Parent
	if( $product->post_parent !== null ) {
		$post_data = array(
			'ID' => $product->ID,
			'post_parent' => $product->post_parent
		);
		if( WOO_PD_DEBUG !== true )
			$response = wp_update_post( $post_data, true );
		else
			$response = true;
		if( $import->advanced_log )
			$import->log .= "<br />>>>>>> " . sprintf( __( 'Setting Post Parent: %s', 'woocommerce-product-importer' ), $product->post_parent );
		else
			$import->log .= "<br />>>>>>> " . __( 'Setting Post Parent', 'woocommerce-product-importer' );
		if( is_wp_error( $response ) ) {
			if( $import->advanced_log )
				$import->log .= "<br />>>>>>>>>> " . sprintf( __( 'Could not set Post Parent using wp_update_post(): %s - Error: %s', 'woocommerce-product-importer' ), $product->post_parent, $response->get_error_message() );
			else
				$import->log .= "<br />>>>>>>>>> " . __( 'Could not set Post Parent using wp_update_post()', 'woocommerce-product-importer' );
			woo_pd_force_update_post( $product->ID, 'post_name', $product->post_parent );
			if( $import->advanced_log )
				$import->log .= "<br />>>>>>>>>> " . sprintf( __( 'Force set Post Parent: %s', 'woocommerce-product-importer' ), $product->post_parent );
			else
				$import->log .= "<br />>>>>>>>>> " . __( 'Force set Post Parent', 'woocommerce-product-importer' );
		}
	} else if( $import->advanced_log ) {
		$import->log .= "<br />>>>>>> " . __( 'Skipping Post Parent', 'woocommerce-product-importer' );
	}

	// Weight
	if( $product->weight !== null ) {
		if( WOO_PD_DEBUG !== true )
			update_post_meta( $product->ID, '_weight', $product->weight );
		if( $import->advanced_log )
			$import->log .= "<br />>>>>>> " . sprintf( __( 'Setting Weight: %s', 'woocommerce-product-importer' ), $product->weight );
		else
			$import->log .= "<br />>>>>>> " . __( 'Setting Weight', 'woocommerce-product-importer' );
	} else if( $import->advanced_log ) {
		$import->log .= "<br />>>>>>> " . __( 'Skipping Weight', 'woocommerce-product-importer' );
	}

	// Dimensions
	if( $product->length !== null || $product->width !== null || $product->height !== null ) {
		if( $import->advanced_log )
			$import->log .= "<br />>>>>>> " . sprintf( __( 'Setting Dimensions: %d%s x %d%s x %d%s', 'woocommerce-product-importer' ), $product->height, $import->default_measurement_unit, $product->width, $import->default_measurement_unit, $product->length, $import->default_measurement_unit );
		else
			$import->log .= "<br />>>>>>> " . __( 'Setting Dimensions', 'woocommerce-product-importer' );
		if( $product->length !== null ) {
			if( WOO_PD_DEBUG !== true )
				update_post_meta( $product->ID, '_length', $product->length );
			if( $import->advanced_log )
				$import->log .= "<br />>>>>>>>>> " . sprintf( __( 'Length: %s%s', 'woocommerce-product-importer' ), $product->length, $import->default_measurement_unit );
		} else if( $import->advanced_log ) {
			$import->log .= "<br />>>>>>>>>> " . __( 'Skipping Length', 'woocommerce-product-importer' );
		}
		if( $product->width !== null ) {
			if( WOO_PD_DEBUG !== true )
				update_post_meta( $product->ID, '_width', $product->width );
			if( $import->advanced_log )
				$import->log .= "<br />>>>>>>>>> " . sprintf( __( 'Width: %s%s', 'woocommerce-product-importer' ), $product->width, $import->default_measurement_unit );
		} else if( $import->advanced_log ) {
			$import->log .= "<br />>>>>>>>>> " . __( 'Skipping Width', 'woocommerce-product-importer' );
		}
		if( $product->height !== null ) {
			if( WOO_PD_DEBUG !== true )
				update_post_meta( $product->ID, '_height', $product->height );
			if( $import->advanced_log )
				$import->log .= "<br />>>>>>>>>> " . sprintf( __( 'Height: %s%s', 'woocommerce-product-importer' ), $product->height, $import->default_measurement_unit );
		} else if( $import->advanced_log ) {
			$import->log .= "<br />>>>>>>>>> " . __( 'Skipping Height', 'woocommerce-product-importer' );
		}
	} else if( $import->advanced_log ) {
		$import->log .= "<br />>>>>>> " . __( 'Skipping Dimensions', 'woocommerce-product-importer' );
	}

	// Product Published
	if( $product->post_date !== null ) {
		$post_data = array(
			'ID' => $product->ID,
			'post_date' => $product->post_date,
			'post_date_gmt' => ( strtotime( $product->post_date ) + ( get_option( 'gmt_offset' ) * HOUR_IN_SECONDS ) )
		);
		if( WOO_PD_DEBUG !== true )
			$response = wp_update_post( $post_data, true );
		else
			$response = true;
		if( $import->advanced_log )
			$import->log .= "<br />>>>>>> " . sprintf( __( 'Setting Product Published: %s', 'woocommerce-product-importer' ), $product->post_date );
		else
			$import->log .= "<br />>>>>>> " . __( 'Setting Product Published', 'woocommerce-product-importer' );
		if( is_wp_error( $response ) ) {
			if( $import->advanced_log )
				$import->log .= "<br />>>>>>>>>> " . sprintf( __( 'Could not set Product Published using wp_update_post(): %s - Error: %s', 'woocommerce-product-importer' ), $product->post_date, $response->get_error_message() );
			else
				$import->log .= "<br />>>>>>>>>> " . __( 'Could not set Product Published using wp_update_post()', 'woocommerce-product-importer' );
			woo_pd_force_update_post( $product->ID, 'post_date', $product->post_date );
			if( $import->advanced_log )
				$import->log .= "<br />>>>>>>>>> " . sprintf( __( 'Force set Product Published: %s', 'woocommerce-product-importer' ), $product->post_date );
			else
				$import->log .= "<br />>>>>>>>>> " . __( 'Force set Product Published', 'woocommerce-product-importer' );
		}
	} else if( $import->advanced_log ) {
		$import->log .= "<br />>>>>>> " . __( 'Skipping Product Published', 'woocommerce-product-importer' );
	}

	// Product Modified
	if( $product->post_modified !== null ) {
		$post_data = array(
			'ID' => $product->ID,
			'post_modified' => $product->post_modified,
			'post_modified_gmt' => ( strtotime( $product->post_modified ) + ( get_option( 'gmt_offset' ) * HOUR_IN_SECONDS ) )
		);
		if( WOO_PD_DEBUG !== true )
			$response = wp_update_post( $post_data, true );
		else
			$response = true;
		if( $import->advanced_log )
			$import->log .= "<br />>>>>>> " . sprintf( __( 'Setting Product Modified: %s', 'woocommerce-product-importer' ), $product->post_modified );
		else
			$import->log .= "<br />>>>>>> " . __( 'Setting Product Modified', 'woocommerce-product-importer' );
		if( is_wp_error( $response ) ) {
			if( $import->advanced_log )
				$import->log .= "<br />>>>>>>>>> " . sprintf( __( 'Could not set Product Modified using wp_update_post(): %s - Error: %s', 'woocommerce-product-importer' ), $product->post_modified, $response->get_error_message() );
			else
				$import->log .= "<br />>>>>>>>>> " . __( 'Could not set Product Modified using wp_update_post()', 'woocommerce-product-importer' );
			woo_pd_force_update_post( $product->ID, 'post_modified', $product->post_modified );
			if( $import->advanced_log )
				$import->log .= "<br />>>>>>>>>> " . sprintf( __( 'Force set Product Modified: %s', 'woocommerce-product-importer' ), $product->post_modified );
			else
				$import->log .= "<br />>>>>>>>>> " . __( 'Force set Product Modified', 'woocommerce-product-importer' );
		}
	} else if( $import->advanced_log ) {
		$import->log .= "<br />>>>>>> " . __( 'Skipping Product Modified', 'woocommerce-product-importer' );
	}

	// Manage Stock
	if( $product->manage_stock !== null ) {
		if( $product->type <> 'variable' ) {
			if( WOO_PD_DEBUG !== true )
				update_post_meta( $product->ID, '_manage_stock', $product->manage_stock );
			if( $import->advanced_log )
				$import->log .= "<br />>>>>>> " . sprintf( __( 'Setting Manage Stock: %s', 'woocommerce-product-importer' ), $product->manage_stock );
			else
				$import->log .= "<br />>>>>>> " . __( 'Setting Manage Stock', 'woocommerce-product-importer' );
		} else {
			if( $import->advanced_log )
				$import->log .= "<br />>>>>>> " . __( 'Skipping Manage Stock, cannot set Manage Stock for Variables', 'woocommerce-product-importer' );
		}
	} else if( $import->advanced_log ) {
		$import->log .= "<br />>>>>>> " . __( 'Skipping Manage Stock', 'woocommerce-product-importer' );
	}

	// Stock Status
	if( $product->stock_status !== null ) {
		if( $product->type <> 'variable' ) {
			if( WOO_PD_DEBUG !== true )
				update_post_meta( $product->ID, '_stock_status', $product->stock_status );
			if( $import->advanced_log )
				$import->log .= "<br />>>>>>> " . sprintf( __( 'Setting Stock Status: %s', 'woocommerce-product-importer' ), $product->stock_status );
			else
				$import->log .= "<br />>>>>>> " . __( 'Setting Stock Status', 'woocommerce-product-importer' );
		} else {
			if( $import->advanced_log )
				$import->log .= "<br />>>>>>> " . __( 'Skipping Stock Status, cannot set Stock Status for Variables', 'woocommerce-product-importer' );
		}
	} else if( $import->advanced_log ) {
		$import->log .= "<br />>>>>>> " . __( 'Skipping Stock Status', 'woocommerce-product-importer' );
	}

	// Quantity
	if( $product->quantity !== null ) {
		if( $product->type <> 'variable' ) {
			if( WOO_PD_DEBUG !== true )
				update_post_meta( $product->ID, '_stock', $product->quantity );
			// Sync the Stock Status change for Variations
			if( $product->type == 'variation' ) {
				if( function_exists( 'wc_update_product_stock_status' ) )
					wc_update_product_stock_status( $product->ID, ( $product->stock_status !== null ? $product->stock_status : 'instock' ) );
			}
			// Override that enables Manage Stock if a value is set for Quantity (even a negative value)
			if( $product->manage_stock == null ) {
				if( WOO_PD_DEBUG !== true )
					update_post_meta( $product->ID, '_manage_stock', 'yes' );
				if( $import->advanced_log )
					$import->log .= "<br />>>>>>> " . sprintf( __( 'Force setting Manage Stock: %s, reason: %s', 'woocommerce-product-importer' ), 'Yes', 'Manage Stock must be turned on for Quantity to stick' );
				else
					$import->log .= "<br />>>>>>> " . __( 'Force setting Manage Stock', 'woocommerce-product-importer' );
			}
			// Override that forces the Stock Status if a negative Quantity is provided
			if( $product->quantity < 0 ) {
				if( WOO_PD_DEBUG !== true )
					update_post_meta( $product->ID, '_stock_status', 'outofstock' );
				if( $import->advanced_log )
					$import->log .= "<br />>>>>>> " . sprintf( __( 'Force setting Stock Status: %s, reason: %s', 'woocommerce-product-importer' ), 'Out of stock', 'Stock Status is changed to Out of stock for negative Quantities' );
				else
					$import->log .= "<br />>>>>>> " . __( 'Force setting Stock Status', 'woocommerce-product-importer' );
			}
			if( $import->advanced_log ) {
				$import->log .= "<br />>>>>>> " . sprintf( __( 'Setting Quantity: %s', 'woocommerce-product-importer' ), $product->quantity );
				if( !$product->manage_stock )
					$import->log .= "<br />>>>>>> " . sprintf( __( 'Setting Manage Stock: %s', 'woocommerce-product-importer' ), 'yes' );
			} else {
				$import->log .= "<br />>>>>>> " . __( 'Setting Quantity', 'woocommerce-product-importer' );
				if( !$product->manage_stock )
					$import->log .= "<br />>>>>>> " . __( 'Setting Manage Stock', 'woocommerce-product-importer' );
			}
		} else {
			if( $import->advanced_log )
				$import->log .= "<br />>>>>>> " . __( 'Skipping Quantity, cannot set Quantity for Variables', 'woocommerce-product-importer' );
		}
	} else if( $import->advanced_log ) {
		$import->log .= "<br />>>>>>> " . __( 'Skipping Quantity', 'woocommerce-product-importer' );
	}

	// Allow Backorders
	if( $product->allow_backorders !== null ) {
		if( WOO_PD_DEBUG !== true )
			update_post_meta( $product->ID, '_backorders', $product->allow_backorders );
		if( $import->advanced_log )
			$import->log .= "<br />>>>>>> " . sprintf( __( 'Setting Allow Backorders: %s', 'woocommerce-product-importer' ), $product->allow_backorders );
		else
			$import->log .= "<br />>>>>>> " . __( 'Setting Allow Backorders', 'woocommerce-product-importer' );
	} else if( $import->advanced_log ) {
		$import->log .= "<br />>>>>>> " . __( 'Skipping Allow Backorders', 'woocommerce-product-importer' );
	}

	// Sold Individually
	if( $product->sold_individually !== null ) {
		if( $product->post_type <> 'product_variation' ) {
			if( WOO_PD_DEBUG !== true )
				update_post_meta( $product->ID, '_sold_individually', $product->sold_individually );
			if( $import->advanced_log )
				$import->log .= "<br />>>>>>> " . sprintf( __( 'Setting Sold Individually: %s', 'woocommerce-product-importer' ), $product->sold_individually );
			else
				$import->log .= "<br />>>>>>> " . __( 'Setting Sold Individually', 'woocommerce-product-importer' );
		} else {
			if( $import->advanced_log )
				$import->log .= "<br />>>>>>> " . __( 'Skipping Sold Individually, cannot set Sold Individually for Variations', 'woocommerce-product-importer' );
		}
	} else if( $import->advanced_log ) {
		$import->log .= "<br />>>>>>> " . __( 'Skipping Sold Individually', 'woocommerce-product-importer' );
	}

	// Up-Sells
	if( $product->upsells !== null ) {
		if( $product->post_type <> 'product_variation' ) {
			if( !empty( $product->upsells ) ) {
				if( WOO_PD_DEBUG !== true )
					update_post_meta( $product->ID, '_upsell_ids', array_map( 'intval', (array)$product->upsells ) );
				if( $import->advanced_log )
					$import->log .= "<br />>>>>>> " . sprintf( __( 'Setting Up-Sells: %s', 'woocommerce-product-importer' ), $product->upsell );
				else
					$import->log .= "<br />>>>>>> " . __( 'Setting Up-Sells', 'woocommerce-product-importer' );
			} else {
				if( $import->advanced_log )
					$import->log .= "<br />>>>>>> " . sprintf( __( 'Skipping Up-Sells, could not find any matching Products: %s', 'woocommerce-product-importer' ), $product->upsell );
				else
					$import->log .= "<br />>>>>>> " . __( 'Skipping Up-Sells', 'woocommerce-product-importer' );
			}
		} else {
			if( $import->advanced_log )
				$import->log .= "<br />>>>>>> " . __( 'Skipping Up-Sells, cannot set Up-Sells for Variations', 'woocommerce-product-importer' );
		}
	} else if( $import->advanced_log ) {
		$import->log .= "<br />>>>>>> " . __( 'Skipping Up-Sells', 'woocommerce-product-importer' );
	}

	// Cross-Sells
	if( $product->crosssells !== null ) {
		if( $product->post_type <> 'product_variation' ) {
			if( !empty( $product->crosssells ) ) {
				if( WOO_PD_DEBUG !== true )
					update_post_meta( $product->ID, '_crosssell_ids', array_map( 'intval', (array)$product->crosssells ) );
				if( $import->advanced_log )
					$import->log .= "<br />>>>>>> " . sprintf( __( 'Setting Cross-Sells: %s', 'woocommerce-product-importer' ), $product->crosssell );
				else
					$import->log .= "<br />>>>>>> " . __( 'Setting Cross-Sells', 'woocommerce-product-importer' );
			} else {
				if( $import->advanced_log )
					$import->log .= "<br />>>>>>> " . sprintf( __( 'Skipping Cross-Sells, could not find any matching Products: %s', 'woocommerce-product-importer' ), $product->crosssell );
			}
		} else {
			if( $import->advanced_log )
				$import->log .= "<br />>>>>>> " . __( 'Skipping Cross-Sells, cannot set Cross-Sells for Variations', 'woocommerce-product-importer' );
			else
				$import->log .= "<br />>>>>>> " . __( 'Skipping Cross-Sells', 'woocommerce-product-importer' );
		}
	} else if( $import->advanced_log ) {
		$import->log .= "<br />>>>>>> " . __( 'Skipping Cross-Sells', 'woocommerce-product-importer' );
	}

	// Tax Status
	if( $product->tax_status !== null ) {
		if( $product->post_type <> 'product_variation' ) {
			if( WOO_PD_DEBUG !== true )
				update_post_meta( $product->ID, '_tax_status', $product->tax_status );
			if( $import->advanced_log )
				$import->log .= "<br />>>>>>> " . sprintf( __( 'Setting Tax Status: %s', 'woocommerce-product-importer' ), $product->tax_status );
			else
				$import->log .= "<br />>>>>>> " . __( 'Setting Tax Status', 'woocommerce-product-importer' );
		} else {
			if( $import->advanced_log )
				$import->log .= "<br />>>>>>> " . __( 'Skipping Tax Status, cannot set Tax Status for Variations', 'woocommerce-product-importer' );
		}
	} else if( $import->advanced_log ) {
		$import->log .= "<br />>>>>>> " . __( 'Skipping Tax Status', 'woocommerce-product-importer' );
	}

	// Tax Class
	if( $product->tax_class !== null ) {
		if( WOO_PD_DEBUG !== true )
			update_post_meta( $product->ID, '_tax_class', $product->tax_class );
		if( $import->advanced_log ) {
			// @mod - Add Tax Class details
			$import->log .= "<br />>>>>>> " . sprintf( __( 'Setting Tax Class: %s', 'woocommerce-product-importer' ), $product->tax_class );
		} else {
			$import->log .= "<br />>>>>>> " . __( 'Setting Tax Class', 'woocommerce-product-importer' );
		}
	} else if( $import->advanced_log ) {
		$import->log .= "<br />>>>>>> " . __( 'Skipping Tax Class', 'woocommerce-product-importer' );
	}

	// Sort Order
	if( $product->sort !== null ) {
		$post_data = array(
			'ID' => $product->ID,
			'menu_order' => $product->sort
		);
		if( WOO_PD_DEBUG !== true )
			$response = wp_update_post( $post_data, true );
		else
			$response = true;
		if( $import->advanced_log )
			$import->log .= "<br />>>>>>> " . sprintf( __( 'Setting Sort: %s', 'woocommerce-product-importer' ), $product->sort );
		else
			$import->log .= "<br />>>>>>> " . __( 'Setting Sort Order', 'woocommerce-product-importer' );
		if( is_wp_error( $response ) ) {
			if( $import->advanced_log )
				$import->log .= "<br />>>>>>>>>> " . sprintf( __( 'Could not set Sort Order using wp_update_post(): %s - Error: %s', 'woocommerce-product-importer' ), $product->sort, $response->get_error_message() );
			else
				$import->log .= "<br />>>>>>>>>> " . __( 'Could not set Sort Order wp_update_post()', 'woocommerce-product-importer' );
			woo_pd_force_update_post( $product->ID, 'menu_order', $product->sort );
			if( $import->advanced_log )
				$import->log .= "<br />>>>>>>>>> " . sprintf( __( 'Force set Sort Order: %s', 'woocommerce-product-importer' ), $product->sort );
			else
				$import->log .= "<br />>>>>>>>>> " . __( 'Force set Sort Order', 'woocommerce-product-importer' );
		}
	} else if( $import->advanced_log ) {
		$import->log .= "<br />>>>>>> " . __( 'Skipping Sort Order', 'woocommerce-product-importer' );
	}

	// Download File URL Path
	if( $product->download_file_url !== null ) {
		$download_files = array();
		foreach( $product->download_file_urls as $key => $download ) {
			$download_key = md5( $download );
			$download_files[$download_key] = array(
				'file' => $download,
				'name' => basename( $download )
			);
			// Override the Download File Name if provided in import
			if( $product->download_file_names !== null ) {
				if( isset( $product->download_file_names[$key] ) ) {
					if( !empty( $product->download_file_names[$key] ) )
						$download_files[$download_key]['name'] = $product->download_file_names[$key];
				}
			}
			unset( $download_key );
		}
		unset( $download );
		if( count( $download_files ) ) {
			if( WOO_PD_DEBUG !== true ) {
				// Force set as Downloadable
				update_post_meta( $product->ID, '_downloadable', 'yes' );

				update_post_meta( $product->ID, '_downloadable_files', $download_files );

				// Allow Plugin/Theme authors to turn off updating existing Orders where required
				if( apply_filters( 'woo_pd_download_files_update_existing_orders', true, $product->ID, $product->is_variation ) ) {
					// grant permission to any newly added files on any existing orders for this product prior to saving
					do_action( 'woocommerce_process_product_file_download_paths', $product->ID, ( $product->is_variation ? $product->ID : false ), $download_files );
				}
			}
			if( $import->advanced_log )
				$import->log .= "<br />>>>>>> " . sprintf( __( 'Setting Download File URL Path: %s', 'woocommerce-product-importer' ), $product->download_file_url );
			else
				$import->log .= "<br />>>>>>> " . __( 'Setting Download File URL Path', 'woocommerce-product-importer' );
		} else {
			$import->log .= "<br />>>>>>> " . __( 'Skipping Download File URL Path, no download files detected', 'woocommerce-product-importer' );
		}
		unset( $download_files );
	} else if( $import->advanced_log ) {
		$import->log .= "<br />>>>>>> " . __( 'Skipping Download File URL Path', 'woocommerce-product-importer' );
	}

	// Download Limit
	if( $product->download_limit !== null ) {
		if( WOO_PD_DEBUG !== true )
			update_post_meta( $product->ID, '_download_limit', $product->download_limit );
		if( $import->advanced_log )
			$import->log .= "<br />>>>>>> " . sprintf( __( 'Setting Download Limit: %s', 'woocommerce-product-importer' ), $product->download_limit );
		else
			$import->log .= "<br />>>>>>> " . __( 'Setting Download Limit', 'woocommerce-product-importer' );
	} else if( $import->advanced_log ) {
		$import->log .= "<br />>>>>>> " . __( 'Skipping Download Limit', 'woocommerce-product-importer' );
	}

	// Download Expiry
	if( $product->download_expiry !== null ) {
		if( WOO_PD_DEBUG !== true )
			update_post_meta( $product->ID, '_download_expiry', $product->download_expiry );
		if( $import->advanced_log )
			$import->log .= "<br />>>>>>> " . sprintf( __( 'Setting Download Expiry: %s', 'woocommerce-product-importer' ), $product->download_expiry );
		else
			$import->log .= "<br />>>>>>> " . __( 'Setting Download Expiry', 'woocommerce-product-importer' );
	} else if( $import->advanced_log ) {
		$import->log .= "<br />>>>>>> " . __( 'Skipping Download Expiry', 'woocommerce-product-importer' );
	}

	// Download Type
	if( $product->download_type !== null ) {
		if( WOO_PD_DEBUG !== true )
			update_post_meta( $product->ID, '_download_type', $product->download_type );
		if( $import->advanced_log )
			$import->log .= "<br />>>>>>> " . sprintf( __( 'Setting Download Type: %s', 'woocommerce-product-importer' ), ( $product->download_type == '' ? 'standard' : $product->download_type ) );
		else
			$import->log .= "<br />>>>>>> " . __( 'Setting Download Type', 'woocommerce-product-importer' );
	} else if( $import->advanced_log ) {
		$import->log .= "<br />>>>>>> " . __( 'Skipping Download Type', 'woocommerce-product-importer' );
	}

	// Product URL
	if( $product->product_url !== null ) {
		if( $product->post_type <> 'product_variation' ) {
			if( WOO_PD_DEBUG !== true )
				update_post_meta( $product->ID, '_product_url', $product->product_url );
			if( $import->advanced_log )
				$import->log .= "<br />>>>>>> " . sprintf( __( 'Setting Product URL: %s', 'woocommerce-product-importer' ), $product->product_url );
			else
				$import->log .= "<br />>>>>>> " . __( 'Setting Product URL', 'woocommerce-product-importer' );
		} else {
			if( $import->advanced_log )
				$import->log .= "<br />>>>>>> " . __( 'Skipping Product URL, cannot set Product URL for Variations', 'woocommerce-product-importer' );
		}
	} else if( $import->advanced_log ) {
		$import->log .= "<br />>>>>>> " . __( 'Skipping Product URL', 'woocommerce-product-importer' );
	}

	// Button Text
	if( $product->button_text !== null ) {
		// Check this isn't a Variation
		if( $product->post_type <> 'product_variation' ) {
			if( WOO_PD_DEBUG !== true )
				update_post_meta( $product->ID, '_button_text', $product->button_text );
			if( $import->advanced_log )
				$import->log .= "<br />>>>>>> " . sprintf( __( 'Setting Button Text: %s', 'woocommerce-product-importer' ), $product->button_text );
			else
				$import->log .= "<br />>>>>>> " . __( 'Setting Button Text', 'woocommerce-product-importer' );
		} else {
			if( $import->advanced_log )
				$import->log .= "<br />>>>>>> " . __( 'Skipping Button Text, cannot set Button Text for Variations', 'woocommerce-product-importer' );
		}
	} else if( $import->advanced_log ) {
		$import->log .= "<br />>>>>>> " . __( 'Skipping Button Text', 'woocommerce-product-importer' );
	}

	// Visibility
	if( $product->visibility !== null ) {
		// Check this isn't a Variation
		if( $product->post_type <> 'product_variation' ) {
			if( WOO_PD_DEBUG !== true )
				update_post_meta( $product->ID, '_visibility', $product->visibility );
			if( $import->advanced_log )
				$import->log .= "<br />>>>>>> " . sprintf( __( 'Setting Visibility: %s', 'woocommerce-product-importer' ), $product->visibility );
			else
				$import->log .= "<br />>>>>>> " . __( 'Setting Visibility', 'woocommerce-product-importer' );
		} else {
			if( $import->advanced_log && woo_pd_detect_import_plugin( 'wc_show_single_variations' ) )
				$import->log .= "<br />>>>>>> " . __( 'Skipping Visibility, see Show Single Variations details', 'woocommerce-product-importer' );
			else if( $import->advanced_log )
				$import->log .= "<br />>>>>>> " . __( 'Skipping Visibility, cannot set Visibility for Variations', 'woocommerce-product-importer' );
		}
	} else if( $import->advanced_log ) {
		$import->log .= "<br />>>>>>> " . __( 'Skipping Visibility', 'woocommerce-product-importer' );
	}

	// Product Status
	if( $product->status !== null ) {
		// Check this isn't a Variation
		if( $product->post_type <> 'product_variation' ) {
			$post_data = array(
				'ID' => $product->ID,
				'post_status' => $product->status
			);
			if( WOO_PD_DEBUG !== true )
				$response = wp_update_post( $post_data, true );
			else
				$response = true;
			if( $import->advanced_log )
				$import->log .= "<br />>>>>>> " . sprintf( __( 'Setting Product Status: %s', 'woocommerce-product-importer' ), $product->status );
			else
				$import->log .= "<br />>>>>>> " . __( 'Setting Product Status', 'woocommerce-product-importer' );
			if( is_wp_error( $response ) ) {
				if( $import->advanced_log )
					$import->log .= "<br />>>>>>>>>> " . sprintf( __( 'Could not set Product Status using wp_update_post(): %s - Error: %s', 'woocommerce-product-importer' ), $product->status, $response->get_error_message() );
				else
					$import->log .= "<br />>>>>>>>>> " . __( 'Could not set Product Status using wp_update_post()', 'woocommerce-product-importer' );
				woo_pd_force_update_post( $product->ID, 'post_status', $product->status );
				if( $import->advanced_log )
					$import->log .= "<br />>>>>>>>>> " . sprintf( __( 'Force set Product Status: %s', 'woocommerce-product-importer' ), $product->status );
				else
					$import->log .= "<br />>>>>>>>>> " . __( 'Force set Product Status', 'woocommerce-product-importer' );
			}
		} else {
			if( $import->advanced_log )
				$import->log .= "<br />>>>>>> " . __( 'Skipping Product Status, cannot set Product Status for Variations', 'woocommerce-product-importer' );
		}
	} else if( $import->advanced_log ) {
		$import->log .= "<br />>>>>>> " . __( 'Skipping Product Status', 'woocommerce-product-importer' );
	}

	// Enable Reviews
	if( $product->comment_status !== null ) {
		// Check this isn't a Variation
		if( $product->post_type <> 'product_variation' ) {
			$post_data = array(
				'ID' => $product->ID,
				'comment_status' => $product->comment_status
			);
			if( WOO_PD_DEBUG !== true )
				$response = wp_update_post( $post_data, true );
			else
				$response = true;
			if( $import->advanced_log )
				$import->log .= "<br />>>>>>> " . sprintf( __( 'Setting Enable Reviews: %s', 'woocommerce-product-importer' ), $product->comment_status );
			else
				$import->log .= "<br />>>>>>> " . __( 'Setting Enable Reviews', 'woocommerce-product-importer' );
			if( is_wp_error( $response ) ) {
				if( $import->advanced_log )
					$import->log .= "<br />>>>>>>>>> " . sprintf( __( 'Could not set Enable Reviews using wp_update_post(): %s - Error: %s', 'woocommerce-product-importer' ), $product->comment_status, $response->get_error_message() );
				else
					$import->log .= "<br />>>>>>>>>> " . __( 'Could not set Enable Reviews using wp_update_post()', 'woocommerce-product-importer' );
				woo_pd_force_update_post( $product->ID, 'comment_status', $product->comment_status );
				if( $import->advanced_log )
					$import->log .= "<br />>>>>>>>>> " . sprintf( __( 'Force set Enable Reviews: %s', 'woocommerce-product-importer' ), $product->comment_status );
				else
					$import->log .= "<br />>>>>>>>>> " . __( 'Force set Enable Reviews', 'woocommerce-product-importer' );
			}
		} else {
			if( $import->advanced_log )
				$import->log .= "<br />>>>>>> " . __( 'Skipping Enable Reviews, cannot set Enable Reviews for Variations', 'woocommerce-product-importer' );
		}
	} else if( $import->advanced_log ) {
		$import->log .= "<br />>>>>>> " . __( 'Skipping Enable Reviews', 'woocommerce-product-importer' );
	}

	// Purchase Note
	if( $product->purchase_note !== null ) {
		// Check this isn't a Variation
		if( $product->post_type <> 'product_variation' ) {
			if( WOO_PD_DEBUG !== true )
				update_post_meta( $product->ID, '_purchase_note', $product->purchase_note );
			if( $import->advanced_log )
				$import->log .= "<br />>>>>>> " . sprintf( __( 'Setting Purchase Note: %s', 'woocommerce-product-importer' ), $product->purchase_note );
			else
				$import->log .= "<br />>>>>>> " . __( 'Setting Purchase Note', 'woocommerce-product-importer' );
		} else {
			if( $import->advanced_log )
				$import->log .= "<br />>>>>>> " . __( 'Skipping Purchase Note, cannot set Purchase Note for Variations', 'woocommerce-product-importer' );
		}
	} else if( $import->advanced_log ) {
		$import->log .= "<br />>>>>>> " . __( 'Skipping Purchase Note', 'woocommerce-product-importer' );
	}

	// Total Sales
	if( $product->total_sales !== null ) {
		if( WOO_PD_DEBUG !== true )
			update_post_meta( $product->ID, 'total_sales', $product->total_sales );
		if( $import->advanced_log )
			$import->log .= "<br />>>>>>> " . sprintf( __( 'Setting Total Sales: %s', 'woocommerce-product-importer' ), $product->total_sales );
		else
			$import->log .= "<br />>>>>>> " . __( 'Setting Total Sales', 'woocommerce-product-importer' );
	} else if( $import->advanced_log ) {
		$import->log .= "<br />>>>>>> " . __( 'Skipping Total Sales', 'woocommerce-product-importer' );
	}

	// Product Attributes
	if( $product->attributes !== null ) {
		if( !empty( $product->attributes ) ) {
			$i = 1;
			$attributes = array();
			foreach( $import->attributes as $attribute ) {
				if( isset( $product->attributes[$attribute['name']] ) && $product->attributes[$attribute['name']] ) {
					switch( $attribute['type'] ) {

						// Type: Select
						case 'select':
						default:
							// Check for default Category separator
							$attribute_values = explode( $import->category_separator, $product->attributes[$attribute['name']] );
							// Check for non-standard comma separator
							if( count( $attribute_values ) == 1 && strpos( $product->attributes[$attribute['name']], ',' ) !== false ) {

								// Allow Plugin/Theme authors to change the Product Attribute value separator
								$value_separator = apply_filters( 'woo_pd_product_attribute_value_separator', ',', $attribute['name'] );

								$attribute_values = explode( $value_separator, $product->attributes[$attribute['name']] );
								unset( $value_separator );
							}
							break;

						// Type: Text
						case 'text':
							$attribute_values = array( $product->attributes[$attribute['name']] );
							break;

					}
					if( !empty( $attribute_values ) ) {

						$import->log .= "<br />>>>>>> " . __( 'Scanning Product for Attributes...', 'woocommerce-product-importer' );

						for( $j = 0; $j < count( $attribute_values ); $j++ ) {
							// Check for an existing Term linked to this Attribute
							$term_taxonomy = sprintf( 'pa_%s', sanitize_title_with_dashes( $attribute['name'] ) );
							$attribute_value = apply_filters( 'pre_term_name', $attribute_values[$j], $term_taxonomy );
							if( $attribute_data = get_term_by( 'name', trim( $attribute_value ), $term_taxonomy ) ) {
								$attribute['term_id'] = $attribute_data->term_id;
								$attribute['term_taxonomy_id'] = $attribute_data->term_taxonomy_id;
								$attribute['slug'] = $attribute_data->slug;
								if( $import->advanced_log )
									$import->log .= "<br />>>>>>>>>> " . sprintf( __( 'Existing Term \'%s\' detected and linked to Attribute \'%s\'', 'woocommerce-product-importer' ), $attribute_values[$j], $attribute['label'] );
							} else {
								$attribute_data = wp_insert_term(
									trim( $attribute_values[$j] ),
									sprintf( 'pa_%s', sanitize_title_with_dashes( $attribute['name'] ) ),
									array(
										'parent' => 0
									)
								);
								if( $attribute_data && !is_wp_error( $attribute_data ) ) {
									$attribute['term_id'] = $attribute_data['term_id'];
									$attribute['term_taxonomy_id'] = $attribute_data['term_taxonomy_id'];
									delete_woocommerce_term_meta( $attribute['term_id'], sprintf( 'order_pa_%s', sanitize_title_with_dashes( $attribute['name'] ) ) );
									update_woocommerce_term_meta( $attribute['term_id'], sprintf( 'order_pa_%s', sanitize_title_with_dashes( $attribute['name'] ) ), $i );
									if( $import->advanced_log )
										$import->log .= "<br />>>>>>>>>> " . sprintf( __( 'Created new Term linked to Attribute %s', 'woocommerce-product-importer' ), $attribute['label'] );
								} else {
									$import->log .= "<br />>>>>>>>>> " . sprintf( __( 'Could not create Term linked to Attribute %s, response: %s', 'woocommerce-product-importer' ), $attribute['label'], $attribute_data->get_error_message() );
									continue;
								}
							}
							if( $attribute['term_taxonomy_id'] ) {
								$wpdb->insert( $wpdb->term_relationships, array(
									'object_id' => $product->ID,
									'term_taxonomy_id' => $attribute['term_taxonomy_id']
								) );
								wp_update_term_count( $attribute['term_taxonomy_id'], sprintf( 'pa_%s', sanitize_title_with_dashes( $attribute['name'] ) ) );
							}
							unset( $attribute_data );
						}
						$attributes[sprintf( 'pa_%s', sanitize_title_with_dashes( $attribute['name'] ) )] = array(
							'name' => sprintf( 'pa_%s', sanitize_title_with_dashes( $attribute['name'] ) ),
							'value' => null,
							'position' => $i,
							'is_visible' => 1,
							'is_variation' => ( $product->type == 'variable' ) ? 1 : 0,
							'is_taxonomy' => 1
						);
						// Save the Attribute detail against Variations
						if( $product->is_variation ) {
							if( WOO_PD_DEBUG !== true )
								update_post_meta( $product->ID, sprintf( 'attribute_pa_%s', sanitize_title_with_dashes( $attribute['name'] ) ), $attribute['slug'] ); 
						}
						if( $import->advanced_log )
							$import->log .= "<br />>>>>>> " . sprintf( __( 'Setting Attribute: %s to %s', 'woocommerce-product-importer' ), $attribute['label'], implode( '|', $attribute_values ) );
						else
							$import->log .= "<br />>>>>>> " . sprintf( __( 'Setting Attribute: %s', 'woocommerce-product-importer' ), $attribute['label'] );

					} else if( $import->advanced_log ) {
						$import->log .= "<br />>>>>>> " . sprintf( __( 'Skipping Attribute: %s', 'woocommerce-product-importer' ), $attribute['label'] );
					}
					unset( $attribute_values );
				}
				$i++;
			}
			// Save the Attribute details against the Variable Product
			if( !empty( $attributes ) && !$product->is_variation ) {
				if( WOO_PD_DEBUG !== true )
					update_post_meta( $product->ID, '_product_attributes', $attributes );
			}
			unset( $attributes, $attribute );
		} else if( $import->advanced_log ) {
			$import->log .= "<br />>>>>>> " . __( 'Skipping Attributes', 'woocommerce-product-importer' );
		}
	} else if( $import->advanced_log ) {
		$import->log .= "<br />>>>>>> " . __( 'Skipping Attributes', 'woocommerce-product-importer' );
	}

	// Default Attributes
	if( $product->default_attributes !== null ) {
		if( $product->type == 'variable' && !empty( $product->attributes ) ) {
			// Check for default Category separator
			$attribute_values = explode( $import->category_separator, $product->default_attributes );
			// Check for non-standard comma separator
			if( count( $attribute_values ) == 1 && strpos( $product->default_attributes, ',' ) !== false ) {

				// Allow Plugin/Theme authors to change the Product Attribute value separator
				$value_separator = apply_filters( 'woo_pd_product_attribute_value_separator', ',', 'default_attributes' );

				$attribute_values = explode( $value_separator, $product->default_attributes );
				unset( $value_separator );
			}
			$default_attributes = array();
			$default_attributes_log = '';
			for( $j = 0; $j < count( $attribute_values ); $j++ ) {
				$pieces = explode( ':', $attribute_values[$j] );
				foreach( $import->attributes as $attribute ) {
					// Check for the matching Attribute Taxonomy
					if( $attribute['label'] == $pieces[0] ) {
						// Check for the matching Attribute Term
						$attribute_slug = woo_pd_get_product_attribute_slug_by_name( ltrim( $pieces[1] ), sprintf( 'pa_%s', $attribute['name'] ) );
						if( !empty( $attribute_slug ) ) {
							$default_attributes[sprintf( 'pa_%s', $attribute['name'] )] = $attribute_slug;
							$default_attributes_log .= sprintf( '%s: %s', $attribute['label'], ltrim( $pieces[1] ) ) . $import->category_separator;
						}
						unset( $attribute_slug );
						break;
					}
				}
			}
			unset( $pieces, $attribute_values );
			if( !empty( $default_attributes ) ) {
				$default_attributes_log = substr( $default_attributes_log, 0, -1 );
				update_post_meta( $product->ID, '_default_attributes', $default_attributes );
				if( $import->advanced_log )
					$import->log .= "<br />>>>>>> " . sprintf( __( 'Setting Default Attributes: %s', 'woocommerce-product-importer' ), $default_attributes_log );
				else
					$import->log .= "<br />>>>>>> " . __( 'Setting Default Attributes', 'woocommerce-product-importer' );
			} else {
				if( $import->advanced_log )
					$import->log .= "<br />>>>>>> " . __( 'Skipping Default Attributes', 'woocommerce-product-importer' );
			}
			unset( $default_attributes_log );
		} else {
			if( $import->advanced_log )
				$import->log .= "<br />>>>>>> " . __( 'Skipping Default Attributes, can only set Default Attributes for Variables', 'woocommerce-product-importer' );
		}
	} else if( $import->advanced_log ) {
		$import->log .= "<br />>>>>>> " . __( 'Skipping Default Attributes', 'woocommerce-product-importer' );
	}

	// Category
	if( $product->category !== null && isset( $product->category_term_id ) ) {
		if( $product->post_type <> 'product_variation' ) {
			$term_taxonomy = 'product_cat';
			if( WOO_PD_DEBUG !== true )
				$term_taxonomy_ids = wp_set_object_terms( $product->ID, array_unique( array_map( 'intval', $product->category_term_id ) ), $term_taxonomy );
			if( $import->advanced_log ) {
				if( count( $product->category_term_id ) == 1 )
					$import->log .= "<br />>>>>>> " . sprintf( __( 'Linking Category: %s', 'woocommerce-product-importer' ), $product->category );
				else
					$import->log .= "<br />>>>>>> " . sprintf( __( 'Linking Categories: %s', 'woocommerce-product-importer' ), $product->category );
			} else {
				if( count( $product->category_term_id ) == 1 )
					$import->log .= "<br />>>>>>> " . __( 'Linking Category', 'woocommerce-product-importer' );
				else
					$import->log .= "<br />>>>>>> " . __( 'Linking Categories', 'woocommerce-product-importer' );
			}
		} else {
			if( $import->advanced_log )
				$import->log .= "<br />>>>>>> " . __( 'Skipping Category, cannot set Category for Variations', 'woocommerce-product-importer' );
		}
	} else if( $import->advanced_log ) {
		$import->log .= "<br />>>>>>> " . __( 'Skipping Category', 'woocommerce-product-importer' );
	}

	// Tag
	if( $product->tag !== null && isset( $product->tag_term_id ) ) {
		if( $product->post_type <> 'product_variation' ) {
			$term_taxonomy = 'product_tag';
			if( WOO_PD_DEBUG !== true )
				$term_taxonomy_ids = wp_set_object_terms( $product->ID, array_unique( array_map( 'intval', $product->tag_term_id ) ), $term_taxonomy );
			if( $import->advanced_log ) {
				if( count( $product->tag_term_id ) == 1 )
					$import->log .= "<br />>>>>>> " . sprintf( __( 'Linking Tag: %s', 'woocommerce-product-importer' ), $product->tag );
				else
					$import->log .= "<br />>>>>>> " . sprintf( __( 'Linking Tags: %s', 'woocommerce-product-importer' ), $product->tag );
			} else {
				if( count( $product->tag_term_id ) == 1 )
					$import->log .= "<br />>>>>>> " . __( 'Linking Tag', 'woocommerce-product-importer' );
				else
					$import->log .= "<br />>>>>>> " . __( 'Linking Tags', 'woocommerce-product-importer' );
			}
		} else {
			if( $import->advanced_log )
				$import->log .= "<br />>>>>>> " . __( 'Skipping Tag, cannot set Tag for Variations', 'woocommerce-product-importer' );
		}
	} else if( $import->advanced_log ) {
		$import->log .= "<br />>>>>>> " . __( 'Skipping Tag', 'woocommerce-product-importer' );
	}

	// Shipping Class
	if( !empty( $product->shipping_class_term_id ) ) {
		$term_taxonomy = 'product_shipping_class';
		if( WOO_PD_DEBUG !== true )
			$term_taxonomy_ids = wp_set_object_terms( $product->ID, array_unique( array_map( 'intval', $product->shipping_class_term_id ) ), $term_taxonomy );
		if( $import->advanced_log ) {
			if( count( $product->shipping_class_term_id ) == 1 )
				$import->log .= "<br />>>>>>> " . sprintf( __( 'Linking Shipping Class: %s', 'woocommerce-product-importer' ), $product->shipping_class );
			else
				$import->log .= "<br />>>>>>> " . sprintf( __( 'Linking Shipping Classes: %s', 'woocommerce-product-importer' ), $product->shipping_class );
		} else {
			if( count( $product->shipping_class_term_id ) == 1 )
				$import->log .= "<br />>>>>>> " . __( 'Linking Shipping Class', 'woocommerce-product-importer' );
			else
				$import->log .= "<br />>>>>>> " . __( 'Linking Shipping Classes', 'woocommerce-product-importer' );
		}
	} else if( $import->advanced_log ) {
		$import->log .= "<br />>>>>>> " . __( 'Skipping Shipping Class', 'woocommerce-product-importer' );
	}

	// Image
	if( !empty( $import->image_method ) ) {
		if( ( isset( $product->images ) && $product->images !== null ) && $product->ID ) {
			if( $import->advanced_log )
				$import->log .= "<br />>>>>>> " . sprintf( __( 'Setting Image: %s', 'woocommerce-product-importer' ), ltrim( implode( '|', $product->images ), '/' ) );
			else
				$import->log .= "<br />>>>>>> " . __( 'Setting Image', 'woocommerce-product-importer' );
			$j = 0;
			switch( $import->image_method ) {

				// Image import method: External (e.g. http://www.domain.com/product_images/product-image-1.jpg)
				case 'external':
					if( isset( $_SERVER['HTTP_USER_AGENT'] ) )
						ini_set( 'user_agent', $_SERVER['HTTP_USER_AGENT'] );
					foreach( $product->images as $image ) {
						$image_filename = '';
						if( is_numeric( $image ) ) {
							// Check if the Attachment is a valid image
							if( wp_attachment_is_image( $image ) ) {
								$image_id = $image;
								// Assign first image as Featured Image
								if( $j == 0 ) {
									$import->log .= "<br />>>>>>>>>> " . sprintf( __( 'Featured Image has been set to Attachment #%d', 'woocommerce-product-importer' ), $image_id );
									if( WOO_PD_DEBUG !== true )
										update_post_meta( $product->ID, '_thumbnail_id', $image_id );
								}
								continue;
							}
						}
						if( woo_pd_http_file_exists( $image ) )
							$image_filename = woo_pd_download_product_image( $image );
						else
							$import->log .= "<br />>>>>>>>>> " . sprintf( __( 'Skipped Image download: %s - No Image could be found at that URL', 'woocommerce-product-importer' ), $image );
						if( $image_filename ) {
							if( strstr( $image_filename, '/' ) )
								$image_id = woo_pd_create_product_image( $image_filename, true );
							else
								$image_id = woo_pd_create_product_image( $image_filename );
							if( $image_id ) {
								if( WOO_PD_DEBUG !== true )
									update_post_meta( $image_id, '_original_url', $image );
								// Assign first image as Featured Image
								if( $j == 0 ) {
									$import->log .= "<br />>>>>>>>>> " . sprintf( __( 'Featured image has been set to %s', 'woocommerce-product-importer' ), basename( $image_filename ) );
									if( $product->featured_image_title !== null ) {
										if( $import->advanced_log )
											$import->log .= "<br />>>>>>> " . sprintf( __( 'Setting Featured Image Title: %s', 'woocommerce-product-importer' ), $product->featured_image_title );
										else
											$import->log .= "<br />>>>>>> " . __( 'Setting Featured Image Title', 'woocommerce-product-importer' );
									} else if( $import->advanced_log ) {
										$import->log .= "<br />>>>>>> " . __( 'Skipping Featured Image Title', 'woocommerce-product-importer' );
									}
									if( $product->featured_image_caption !== null ) {
										if( $import->advanced_log )
											$import->log .= "<br />>>>>>> " . sprintf( __( 'Setting Featured Image Caption: %s', 'woocommerce-product-importer' ), $product->featured_image_caption );
										else
											$import->log .= "<br />>>>>>> " . __( 'Setting Featured Image Caption', 'woocommerce-product-importer' );
									} else if( $import->advanced_log ) {
										$import->log .= "<br />>>>>>> " . __( 'Skipping Featured Image Caption', 'woocommerce-product-importer' );
									}
									if( $product->featured_image_alt_text !== null ) {
										if( $import->advanced_log )
											$import->log .= "<br />>>>>>> " . sprintf( __( 'Setting Featured Image Alternative Text: %s', 'woocommerce-product-importer' ), $product->featured_image_alt_text );
										else
											$import->log .= "<br />>>>>>> " . __( 'Setting Featured Image Alternative Text', 'woocommerce-product-importer' );
									} else if( $import->advanced_log ) {
										$import->log .= "<br />>>>>>> " . __( 'Skipping Featured Image Alternative Text', 'woocommerce-product-importer' );
									}
									if( $product->featured_image_description !== null ) {
										if( $import->advanced_log )
											$import->log .= "<br />>>>>>> " . sprintf( __( 'Setting Featured Image Description: %s', 'woocommerce-product-importer' ), $product->featured_image_description );
										else
											$import->log .= "<br />>>>>>> " . __( 'Setting Featured Image Description', 'woocommerce-product-importer' );
									} else if( $import->advanced_log ) {
										$import->log .= "<br />>>>>>> " . __( 'Skipping Featured Image Description', 'woocommerce-product-importer' );
									}
									if( WOO_PD_DEBUG !== true )
										update_post_meta( $product->ID, '_thumbnail_id', $image_id );
								}
							}
						}
						$j++;
						unset( $image, $image_filename );
					}
					break;

				// Image import method: Manual upload (e.g. image-1.jpg)
				case 'csv':
					foreach( $product->images as $image ) {
						$image_filename = '';
						// Check if the image is a WordPress Media ID
						if( is_numeric( $image ) ) {
							// Check if the Attachment is a valid image
							if( wp_attachment_is_image( $image ) ) {
								$image_id = $image;
								// Assign first image as Featured Image
								if( $j == 0 ) {
									$import->log .= "<br />>>>>>>>>> " . sprintf( __( 'Featured Image has been set to Attachment #%d', 'woocommerce-product-importer' ), $image_id );
									if( WOO_PD_DEBUG !== true )
										update_post_meta( $product->ID, '_thumbnail_id', $image_id );
								}
								continue;
							}
						}
						$image = woo_pd_format_path( $image );
						// Check that the image file path exists
						if( $import->image_filename_method == 'uploads' ) {
							if( file_exists( woo_pd_format_path( $import->uploads_basedir . $image ) ) ) {
								$image_filename = $image;
							} else {
								$import->log .= "<br />>>>>>>>>> " . sprintf( __( 'Skipped Image download: %s - No Image could be found matching that relative file path', 'woocommerce-product-importer' ), woo_pd_format_path( $image ) );
							}
						} else if( $import->image_filename_method == 'base' ) {
							if( file_exists( woo_pd_format_path( ABSPATH . $image ) ) ) {
								$image_filename = $image;
							} else {
								$import->log .= "<br />>>>>>>>>> " . sprintf( __( 'Skipped Image download: %s - No Image could be found matching that file path', 'woocommerce-product-importer' ), woo_pd_format_path( ABSPATH . $image ) );
							}
						}
						if( !empty( $image_filename ) ) {
							if( $image_id = woo_pd_create_product_image( $image ) ) {
								if( WOO_PD_DEBUG !== true )
									update_post_meta( $image_id, '_original_url', $image );
								// Assign first image as Featured Image
								if( $j == 0 ) {
									$import->log .= "<br />>>>>>>>>> " . sprintf( __( 'Featured Image has been set to %s', 'woocommerce-product-importer' ), basename( $image_filename ) );
									if( $product->featured_image_title !== null ) {
										if( $import->advanced_log )
											$import->log .= "<br />>>>>>> " . sprintf( __( 'Setting Featured Image Title: %s', 'woocommerce-product-importer' ), $product->featured_image_title );
										else
											$import->log .= "<br />>>>>>> " . __( 'Setting Featured Image Title', 'woocommerce-product-importer' );
									} else if( $import->advanced_log ) {
										$import->log .= "<br />>>>>>> " . __( 'Skipping Featured Image Title', 'woocommerce-product-importer' );
									}
									if( $product->featured_image_caption !== null ) {
										if( $import->advanced_log )
											$import->log .= "<br />>>>>>> " . sprintf( __( 'Setting Featured Image Caption: %s', 'woocommerce-product-importer' ), $product->featured_image_caption );
										else
											$import->log .= "<br />>>>>>> " . __( 'Setting Featured Image Caption', 'woocommerce-product-importer' );
									} else if( $import->advanced_log ) {
										$import->log .= "<br />>>>>>> " . __( 'Skipping Featured Image Caption', 'woocommerce-product-importer' );
									}
									if( $product->featured_image_alt_text !== null ) {
										if( $import->advanced_log )
											$import->log .= "<br />>>>>>> " . sprintf( __( 'Setting Featured Image Alternative Text: %s', 'woocommerce-product-importer' ), $product->featured_image_alt_text );
										else
											$import->log .= "<br />>>>>>> " . __( 'Setting Featured Image Alternative Text', 'woocommerce-product-importer' );
									} else if( $import->advanced_log ) {
										$import->log .= "<br />>>>>>> " . __( 'Skipping Featured Image Alternative Text', 'woocommerce-product-importer' );
									}
									if( $product->featured_image_description !== null ) {
										if( $import->advanced_log )
											$import->log .= "<br />>>>>>> " . sprintf( __( 'Setting Featured Image Description: %s', 'woocommerce-product-importer' ), $product->featured_image_description );
										else
											$import->log .= "<br />>>>>>> " . __( 'Setting Featured Image Description', 'woocommerce-product-importer' );
									} else if( $import->advanced_log ) {
										$import->log .= "<br />>>>>>> " . __( 'Skipping Featured Image Description', 'woocommerce-product-importer' );
									}
									if( WOO_PD_DEBUG !== true )
										update_post_meta( $product->ID, '_thumbnail_id', $image_id );
								}
							}
						}
						$j++;
						unset( $image, $image_filename );
					}
					break;

			}
			unset( $image_filename, $image_id );
		} else if( $import->advanced_log ) {
			$import->log .= "<br />>>>>>> " . __( 'Skipping Featured Image', 'woocommerce-product-importer' );
		}
	} else if( $import->advanced_log ) {
		$import->log .= "<br />>>>>>> " . __( 'Skipping Featured Image', 'woocommerce-product-importer' );
	}

	// Product Gallery
	if( $product->product_gallery !== null ) {
		if( $import->advanced_log )
			$import->log .= "<br />>>>>>> " . sprintf( __( 'Setting Product Gallery: %s', 'woocommerce-product-importer' ), $product->product_gallery );
		else
			$import->log .= "<br />>>>>>> " . __( 'Setting Product Gallery', 'woocommerce-product-importer' );
		foreach( $product->product_galleries as $key => $image ) {
			if( is_numeric( $image ) !== false ) {
				// Check if WordPress Media ID exists
				$post_type = 'attachment';
				$args = array(
					'post_type' => $post_type,
					'post_mime_type' => 'image',
					'posts_per_page' => 1,
					'fields' => 'ids'
				);
				$query = new WP_Query( $args );
				$size = $query->found_posts;
				if( empty( $size ) ) {
					$import->log .= "<br />>>>>>>>>> " . sprintf( __( 'Skipped WordPress Media ID: %s - No Image could be found by that WordPress Media ID', 'woocommerce-product-importer' ), $image );
					unset( $product->product_galleries[$key] );
				}
				unset( $query, $size );
			} else {
				// Import new image
				switch( $import->image_method ) {

					// Image import method: External (e.g. http://www.domain.com/product_images/product-image-1.jpg)
					case 'external':
						ini_set( 'user_agent', $_SERVER['HTTP_USER_AGENT'] );
						$image_filename = '';
						if( woo_pd_http_file_exists( $image ) ) {
							$image_filename = woo_pd_download_product_image( $image );
						} else {
							$import->log .= "<br />>>>>>>>>> " . sprintf( __( 'Skipped Image download: %s - No Image could be found at that URL', 'woocommerce-product-importer' ), $image );
							unset( $product->product_galleries[$key] );
						}
						if( $image_filename ) {
							if( strstr( $image_filename, '/' ) )
								$image_id = woo_pd_create_product_image( $image_filename, true );
							else
								$image_id = woo_pd_create_product_image( $image_filename );
							// Assign URL to custom Post meta for later comparison when merging
							if( WOO_PD_DEBUG !== true )
								update_post_meta( $image_id, '_original_url', $image );
							$product->product_galleries[$key] = $image_id;
						}
						unset( $image, $image_filename );
						break;

					// Image import method: Manual upload (e.g. image-1.jpg)
					case 'csv':
						$image_filename = '';
						if( file_exists( $import->uploads_path . $image ) ) {
							$image_filename = $image;
						} else {
							$import->log .= "<br />>>>>>>>>> " . sprintf( __( 'Skipped Image download: %s - No Image could be found matching that filename', 'woocommerce-product-importer' ), $import->uploads_path . $image );
						}
						if( $image_filename ) {
							if( $image_id = woo_pd_create_product_image( $image ) ) {
								// Assign URL to custom Post meta for later comparison when merging
								if( WOO_PD_DEBUG !== true )
									update_post_meta( $image_id, '_original_url', $image );
								$product->product_galleries[$key] = $image_id;
							}
						}
						unset( $image, $image_filename );
						break;

				}
			}
		}
		if( !empty( $product->product_galleries ) ) {
			if( WOO_PD_DEBUG !== true )
				update_post_meta( $product->ID, '_product_image_gallery', implode( ',', $product->product_galleries ) );
		}
	} else if( $import->advanced_log ) {
		$import->log .= "<br />>>>>>> " . __( 'Skipping Product Gallery', 'woocommerce-product-importer' );
	}

	// Allow Plugin/Theme authors to add support for additional Product details
	$product = apply_filters( 'woo_pd_create_product_addons', $product, $import );
	$import = apply_filters( 'woo_pd_create_product_log_addons', $import, $product );

	// Custom Product meta
	$custom_products = woo_pd_get_option( 'custom_products', '' );
	if( !empty( $custom_products ) ) {
		foreach( $custom_products as $custom_product ) {
			if( !empty( $custom_product ) ) {
				if( $product->{sanitize_key( $custom_product )} !== null ) {
					if( WOO_PD_DEBUG !== true )
						update_post_meta( $product->ID, $custom_product, $product->{sanitize_key( $custom_product )} );
					if( $import->advanced_log )
						$import->log .= "<br />>>>>>> " . sprintf( __( 'Setting custom Product meta: %s: %s', 'woocommerce-product-importer' ), $custom_product, $product->{sanitize_key( $custom_product )} );
					else
						$import->log .= "<br />>>>>>> " . sprintf( __( 'Setting custom Product meta: %s', 'woocommerce-product-importer' ), $custom_product );
				} else if( $import->advanced_log ) {
					$import->log .= "<br />>>>>>> " . sprintf( __( 'Skipping custom Product meta: %s', 'woocommerce-product-importer' ), $custom_product );
				}
			}
		}
	}
	unset( $custom_products, $custom_product );

	// Custom Attributes
	// @mod - put import logic here

/*
	// Sample Product Meta
	if( $product->sample !== null ) {
		if( WOO_PD_DEBUG !== true )
			update_post_meta( $product->ID, '_sample', $product->sample );
		if( $import->advanced_log )
			$import->log .= "<br />>>>>>> " . sprintf( __( 'Setting Sample Product Meta: %s', 'woocommerce-product-importer' ), $product->sample );
		else
			$import->log .= "<br />>>>>>> " . __( 'Setting Sample Product Meta', 'woocommerce-product-importer' );
	} else if( $import->advanced_log ) {
		$import->log .= "<br />>>>>>> " . __( 'Skipping Sample Product Meta', 'woocommerce-product-importer' );
	}
*/

	// WooCommerce Brands Addon - http://woothemes.com/woocommerce/
	// WooCommerce Brands - http://proword.net/Woocommerce_Brands/
	if( woo_pd_detect_product_brands() ) {
		$term_taxonomy = apply_filters( 'woo_pd_brand_term_taxonomy', 'product_brand' );
		if( $product->brand !== null && !empty( $product->brand_term_id ) ) {
			if( $product->post_type <> 'product_variation' ) {
				$term_taxonomy_ids = wp_set_object_terms( $product->ID, array_unique( array_map( 'intval', $product->brand_term_id ) ), $term_taxonomy );
				if( $import->advanced_log ) {
					if( count( $product->brand_term_id ) == 1 )
						$import->log .= "<br />>>>>>> " . sprintf( __( 'Linking Brand: %s', 'woocommerce-product-importer' ), $product->brand );
					else
						$import->log .= "<br />>>>>>> " . sprintf( __( 'Linking Brands: %s', 'woocommerce-product-importer' ), $product->brand );
				} else {
					if( count( $product->brand_term_id ) == 1 )
						$import->log .= "<br />>>>>>> " . __( 'Linking Brand', 'woocommerce-product-importer' );
					else
						$import->log .= "<br />>>>>>> " . __( 'Linking Brands', 'woocommerce-product-importer' );
				}
			} else {
				if( $import->advanced_log )
					$import->log .= "<br />>>>>>> " . __( 'Skipping Brand, cannot set Brand for Variations', 'woocommerce-product-importer' );
			}
		} else if( $import->advanced_log ) {
			$import->log .= "<br />>>>>>> " . __( 'Skipping Brand', 'woocommerce-product-importer' );
		}
	}

	// Advanced Google Product Feed
	if( woo_pd_detect_import_plugin( 'gpf' ) ) {
		if( $product->gpf_availability !== null || $product->gpf_condition !== null || $product->gpf_brand !== null || $product->gpf_product_type !== null || $product->gpf_google_product_category !== null || $product->gpf_gtin !== null || $product->gpf_mpn !== null || $product->gpf_gender !== null || $product->gpf_age_group !== null || $product->gpf_color !== null || $product->gpf_size !== null ) {
			if( $product->gpf_availability !== null ) {
				if( $import->advanced_log )
					$import->log .= "<br />>>>>>> " . sprintf( __( 'Setting Advanced Google Product Feed - Availability: %s', 'woocommerce-product-importer' ), $product->gpf_availability );
				else
					$import->log .= "<br />>>>>>> " . __( 'Setting Advanced Google Product Feed - Availability', 'woocommerce-product-importer' );
			} else {
				$product->gpf_availability = '';
			}
			if( $product->gpf_condition !== null ) {
				if( $import->advanced_log )
					$import->log .= "<br />>>>>>> " . sprintf( __( 'Setting Advanced Google Product Feed - Condition: %s', 'woocommerce-product-importer' ), $product->gpf_condition );
				else
					$import->log .= "<br />>>>>>> " . __( 'Setting Advanced Google Product Feed - Condition', 'woocommerce-product-importer' );
			} else {
				$product->gpf_condition = '';
			}
			if( $product->gpf_brand !== null ) {
				if( $import->advanced_log )
					$import->log .= "<br />>>>>>> " . sprintf( __( 'Setting Advanced Google Product Feed - Brand: %s', 'woocommerce-product-importer' ), $product->gpf_brand );
				else
					$import->log .= "<br />>>>>>> " . __( 'Setting Advanced Google Product Feed - Brand', 'woocommerce-product-importer' );
			} else {
				$product->gpf_brand = '';
			}
			if( $product->gpf_product_type !== null ) {
				if( $import->advanced_log )
					$import->log .= "<br />>>>>>> " . sprintf( __( 'Setting Advanced Google Product Feed - Product Type: %s', 'woocommerce-product-importer' ), $product->gpf_product_type );
				else
					$import->log .= "<br />>>>>>> " . __( 'Setting Advanced Google Product Feed - Product Type', 'woocommerce-product-importer' );
			} else {
				$product->gpf_product_type = '';
			}
			if( $product->gpf_google_product_category !== null ) {
				if( $import->advanced_log )
					$import->log .= "<br />>>>>>> " . sprintf( __( 'Setting Advanced Google Product Feed - Google Product Category: %s', 'woocommerce-product-importer' ), $product->gpf_google_product_category );
				else
					$import->log .= "<br />>>>>>> " . __( 'Setting Advanced Google Product Feed - Google Product Category', 'woocommerce-product-importer' );
			} else {
				$product->gpf_google_product_category = '';
			}
			if( $product->gpf_gtin !== null ) {
				if( $import->advanced_log )
					$import->log .= "<br />>>>>>> " . sprintf( __( 'Setting Advanced Google Product Feed - Global Trade Item Number (GTIN): %s', 'woocommerce-product-importer' ), $product->gpf_gtin );
				else
					$import->log .= "<br />>>>>>> " . __( 'Setting Advanced Google Product Feed - Global Trade Item Number (GTIN)', 'woocommerce-product-importer' );
			} else {
				$product->gpf_gtin = '';
			}
			if( $product->gpf_mpn !== null ) {
				if( $import->advanced_log )
					$import->log .= "<br />>>>>>> " . sprintf( __( 'Setting Advanced Google Product Feed - Manufacturer Part Number (MPN): %s', 'woocommerce-product-importer' ), $product->gpf_mpn );
				else
					$import->log .= "<br />>>>>>> " . __( 'Setting Advanced Google Product Feed - Manufacturer Part Number (MPN)', 'woocommerce-product-importer' );
			} else {
				$product->gpf_mpn = '';
			}
			if( $product->gpf_gender !== null ) {
				if( $import->advanced_log )
					$import->log .= "<br />>>>>>> " . sprintf( __( 'Setting Advanced Google Product Feed - Gender: %s', 'woocommerce-product-importer' ), $product->gpf_gender );
				else
					$import->log .= "<br />>>>>>> " . __( 'Setting Advanced Google Product Feed - Gender', 'woocommerce-product-importer' );
			} else {
				$product->gpf_gender = '';
			}
			if( $product->gpf_age_group !== null ) {
				if( $import->advanced_log )
					$import->log .= "<br />>>>>>> " . sprintf( __( 'Setting Advanced Google Product Feed - Age Group: %s', 'woocommerce-product-importer' ), $product->gpf_age_group );
				else
					$import->log .= "<br />>>>>>> " . __( 'Setting Advanced Google Product Feed - Age Group', 'woocommerce-product-importer' );
			} else {
				$product->gpf_age_group = '';
			}
			if( $product->gpf_color !== null ) {
				if( $import->advanced_log )
					$import->log .= "<br />>>>>>> " . sprintf( __( 'Setting Advanced Google Product Feed - Colour: %s', 'woocommerce-product-importer' ), $product->gpf_color );
				else
					$import->log .= "<br />>>>>>> " . __( 'Setting Advanced Google Product Feed - Colour', 'woocommerce-product-importer' );
			} else {
				$product->gpf_color = '';
			}
			if( $product->gpf_size !== null ) {
				if( $import->advanced_log )
					$import->log .= "<br />>>>>>> " . sprintf( __( 'Setting Advanced Google Product Feed - Size: %s', 'woocommerce-product-importer' ), $product->gpf_size );
				else
					$import->log .= "<br />>>>>>> " . __( 'Setting Advanced Google Product Feed - Size', 'woocommerce-product-importer' );
			} else {
				$product->gpf_size = '';
			}
			$product->gpf_data = array(
				'availability' => $product->gpf_availability,
				'condition' => $product->gpf_condition,
				'brand' => $product->gpf_brand,
				'product_type' => $product->gpf_product_type,
				'google_product_category' => $product->gpf_google_product_category,
				'gtin' => $product->gpf_gtin,
				'mpn' => $product->gpf_mpn,
				'gender' => $product->gpf_gender,
				'age_group' => $product->gpf_age_group,
				'color' => $product->gpf_color,
				'size' => $product->gpf_size
			);
			if( WOO_PD_DEBUG !== true )
				update_post_meta( $product->ID, '_woocommerce_gpf_data', $product->gpf_data );
			$import->log .= "<br />>>>>>> " . __( 'Linking Advanced Google Product Feed details', 'woocommerce-product-importer' );
		} else if( $import->advanced_log ) {
			$import->log .= "<br />>>>>>> " . __( 'Skipping Advanced Google Product Feed', 'woocommerce-product-importer' );
		}
	}

	// All in One SEO Pack - http://wordpress.org/extend/plugins/all-in-one-seo-pack/
	if( woo_pd_detect_import_plugin( 'aioseop' ) ) {
		if( $product->aioseop_keywords !== null || $product->aioseop_description !== null || $product->aioseop_title !== null || $product->aioseop_titleatr !== null || $product->aioseop_menulabel !== null ) {
			$import->log .= "<br />>>>>>> " . __( 'Linking All in One SEO Pack meta details', 'woocommerce-product-importer' );
			if( $product->aioseop_keywords !== null ) {
				if( WOO_PD_DEBUG !== true )
					update_post_meta( $product->ID, '_aioseop_keywords', $product->aioseop_keywords );
				if( $import->advanced_log )
					$import->log .= "<br />>>>>>>>>> " . sprintf( __( 'Keywords: %s', 'woocommerce-product-importer' ), $product->aioseop_keywords );
				else
					$import->log .= "<br />>>>>>>>>> " . __( 'Setting Keywords', 'woocommerce-product-importer' );
			} else if( $import->advanced_log ) {
				$import->log .= "<br />>>>>>>>>> " . __( 'Skipping Keywords', 'woocommerce-product-importer' );
			}
			if( $product->aioseop_description !== null ) {
				if( WOO_PD_DEBUG !== true )
					update_post_meta( $product->ID, '_aioseop_description', $product->aioseop_description );
				if( $import->advanced_log )
					$import->log .= "<br />>>>>>>>>> " . sprintf( __( 'Description: %s', 'woocommerce-product-importer' ), $product->aioseop_description );
				else
					$import->log .= "<br />>>>>>>>>> " . __( 'Setting Description', 'woocommerce-product-importer' );
			} else if( $import->advanced_log ) {
				$import->log .= "<br />>>>>>>>>> " . __( 'Skipping Description', 'woocommerce-product-importer' );
			}
			if( $product->aioseop_title !== null ) {
				if( WOO_PD_DEBUG !== true )
					update_post_meta( $product->ID, '_aioseop_title', $product->aioseop_title );
				if( $import->advanced_log )
					$import->log .= "<br />>>>>>>>>> " . sprintf( __( 'Title: %s', 'woocommerce-product-importer' ), $product->aioseop_title );
				else
					$import->log .= "<br />>>>>>>>>> " . __( 'Setting Title', 'woocommerce-product-importer' );
			} else if( $import->advanced_log ) {
				$import->log .= "<br />>>>>>>>>> " . __( 'Skipping Title', 'woocommerce-product-importer' );
			}
			if( $product->aioseop_titleatr !== null ) {
				if( WOO_PD_DEBUG !== true )
					update_post_meta( $product->ID, '_aioseop_titleatr', $product->aioseop_titleatr );
				if( $import->advanced_log )
					$import->log .= "<br />>>>>>>>>> " . sprintf( __( 'Title Attributes: %s', 'woocommerce-product-importer' ), $product->aioseop_titleatr );
				else
					$import->log .= "<br />>>>>>>>>> " . __( 'Setting Title Attributes', 'woocommerce-product-importer' );
			} else if( $import->advanced_log ) {
				$import->log .= "<br />>>>>>>>>> " . __( 'Skipping Title Attributes', 'woocommerce-product-importer' );
			}
			if( $product->aioseop_menulabel !== null ) {
				if( WOO_PD_DEBUG !== true )
					update_post_meta( $product->ID, '_aioseop_menulabel', $product->aioseop_menulabel );
				if( $import->advanced_log )
					$import->log .= "<br />>>>>>>>>> " . sprintf( __( 'Menu Label: %s', 'woocommerce-product-importer' ), $product->aioseop_menulabel );
				else
					$import->log .= "<br />>>>>>>>>> " . __( 'Setting Menu Label', 'woocommerce-product-importer' );
			} else if( $import->advanced_log ) {
				$import->log .= "<br />>>>>>>>>> " . __( 'Skipping Menu Label', 'woocommerce-product-importer' );
			}
		} else if( $import->advanced_log ) {
			$import->log .= "<br />>>>>>> " . __( 'Skipping All in One SEO Pack', 'woocommerce-product-importer' );
		}
	}

	// Ultimate SEO
	if( woo_pd_detect_import_plugin( 'ultimate_seo' ) ) {
		if( $product->useo_meta_title !== null || $product->useo_meta_description !== null || $product->useo_meta_keywords !== null || $product->useo_social_title !== null || $product->useo_social_description !== null || $product->useo_meta_noindex !== null || $product->useo_meta_noautolinks !== null ) {
			if( $product->useo_meta_title !== null ) {
				if( $import->advanced_log )
					$import->log .= "<br />>>>>>> " . sprintf( __( 'Setting Ultimate SEO - Title Tag: %s', 'woocommerce-product-importer' ), $product->useo_meta_title );
				else
					$import->log .= "<br />>>>>>> " . __( 'Setting Ultimate SEO - Title Tag', 'woocommerce-product-importer' );
				if( WOO_PD_DEBUG !== true )
					update_post_meta( $product->ID, '_su_title', $product->useo_meta_title );
			}
			if( $product->useo_meta_description !== null ) {
				if( $import->advanced_log )
					$import->log .= "<br />>>>>>> " . sprintf( __( 'Setting Ultimate SEO - Meta Description: %s', 'woocommerce-product-importer' ), $product->useo_meta_description );
				else
					$import->log .= "<br />>>>>>> " . __( 'Setting Ultimate SEO - Meta Description', 'woocommerce-product-importer' );
				if( WOO_PD_DEBUG !== true )
					update_post_meta( $product->ID, '_su_description', $product->useo_meta_description );
			}
			if( $product->useo_meta_keywords !== null ) {
				if( $import->advanced_log )
					$import->log .= "<br />>>>>>> " . sprintf( __( 'Setting Ultimate SEO - Meta Keywords: %s', 'woocommerce-product-importer' ), $product->useo_meta_keywords );
				else
					$import->log .= "<br />>>>>>> " . __( 'Setting Ultimate SEO - Meta Keywords', 'woocommerce-product-importer' );
				if( WOO_PD_DEBUG !== true )
					update_post_meta( $product->ID, '_su_keywords', $product->useo_meta_keywords );
			}
			if( $product->useo_social_title !== null ) {
				if( $import->advanced_log )
					$import->log .= "<br />>>>>>> " . sprintf( __( 'Setting Ultimate SEO - Social Title: %s', 'woocommerce-product-importer' ), $product->useo_social_title );
				else
					$import->log .= "<br />>>>>>> " . __( 'Setting Ultimate SEO - Social Title', 'woocommerce-product-importer' );
				if( WOO_PD_DEBUG !== true )
					update_post_meta( $product->ID, '_su_og_title', $product->useo_social_title );
			}
			if( $product->useo_social_description !== null ) {
				if( $import->advanced_log )
					$import->log .= "<br />>>>>>> " . sprintf( __( 'Setting Ultimate SEO - Social Description: %s', 'woocommerce-product-importer' ), $product->useo_social_description );
				else
					$import->log .= "<br />>>>>>> " . __( 'Setting Ultimate SEO - Social Description', 'woocommerce-product-importer' );
				if( WOO_PD_DEBUG !== true )
					update_post_meta( $product->ID, '_su_og_description', $product->useo_social_description );
			}
			if( $product->useo_meta_noindex !== null ) {
				if( $import->advanced_log )
					$import->log .= "<br />>>>>>> " . sprintf( __( 'Setting Ultimate SEO - NoIndex: %s', 'woocommerce-product-importer' ), $product->useo_meta_noindex );
				else
					$import->log .= "<br />>>>>>> " . __( 'Setting Ultimate SEO - NoIndex', 'woocommerce-product-importer' );
				if( WOO_PD_DEBUG !== true )
					update_post_meta( $product->ID, '_su_meta_robots_noindex', $product->useo_meta_noindex );
			}
			if( $product->useo_meta_noautolinks !== null ) {
				if( $import->advanced_log )
					$import->log .= "<br />>>>>>> " . sprintf( __( 'Setting Ultimate SEO - Disable Autolinks: %s', 'woocommerce-product-importer' ), $product->useo_meta_noautolinks );
				else
					$import->log .= "<br />>>>>>> " . __( 'Setting Ultimate SEO - Disable Autolinks', 'woocommerce-product-importer' );
				if( WOO_PD_DEBUG !== true )
					update_post_meta( $product->ID, '_su_disable_autolinks', $product->useo_meta_noautolinks );
			}
		} else if( $import->advanced_log ) {
			$import->log .= "<br />>>>>>> " . __( 'Skipping Ultimate SEO', 'woocommerce-product-importer' );
		}
	}

	// WordPress SEO - http://yoast.com/wordpress/seo/#utm_source=wpadmin&utm_medium=plugin&utm_campaign=wpseoplugin
	if( woo_pd_detect_import_plugin( 'wpseo' ) ) {
		if( $product->wpseo_focuskw !== null || $product->wpseo_metadesc !== null || $product->wpseo_title !== null || $product->wpseo_googleplus_description !== null || $product->wpseo_opengraph_description !== null || $product->wpseo_meta_robots_noindex !== null || $product->wpseo_meta_robots_nofollow !== null || $product->wpseo_meta_robots_adv !== null || $product->wpseo_sitemap_html_include !== null || $product->wpseo_authorship !== null || $product->wpseo_canonical !== null || $product->wpseo_redirect !== null ) {
			if( $product->wpseo_focuskw !== null ) {
				if( $import->advanced_log )
					$import->log .= "<br />>>>>>> " . sprintf( __( 'Setting WordPress SEO - Focus Keyword: %s', 'woocommerce-product-importer' ), $product->wpseo_focuskw );
				else
					$import->log .= "<br />>>>>>> " . __( 'Setting WordPress SEO - Focus Keyword', 'woocommerce-product-importer' );
				if( WOO_PD_DEBUG !== true )
					update_post_meta( $product->ID, '_yoast_wpseo_focuskw', $product->wpseo_focuskw );
			}
			if( $product->wpseo_metadesc !== null ) {
				if( $import->advanced_log )
					$import->log .= "<br />>>>>>> " . sprintf( __( 'Setting WordPress SEO - Meta Description: %s', 'woocommerce-product-importer' ), $product->wpseo_metadesc );
				else
					$import->log .= "<br />>>>>>> " . __( 'Setting WordPress SEO - Meta Description', 'woocommerce-product-importer' );
				if( WOO_PD_DEBUG !== true )
					update_post_meta( $product->ID, '_yoast_wpseo_metadesc', $product->wpseo_metadesc );
			}
			if( $product->wpseo_title !== null ) {
				if( $import->advanced_log )
					$import->log .= "<br />>>>>>> " . sprintf( __( 'Setting WordPress SEO - SEO Title: %s', 'woocommerce-product-importer' ), $product->wpseo_title );
				else
					$import->log .= "<br />>>>>>> " . __( 'Setting WordPress SEO - SEO Title', 'woocommerce-product-importer' );
				if( WOO_PD_DEBUG !== true )
					update_post_meta( $product->ID, '_yoast_wpseo_title', $product->wpseo_title );
			}
			if( $product->wpseo_googleplus_description !== null ) {
				if( $import->advanced_log )
					$import->log .= "<br />>>>>>> " . sprintf( __( 'Setting WordPress SEO - Google+ Description: %s', 'woocommerce-product-importer' ), $product->wpseo_googleplus_description );
				else
					$import->log .= "<br />>>>>>> " . __( 'Setting WordPress SEO - Google+ Description', 'woocommerce-product-importer' );
				if( WOO_PD_DEBUG !== true )
					update_post_meta( $product->ID, '_yoast_wpseo_google-plus-description', $product->wpseo_googleplus_description );
			}
			if( $product->wpseo_opengraph_description !== null ) {
				if( $import->advanced_log )
					$import->log .= "<br />>>>>>> " . sprintf( __( 'Setting WordPress SEO - Facebook+ Description: %s', 'woocommerce-product-importer' ), $product->wpseo_opengraph_description );
				else
					$import->log .= "<br />>>>>>> " . __( 'Setting WordPress SEO - Facebook Description', 'woocommerce-product-importer' );
				if( WOO_PD_DEBUG !== true )
					update_post_meta( $product->ID, '_yoast_wpseo_opengraph-description', $product->wpseo_opengraph_description );
			}
			if( $product->wpseo_meta_robots_noindex !== null ) {
				if( $import->advanced_log )
					$import->log .= "<br />>>>>>> " . sprintf( __( 'Setting WordPress SEO - Meta Robots Index: %s', 'woocommerce-product-importer' ), $product->wpseo_meta_robots_noindex );
				else
					$import->log .= "<br />>>>>>> " . __( 'Setting WordPress SEO - Meta Robots Index', 'woocommerce-product-importer' );
				if( WOO_PD_DEBUG !== true )
					update_post_meta( $product->ID, '_yoast_wpseo_meta-robots-noindex', $product->wpseo_meta_robots_noindex );
			}
			if( $product->wpseo_meta_robots_nofollow !== null ) {
				if( $import->advanced_log )
					$import->log .= "<br />>>>>>> " . sprintf( __( 'Setting WordPress SEO - Meta Robots Follow: %s', 'woocommerce-product-importer' ), $product->wpseo_meta_robots_nofollow );
				else
					$import->log .= "<br />>>>>>> " . __( 'Setting WordPress SEO - Meta Robots Follow', 'woocommerce-product-importer' );
				if( WOO_PD_DEBUG !== true )
					update_post_meta( $product->ID, '_yoast_wpseo_meta-robots-nofollow', $product->wpseo_meta_robots_nofollow );
			}
			if( $product->wpseo_meta_robots_advs !== null ) {
				if( $import->advanced_log )
					$import->log .= "<br />>>>>>> " . sprintf( __( 'Setting WordPress SEO - Meta Robots Advanced: %s', 'woocommerce-product-importer' ), $product->wpseo_meta_robots_adv );
				else
					$import->log .= "<br />>>>>>> " . __( 'Setting WordPress SEO - Meta Robots Advanced', 'woocommerce-product-importer' );
				if( WOO_PD_DEBUG !== true )
					update_post_meta( $product->ID, '_yoast_wpseo_meta-robots-adv', implode( ',', $product->wpseo_meta_robots_advs ) );
			}
			if( $product->wpseo_sitemap_html_include !== null ) {
				if( $import->advanced_log )
					$import->log .= "<br />>>>>>> " . sprintf( __( 'Setting WordPress SEO - Include in HTML Sitemap: %s', 'woocommerce-product-importer' ), $product->wpseo_sitemap_html_include );
				else
					$import->log .= "<br />>>>>>> " . __( 'Setting WordPress SEO - Include in HTML Sitemap', 'woocommerce-product-importer' );
				if( WOO_PD_DEBUG !== true )
					update_post_meta( $product->ID, '_yoast_wpseo_sitemap-html-include', $product->wpseo_sitemap_html_include );
			}
			if( $product->wpseo_authorship !== null ) {
				if( $import->advanced_log )
					$import->log .= "<br />>>>>>> " . sprintf( __( 'Setting WordPress SEO - Authorship: %s', 'woocommerce-product-importer' ), $product->wpseo_authorship );
				else
					$import->log .= "<br />>>>>>> " . __( 'Setting WordPress SEO - Authorship', 'woocommerce-product-importer' );
				if( WOO_PD_DEBUG !== true )
					update_post_meta( $product->ID, '_yoast_wpseo_authorship', $product->wpseo_authorship );
			}
			if( $product->wpseo_canonical !== null ) {
				if( $import->advanced_log )
					$import->log .= "<br />>>>>>> " . sprintf( __( 'Setting WordPress SEO - Canonical URL: %s', 'woocommerce-product-importer' ), $product->wpseo_canonical );
				else
					$import->log .= "<br />>>>>>> " . __( 'Setting WordPress SEO - Canonical URL', 'woocommerce-product-importer' );
				if( WOO_PD_DEBUG !== true )
					update_post_meta( $product->ID, '_yoast_wpseo_canonical', $product->wpseo_canonical );
			}
			if( $product->wpseo_redirect !== null ) {
				if( $import->advanced_log )
					$import->log .= "<br />>>>>>> " . sprintf( __( 'Setting WordPress SEO - 301 Redirect: %s', 'woocommerce-product-importer' ), $product->wpseo_redirect );
				else
					$import->log .= "<br />>>>>>> " . __( 'Setting WordPress SEO - 301 Redirect', 'woocommerce-product-importer' );
				if( WOO_PD_DEBUG !== true )
					update_post_meta( $product->ID, '_yoast_wpseo_redirect', $product->wpseo_redirect );
			}
		} else if( $import->advanced_log ) {
			$import->log .= "<br />>>>>>> " . __( 'Skipping WordPress SEO', 'woocommerce-product-importer' );
		}
	}

	// Cost of Goods - http://www.skyverge.com/product/woocommerce-cost-of-goods-tracking/
	if( woo_pd_detect_import_plugin( 'wc_cog' ) ) {
		if( $product->cost_of_goods !== null ) {
			if( $import->advanced_log )
				$import->log .= "<br />>>>>>> " . sprintf( __( 'Setting Cost of Goods: %s', 'woocommerce-product-importer' ), $product->cost_of_goods );
			else
				$import->log .= "<br />>>>>>> " . __( 'Setting Cost of Goods', 'woocommerce-product-importer' );
			if( WOO_PD_DEBUG !== true )
				update_post_meta( $product->ID, '_wc_cog_cost', $product->cost_of_goods );
		} else if( $import->advanced_log ) {
			$import->log .= "<br />>>>>>> " . __( 'Skipping Cost of Goods', 'woocommerce-product-importer' );
		}
	}

	// WooCommerce Profit of Sales Report - http://codecanyon.net/item/woocommerce-profit-of-sales-report/9190590
	if( woo_pd_detect_import_plugin( 'wc_posr' ) ) {
		if( $product->cost_of_goods !== null ) {
			if( $import->advanced_log )
				$import->log .= "<br />>>>>>> " . sprintf( __( 'Setting Cost of Goods: %s', 'woocommerce-product-importer' ), $product->cost_of_goods );
			else
				$import->log .= "<br />>>>>>> " . __( 'Setting Cost of Goods', 'woocommerce-product-importer' );
			if( WOO_PD_DEBUG !== true )
				update_post_meta( $product->ID, '_posr_cost_of_good', $product->cost_of_goods );
		} else if( $import->advanced_log ) {
			$import->log .= "<br />>>>>>> " . __( 'Skipping Cost of Goods', 'woocommerce-product-importer' );
		}
	}

	// MSRP Pricing - http://www.woothemes.com/products/msrp-pricing/
	if( woo_pd_detect_import_plugin( 'wc_msrp' ) ) {
		if( $product->msrp !== null ) {
			if( $import->advanced_log )
				$import->log .= "<br />>>>>>> " . sprintf( __( 'Setting MSRP Pricing: %s', 'woocommerce-product-importer' ), $product->msrp );
			else
				$import->log .= "<br />>>>>>> " . __( 'Setting MSRP Pricing', 'woocommerce-product-importer' );
			if( WOO_PD_DEBUG !== true )
				update_post_meta( $product->ID, '_msrp_price', $product->msrp );
		} else if( $import->advanced_log ) {
			$import->log .= "<br />>>>>>> " . __( 'Skipping MSRP Pricing', 'woocommerce-product-importer' );
		}
	}

	// Product Vendors - http://www.woothemes.com/products/product-vendors/
	if( woo_pd_detect_import_plugin( 'vendors' ) ) {
		$term_taxonomy = 'shop_vendor';
		if( $product->vendor !== null && !empty( $product->vendor_term_id ) ) {
			if( $product->post_type <> 'product_variation' ) {
				$term_taxonomy_ids = wp_set_object_terms( $product->ID, array_unique( array_map( 'intval', $product->vendor_term_id ) ), $term_taxonomy );
				if( $import->advanced_log ) {
					if( count( $product->vendor_term_id ) == 1 )
						$import->log .= "<br />>>>>>> " . sprintf( __( 'Linking Product Vendor: %s', 'woocommerce-product-importer' ), $product->product_vendor );
					else
						$import->log .= "<br />>>>>>> " . sprintf( __( 'Linking Product Vendors: %s', 'woocommerce-product-importer' ), $product->product_vendor );
				} else {
					if( count( $product->vendor_term_id ) == 1 )
						$import->log .= "<br />>>>>>> " . __( 'Linking Product Vendor', 'woocommerce-product-importer' );
					else
						$import->log .= "<br />>>>>>> " . __( 'Linking Product Vendors', 'woocommerce-product-importer' );
				}
			} else {
				if( $import->advanced_log )
					$import->log .= "<br />>>>>>> " . __( 'Skipping Product Vendor, cannot set Product Vendor for Variations', 'woocommerce-product-importer' );
			}
		} else if( $import->advanced_log ) {
			$import->log .= "<br />>>>>>> " . __( 'Skipping Product Vendor', 'woocommerce-product-importer' );
		}
	}

	// WooCommerce Custom Fields - http://www.rightpress.net/woocommerce-custom-fields
	if( woo_pd_detect_import_plugin( 'wc_customfields' ) ) {
		if( !get_option( 'wccf_migrated_to_20' ) ) {
			// Legacy WooCommerce Custom Fields was stored in a single Option
			$options = get_option( 'rp_wccf_options' );
			if( !empty( $options ) ) {
				$custom_fields = ( isset( $options[1]['product_admin_fb_config'] ) ? $options[1]['product_admin_fb_config'] : false );
				if( !empty( $custom_fields ) ) {
					$product_custom_fields = get_post_meta( $product_id, '_wccf_product_admin', true );
					foreach( $custom_fields as $custom_field ) {
						if( $product->{sprintf( 'wccf_%s', sanitize_key( $custom_field['key'] ) )} !== null ) {
							if( !empty( $product_custom_fields ) ) {
								foreach( $product_custom_fields as $key => $product_custom_field ) {

									if( sanitize_key( $product_custom_field['key'] ) <> sanitize_key( $custom_field['key'] ) )
										continue;

									if( $import->advanced_log )
										$import->log .= "<br />>>>>>> " . sprintf( __( 'Setting Product Properties: %s: %s', 'woocommerce-product-importer' ), ucfirst( $custom_field['label'] ), $product->{sprintf( 'wccf_%s', sanitize_key( $custom_field['key'] ) )} );
									else
										$import->log .= "<br />>>>>>> " . sprintf( __( 'Setting Product Properties: %s', 'woocommerce-product-importer' ), ucfirst( $custom_field['label'] ) );
									$product_custom_fields[$key]['value'] = $product->{sprintf( 'wccf_%s', sanitize_key( $custom_field['key'] ) )};
								}
							}
						} else if( $import->advanced_log ) {
							$import->log .= "<br />>>>>>> " . sprintf( __( 'Skipping Product Properties: %s', 'woocommerce-product-importer' ), ucfirst( $custom_field['label'] ) );
						}
					}
					if( WOO_PD_DEBUG !== true )
						update_post_meta( $product_id, '_wccf_product_admin', $product_custom_fields );
					unset( $product_custom_fields );
				}
			}
			unset( $options );
		} else {
			// WooCommerce Custom Fields uses CPT for Product properties
			$custom_fields = woo_pd_get_wccf_product_properties();
			if( !empty( $custom_fields ) ) {
				foreach( $custom_fields as $custom_field ) {
					$label = get_post_meta( $custom_field->ID, 'label', true );
					$key = get_post_meta( $custom_field->ID, 'key', true );
					if( $product->{sprintf( 'wccf_pp_%s', sanitize_key( $key ) )} !== null ) {
						if( $import->advanced_log )
							$import->log .= "<br />>>>>>> " . sprintf( __( 'Setting Product Properties: %s: %s', 'woocommerce-product-importer' ), ucfirst( $label ), $product->{sprintf( 'wccf_pp_%s', sanitize_key( $key ) )} );
						else
							$import->log .= "<br />>>>>>> " . sprintf( __( 'Setting Product Properties: %s', 'woocommerce-product-importer' ), ucfirst( $label ) );
						if( WOO_PD_DEBUG !== true ) {
							update_post_meta( $product->ID, sprintf( '_wccf_pp_data_%s', sanitize_key( $key ) ), array() );
							update_post_meta( $product->ID, sprintf( '_wccf_pp_id_%s', sanitize_key( $key ) ), $custom_field->ID );
							update_post_meta( $product->ID, sprintf( '_wccf_pp_%s', sanitize_key( $key ) ), $product->{sprintf( 'wccf_pp_%s', sanitize_key( $key ) )} );
						}
					} else if( $import->advanced_log ) {
						$import->log .= "<br />>>>>>> " . sprintf( __( 'Skipping Product Properties: %s', 'woocommerce-product-importer' ), ucfirst( $label ) );
					}
				}
			}
			unset( $label, $key );
		}
		unset( $custom_fields, $custom_field );
	}

	// Barcodes for WooCommerce - http://www.wolkenkraft.com/produkte/barcodes-fuer-woocommerce/
	if( woo_pd_detect_import_plugin( 'wc_barcodes' ) ) {
		if( $product->barcode_type !== null ) {
			if( $import->advanced_log )
				$import->log .= "<br />>>>>>> " . sprintf( __( 'Setting Barcode Type: %s', 'woocommerce-product-importer' ), $product->barcode_type );
			else
				$import->log .= "<br />>>>>>> " . __( 'Setting Barcode Type', 'woocommerce-product-importer' );
			if( WOO_PD_DEBUG !== true )
				update_post_meta( $product->ID, '_barcode_type', $product->barcode_type );
		} else if( $import->advanced_log ) {
			$import->log .= "<br />>>>>>> " . __( 'Skipping Barcode Type', 'woocommerce-product-importer' );
		}
		if( $product->barcode !== null ) {
			if( $import->advanced_log )
				$import->log .= "<br />>>>>>> " . sprintf( __( 'Setting Barcode: %s', 'woocommerce-product-importer' ), $product->barcode );
			else
				$import->log .= "<br />>>>>>> " . __( 'Setting Barcode', 'woocommerce-product-importer' );
			if( WOO_PD_DEBUG !== true )
				update_post_meta( $product->ID, '_barcode', $product->barcode );
		} else if( $import->advanced_log ) {
			$import->log .= "<br />>>>>>> " . __( 'Skipping Barcode', 'woocommerce-product-importer' );
		}
	}

	// WooCommerce Subscriptions - http://www.woothemes.com/products/woocommerce-subscriptions/
	if( woo_pd_detect_import_plugin( 'subscriptions' ) ) {
		if( $product->subscription_price !== null ) {
			if( $import->advanced_log )
				$import->log .= "<br />>>>>>> " . sprintf( __( 'Setting Subscription Price: %s', 'woocommerce-product-importer' ), $product->subscription_price );
			else
				$import->log .= "<br />>>>>>> " . __( 'Setting Subscription Price', 'woocommerce-product-importer' );
			if( WOO_PD_DEBUG !== true )
				update_post_meta( $product->ID, '_subscription_price', $product->subscription_price );
		} else if( $import->advanced_log ) {
			$import->log .= "<br />>>>>>> " . __( 'Skipping Subscription Price', 'woocommerce-product-importer' );
		}
		if( $product->subscription_period_interval !== null ) {
			if( $import->advanced_log )
				$import->log .= "<br />>>>>>> " . sprintf( __( 'Setting Subscription Period Interval: %s', 'woocommerce-product-importer' ), $product->subscription_period_interval );
			else
				$import->log .= "<br />>>>>>> " . __( 'Setting Subscription Period Interval', 'woocommerce-product-importer' );
			if( WOO_PD_DEBUG !== true )
				update_post_meta( $product->ID, '_subscription_period_interval', $product->subscription_period_interval );
		} else if( $import->advanced_log ) {
			$import->log .= "<br />>>>>>> " . __( 'Skipping Subscription Period Interval', 'woocommerce-product-importer' );
		}
		if( $product->subscription_period !== null ) {
			if( $import->advanced_log )
				$import->log .= "<br />>>>>>> " . sprintf( __( 'Setting Subscription Period: %s', 'woocommerce-product-importer' ), $product->subscription_period );
			else
				$import->log .= "<br />>>>>>> " . __( 'Setting Subscription Period', 'woocommerce-product-importer' );
			if( WOO_PD_DEBUG !== true )
				update_post_meta( $product->ID, '_subscription_period', $product->subscription_period );
		} else if( $import->advanced_log ) {
			$import->log .= "<br />>>>>>> " . __( 'Skipping Subscription Period', 'woocommerce-product-importer' );
		}
		if( $product->subscription_length !== null ) {
			if( $import->advanced_log )
				$import->log .= "<br />>>>>>> " . sprintf( __( 'Setting Subscription Length: %s', 'woocommerce-product-importer' ), $product->subscription_length );
			else
				$import->log .= "<br />>>>>>> " . __( 'Setting Subscription Length', 'woocommerce-product-importer' );
			if( WOO_PD_DEBUG !== true )
				update_post_meta( $product->ID, '_subscription_length', $product->subscription_length );
		} else if( $import->advanced_log ) {
			$import->log .= "<br />>>>>>> " . __( 'Skipping Subscription Length', 'woocommerce-product-importer' );
		}
		if( $product->subscription_sign_up_fee !== null ) {
			if( $import->advanced_log )
				$import->log .= "<br />>>>>>> " . sprintf( __( 'Setting Subscription Sign-up Fee: %s', 'woocommerce-product-importer' ), $product->subscription_sign_up_fee );
			else
				$import->log .= "<br />>>>>>> " . __( 'Setting Subscription Sign-up Fee', 'woocommerce-product-importer' );
			if( WOO_PD_DEBUG !== true )
				update_post_meta( $product->ID, '_subscription_sign_up_fee', $product->subscription_sign_up_fee );
		} else if( $import->advanced_log ) {
			$import->log .= "<br />>>>>>> " . __( 'Skipping Subscription Sign-up Fee', 'woocommerce-product-importer' );
		}
		if( $product->subscription_trial_length !== null ) {
			if( $import->advanced_log )
				$import->log .= "<br />>>>>>> " . sprintf( __( 'Setting Subscription Trial Length: %s', 'woocommerce-product-importer' ), $product->subscription_trial_length );
			else
				$import->log .= "<br />>>>>>> " . __( 'Setting Subscription Trial Length', 'woocommerce-product-importer' );
			if( WOO_PD_DEBUG !== true )
				update_post_meta( $product->ID, '_subscription_trial_length', $product->subscription_trial_length );
		} else if( $import->advanced_log ) {
			$import->log .= "<br />>>>>>> " . __( 'Skipping Subscription Trial Length', 'woocommerce-product-importer' );
		}
		if( $product->subscription_trial_period !== null ) {
			if( $import->advanced_log )
				$import->log .= "<br />>>>>>> " . sprintf( __( 'Setting Subscription Trial Period: %s', 'woocommerce-product-importer' ), $product->subscription_trial_period );
			else
				$import->log .= "<br />>>>>>> " . __( 'Setting Subscription Trial Period', 'woocommerce-product-importer' );
			if( WOO_PD_DEBUG !== true )
				update_post_meta( $product->ID, '_subscription_trial_period', $product->subscription_trial_period );
		} else if( $import->advanced_log ) {
			$import->log .= "<br />>>>>>> " . __( 'Skipping Subscription Trial Period', 'woocommerce-product-importer' );
		}
		if( $product->subscription_limit !== null ) {
			if( $import->advanced_log )
				$import->log .= "<br />>>>>>> " . sprintf( __( 'Setting Subscription Limit: %s', 'woocommerce-product-importer' ), $product->subscription_limit );
			else
				$import->log .= "<br />>>>>>> " . __( 'Setting Subscription Limit', 'woocommerce-product-importer' );
			if( WOO_PD_DEBUG !== true )
				update_post_meta( $product->ID, '_subscription_limit', $product->subscription_limit );
		} else if( $import->advanced_log ) {
			$import->log .= "<br />>>>>>> " . __( 'Skipping Subscription Limit', 'woocommerce-product-importer' );
		}
	}

	// WooCommerce Ultimate Multi Currency Suite - https://codecanyon.net/item/woocommerce-ultimate-multi-currency-suite/11997014
	if( woo_pd_detect_import_plugin( 'wc_umcs' ) ) {
		$currencies = json_decode( get_option( 'wcumcs_available_currencies' ) );
		if( !empty( $currencies ) ) {
			$current_currency = ( function_exists( 'get_woocommerce_currency' ) ? get_woocommerce_currency() : false );
			if( $import->advanced_log )
				$import->log .= "<br />>>> " . sprintf( __( 'WooCommerce base currency: %s', 'woocommerce-product-importer' ), $current_currency );
			foreach( $currencies as $currency_code => $currency_data ) {
				// Skip the base currency
				if( $currency_code == $current_currency ) {
					if( $product->{sprintf( 'wcumcs_regular_price_%s', sanitize_key( $currency_code ) )} !== null ) {
						if( $import->advanced_log )
							$import->log .= "<br />>>> " . sprintf( __( 'Skipping Regular Price (%s) as it is the base currency', 'woocommerce-product-importer' ), $currency_code );
					}
					if( $product->{sprintf( 'wcumcs_regular_price_%s', sanitize_key( $currency_code ) )} !== null ) {
						if( $import->advanced_log )
							$import->log .= "<br />>>> " . sprintf( __( 'Skipping Sale Price (%s) as it is the base currency', 'woocommerce-product-importer' ), $currency_code );
					}
					continue;
				}
				if( $product->{sprintf( 'wcumcs_regular_price_%s', sanitize_key( $currency_code ) )} !== null ) {
					if( $import->advanced_log )
						$import->log .= "<br />>>>>>> " . sprintf( __( 'Setting Regular Price (%s): %s', 'woocommerce-product-importer' ), $currency_code, $product->{sprintf( 'wcumcs_regular_price_%s', sanitize_key( $currency_code ) )} );
					else
						$import->log .= "<br />>>>>>> " . sprintf( __( 'Setting Regular Price (%s)', 'woocommerce-product-importer' ), $currency_code );
					if( WOO_PD_DEBUG !== true )
						update_post_meta( $product->ID, sprintf( '_wcumcs_regular_price_%s', $currency_code ), $product->{sprintf( 'wcumcs_regular_price_%s', sanitize_key( $currency_code ) )} );
				} else if( $import->advanced_log ) {
					$import->log .= "<br />>>>>>> " . sprintf( __( 'Skipping Regular Price (%s)', 'woocommerce-product-importer' ), $currency_code );
				}
				if( $product->{sprintf( 'wcumcs_sale_price_%s', sanitize_key( $currency_code ) )} !== null ) {
					if( $import->advanced_log )
						$import->log .= "<br />>>>>>> " . sprintf( __( 'Setting Sale Price (%s): %s', 'woocommerce-product-importer' ), $currency_code, $product->{sprintf( 'wcumcs_sale_price_%s', sanitize_key( $currency_code ) )} );
					else
						$import->log .= "<br />>>>>>> " . sprintf( __( 'Setting Sale Price (%s)', 'woocommerce-product-importer' ), $currency_code );
					if( WOO_PD_DEBUG !== true )
						update_post_meta( $product->ID, sprintf( '_wcumcs_sale_price_%s', $currency_code ), $product->{sprintf( 'wcumcs_sale_price_%s', sanitize_key( $currency_code ) )} );
				} else if( $import->advanced_log ) {
					$import->log .= "<br />>>>>>> " . sprintf( __( 'Skipping Sale Price (%s)', 'woocommerce-product-importer' ), $currency_code );
				}
			}
		}
	}

	// WooCommerce Wholesale Prices - https://wordpress.org/plugins/woocommerce-wholesale-prices/
	if( woo_pd_detect_import_plugin( 'wc_wholesale_prices' ) ) {
		$wholesale_roles = woo_pd_get_wholesale_prices_roles();
		if( !empty( $wholesale_roles ) ) {
			foreach( $wholesale_roles as $key => $wholesale_role ) {
				if( $product->{sprintf( '%s_wholesale_price', $key )} !== null ) {
					if( WOO_PD_DEBUG !== true )
						update_post_meta( $product->ID, sprintf( '%s_wholesale_price', $key ), $product->{sprintf( '%s_wholesale_price', $key )} );
					if( $import->advanced_log )
						$import->log .= "<br />>>>>>> " . sprintf( __( 'Setting Wholesale Price (%s)', 'woocommerce-product-importer' ), $wholesale_role['roleName'], $product->{sprintf( '%s_wholesale_price', $key )} );
					else
						$import->log .= "<br />>>>>>> " . sprintf( __( 'Setting Wholesale Price (%s)', 'woocommerce-product-importer' ), $wholesale_role['roleName'] );
				} else if( $import->advanced_log ) {
					$import->log .= "<br />>>>>>> " . sprintf( __( 'Skipping Wholesale Price (%s)', 'woocommerce-product-importer' ), $wholesale_role['roleName'] );
				}
			}
		}
		unset( $wholesale_roles, $wholesale_role, $key );
	}

	// WooCommerce Multilingual - https://wordpress.org/plugins/woocommerce-multilingual/
	if( woo_pd_detect_wpml() || woo_pd_detect_import_plugin( 'wpml_wc' ) ) {

		global $sitepress;

		if( $product->language !== null ) {
			// Skip this for the primary Language
			if( $product->language <> $sitepress->get_default_language() ) {
				if( $import->advanced_log )
					$import->log .= "<br />>>>>>> " . sprintf( __( 'Setting Language: %s', 'woocommerce-product-importer' ), $product->language );
				else
					$import->log .= "<br />>>>>>> " . __( 'Setting Language', 'woocommerce-product-importer' );
				if( WOO_PD_DEBUG !== true ) {
					if( function_exists( 'wpml_get_content_trid' ) ) {
						if( isset( $product->sku ) && !$product->duplicate_exists ) {
							$duplicate_exists = woo_pd_get_product_id_by_sku( $product->sku );
							if( $duplicate_exists )
								$trid = wpml_get_content_trid( sprintf( 'post_%s', 'product' ), $duplicate_exists );
						}
					}
					$sitepress->set_element_language_details( $product->ID, sprintf( 'post_%s', 'product' ), $trid, $product->language );
				}
			} else {
				if( $import->advanced_log )
					$import->log .= "<br />>>>>>> " . __( 'Skipping Language, cannot set Language for default Language', 'woocommerce-product-importer' );
				else
					$import->log .= "<br />>>>>>> " . __( 'Skipping Language', 'woocommerce-product-importer' );
			}
		} else if( $import->advanced_log ) {
			$import->log .= "<br />>>>>>> " . __( 'Skipping Language', 'woocommerce-product-importer' );
		}
	}

	// WooCommerce Bookings - http://www.woothemes.com/products/woocommerce-bookings/
	if( woo_pd_detect_import_plugin( 'woocommerce_bookings' ) ) {
		if( $product->booking_has_persons !== null ) {
			if( $import->advanced_log )
				$import->log .= "<br />>>>>>> " . sprintf( __( 'Setting Booking Has Persons: %s', 'woocommerce-product-importer' ), $product->booking_has_persons );
			else
				$import->log .= "<br />>>>>>> " . __( 'Setting Booking Has Persons', 'woocommerce-product-importer' );
			if( WOO_PD_DEBUG !== true )
				update_post_meta( $product->ID, '_wc_booking_has_persons', $product->booking_has_persons );
		} else if( $import->advanced_log ) {
			$import->log .= "<br />>>>>>> " . __( 'Skipping Booking Has Persons', 'woocommerce-product-importer' );
		}
		if( $product->booking_has_resources !== null ) {
			if( $import->advanced_log )
				$import->log .= "<br />>>>>>> " . sprintf( __( 'Setting Booking Has Resources: %s', 'woocommerce-product-importer' ), $product->booking_has_resources );
			else
				$import->log .= "<br />>>>>>> " . __( 'Setting Booking Has Resources', 'woocommerce-product-importer' );
			if( WOO_PD_DEBUG !== true )
				update_post_meta( $product->ID, '_wc_booking_has_resources', $product->booking_has_resources );
		} else if( $import->advanced_log ) {
			$import->log .= "<br />>>>>>> " . __( 'Skipping Booking Has Resources', 'woocommerce-product-importer' );
		}
		if( $product->booking_base_cost !== null ) {
			if( $import->advanced_log )
				$import->log .= "<br />>>>>>> " . sprintf( __( 'Setting Booking Base Cost: %s', 'woocommerce-product-importer' ), $product->booking_base_cost );
			else
				$import->log .= "<br />>>>>>> " . __( 'Setting Booking Base Cost', 'woocommerce-product-importer' );
			if( WOO_PD_DEBUG !== true )
				update_post_meta( $product->ID, '_wc_booking_cost', $product->booking_base_cost );
		} else if( $import->advanced_log ) {
			$import->log .= "<br />>>>>>> " . __( 'Skipping Booking Base Cost', 'woocommerce-product-importer' );
		}
		if( $product->booking_block_cost !== null ) {
			if( $import->advanced_log )
				$import->log .= "<br />>>>>>> " . sprintf( __( 'Setting Booking Block Cost: %s', 'woocommerce-product-importer' ), $product->booking_block_cost );
			else
				$import->log .= "<br />>>>>>> " . __( 'Setting Booking Block Cost', 'woocommerce-product-importer' );
			if( WOO_PD_DEBUG !== true )
				update_post_meta( $product->ID, '_wc_booking_base_cost', $product->booking_block_cost );
		} else if( $import->advanced_log ) {
			$import->log .= "<br />>>>>>> " . __( 'Skipping Booking Block Cost', 'woocommerce-product-importer' );
		}
		if( $product->booking_display_cost !== null ) {
			if( $import->advanced_log )
				$import->log .= "<br />>>>>>> " . sprintf( __( 'Setting Booking Display Cost: %s', 'woocommerce-product-importer' ), $product->booking_display_cost );
			else
				$import->log .= "<br />>>>>>> " . __( 'Setting Booking Display Cost', 'woocommerce-product-importer' );
			if( WOO_PD_DEBUG !== true )
				update_post_meta( $product->ID, '_wc_display_cost', $product->booking_display_cost );
		} else if( $import->advanced_log ) {
			$import->log .= "<br />>>>>>> " . __( 'Skipping Booking Display Cost', 'woocommerce-product-importer' );
		}
		if( $product->booking_requires_confirmation !== null ) {
			if( $import->advanced_log )
				$import->log .= "<br />>>>>>> " . sprintf( __( 'Setting Booking Requires Confirmation: %s', 'woocommerce-product-importer' ), $product->booking_requires_confirmation );
			else
				$import->log .= "<br />>>>>>> " . __( 'Setting Booking Requires Confirmation', 'woocommerce-product-importer' );
			if( WOO_PD_DEBUG !== true )
				update_post_meta( $product->ID, '_wc_booking_requires_confirmation', $product->booking_requires_confirmation );
		} else if( $import->advanced_log ) {
			$import->log .= "<br />>>>>>> " . __( 'Skipping Booking Requires Confirmation', 'woocommerce-product-importer' );
		}
		if( $product->booking_user_can_cancel !== null ) {
			if( $import->advanced_log )
				$import->log .= "<br />>>>>>> " . sprintf( __( 'Setting Booking Can Be Cancelled: %s', 'woocommerce-product-importer' ), $product->booking_user_can_cancel );
			else
				$import->log .= "<br />>>>>>> " . __( 'Setting Booking Can Be Cancelled', 'woocommerce-product-importer' );
			if( WOO_PD_DEBUG !== true )
				update_post_meta( $product->ID, '_wc_booking_user_can_cancel', $product->booking_user_can_cancel );
		} else if( $import->advanced_log ) {
			$import->log .= "<br />>>>>>> " . __( 'Skipping Booking Can Be Cancelled', 'woocommerce-product-importer' );
		}

	}

	// WooCommerce Show Single Variations - https://iconicwp.com/products/woocommerce-show-single-variations/
	if( woo_pd_detect_import_plugin( 'wc_show_single_variations' ) ) {
		if( $product->disable_add_to_cart !== null || $product->show_search_results !== null || $product->show_filtered_results !== null || $product->show_catalog !== null ) {
			if( $product->post_type == 'product_variation' ) {
				if( $product->disable_add_to_cart !== null ) {
					if( $import->advanced_log )
						$import->log .= "<br />>>>>>> " . sprintf( __( 'Setting WooCommerce Show Single Variations - Disable Add to Cart: %s', 'woocommerce-product-importer' ), $product->disable_add_to_cart );
					else
						$import->log .= "<br />>>>>>> " . __( 'Setting WooCommerce Show Single Variations - Disable Add to Cart', 'woocommerce-product-importer' );
					if( WOO_PD_DEBUG !== true )
						update_post_meta( $product->ID, '_disable_add_to_cart', $product->disable_add_to_cart );
				} else if( $import->advanced_log ) {
					$import->log .= "<br />>>>>>> " . __( 'Skipping WooCommerce Show Single Variations - Disable Add to Cart', 'woocommerce-product-importer' );
				}
				if( $product->show_search_results !== null || $product->show_filtered_results !== null || $product->show_catalog !== null ) {

					// Let's store a bunch of values within a single Post meta array...
					$visibility_data = array();

					if( $product->show_search_results !== null ) {
						if( $product->show_search_results ) {
							$visibility_data[] = 'search';
							if( $import->advanced_log )
								$import->log .= "<br />>>>>>> " . sprintf( __( 'Setting WooCommerce Show Single Variations - Show in Search Results: %s', 'woocommerce-product-importer' ), $product->show_search_results );
							else
								$import->log .= "<br />>>>>>> " . __( 'Setting WooCommerce Show Single Variations - Show in Search Results', 'woocommerce-product-importer' );
						}
					} else if( $import->advanced_log ) {
						$import->log .= "<br />>>>>>> " . __( 'Skipping WooCommerce Show Single Variations - Show in Search Results', 'woocommerce-product-importer' );
					}
					if( $product->show_filtered_results !== null ) {
						if( $product->show_filtered_results ) {
							$visibility_data[] = 'filtered';
							if( $import->advanced_log )
								$import->log .= "<br />>>>>>> " . sprintf( __( 'Setting WooCommerce Show Single Variations - Show in Filtered Results: %s', 'woocommerce-product-importer' ), $product->show_filtered_results );
							else
								$import->log .= "<br />>>>>>> " . __( 'Setting WooCommerce Show Single Variations - Show in Filtered Results', 'woocommerce-product-importer' );
						}
					} else if( $import->advanced_log ) {
						$import->log .= "<br />>>>>>> " . __( 'Skipping WooCommerce Show Single Variations - Show in Filtered Results', 'woocommerce-product-importer' );
					}
					if( $product->show_catalog !== null ) {
						if( $product->show_catalog ) {
							$visibility_data[] = 'catalog';
							if( $import->advanced_log )
								$import->log .= "<br />>>>>>> " . sprintf( __( 'Setting WooCommerce Show Single Variations - Show in Catalog: %s', 'woocommerce-product-importer' ), $product->show_catalog );
							else
								$import->log .= "<br />>>>>>> " . __( 'Setting WooCommerce Show Single Variations - Show in Catalog', 'woocommerce-product-importer' );
						}
					} else if( $import->advanced_log ) {
						$import->log .= "<br />>>>>>> " . __( 'Skipping WooCommerce Show Single Variations - Show in Catalog', 'woocommerce-product-importer' );
					}
					if( WOO_PD_DEBUG !== true )
						update_post_meta( $product->ID, '_visibility', $visibility_data );
					unset( $visibility_data );
				}
				$import->log .= "<br />>>>>>> " . __( 'Linking WooCommerce Show Single Variations details', 'woocommerce-product-importer' );
			} else {
				if( $import->advanced_log )
					$import->log .= "<br />>>>>>> " . __( 'Skipping WooCommerce Show Single Variations, cannot set WooCommerce Show Single Variations details for non-Variations', 'woocommerce-product-importer' );
			}
		} else {
			$import->log .= "<br />>>>>>> " . __( 'Skipping WooCommerce Show Single Variations', 'woocommerce-product-importer' );
		}
	}

	// WooCommerce Tab Manager - http://www.woothemes.com/products/woocommerce-tab-manager/
	if( woo_pd_detect_import_plugin( 'wc_tabmanager' ) ) {
		$product_tabs = woo_pd_get_product_tabs();
		if( !empty( $product_tabs ) ) {
			foreach( $product_tabs as $product_tab ) {
				if( !empty( $product_tab ) ) {
/*
					if( $product->sprintf( 'product_tab_%s', sanitize_key( $product_tab->post_name ) ) !== null ) {
						// @mod - Add support for importing Product Tabs in 4.0
						// Do something
					} else if( $import->advanced_log ) {
						$import->log .= "<br />>>>>>> " . sprintf( __( 'Skipping %s', 'woocommerce-product-importer' ), sprintf( __( 'Product Tab: %s', 'woocommerce-product-importer' ), $product_tab->post_title ) );
					}
*/
				}
			}
		}
		unset( $product_tabs, $product_tab );
	}

}
?>